@@ -740,6 +740,22 @@ bool CSG_String::asInt(int &Value) const
 	return( false );
 }
 
+//---------------------------------------------------------
+sLong CSG_String::asLongLong(void) const
+{
+	sLong Value; return( asLongLong(Value) ? Value : 0 );
+}
+
+bool CSG_String::asLongLong(sLong &Value) const
+{
+	if( m_pString->ToLongLong(&Value) )
+	{
+		return( true );
+	}
+
+	return( false );
+}
+
 //---------------------------------------------------------
 double CSG_String::asDouble(void) const
 {
@@ -770,16 +786,16 @@ bool CSG_String::asDouble(double &Value) const
 //---------------------------------------------------------
 CSG_String CSG_String::from_UTF8(const char *String, size_t Length)
 {
-	CSG_String	s;
+	CSG_String s;
 
 	if( String )
 	{
 		if( !Length )
 		{
-			Length	= strlen(String);
+			Length = strlen(String);
 		}
 
-		*s.m_pString	= wxString::FromUTF8(String, Length);
+		*s.m_pString = wxString::FromUTF8(String, Length);
 	}
 
 	return( s );
@@ -796,19 +812,16 @@ CSG_String CSG_String::from_UTF8(const char *String, size_t Length)
 //---------------------------------------------------------
 size_t CSG_String::to_UTF8(char **pString) const
 {
-	if( !is_Empty() )
+	CSG_Buffer String(to_UTF8());
+
+	if( String.Get_Size() > 1 && (*pString = (char *)SG_Malloc(String.Get_Size())) != NULL )
 	{
-		const wxScopedCharBuffer	Buffer	= m_pString->utf8_str();
+		memcpy(*pString, String.Get_Data(), String.Get_Size());
 
-		if( (*pString = (char *)SG_Malloc(Buffer.length())) != NULL )
-		{
-			memcpy(*pString, Buffer.data(), Buffer.length());
-
-			return( Buffer.length() );
-		}
+		return( String.Get_Size() - 1 );
 	}
 
-	*pString	= NULL;
+	*pString = NULL;
 
 	return( 0 );
 }
@@ -816,14 +829,11 @@ size_t CSG_String::to_UTF8(char **pString) const
 //---------------------------------------------------------
 CSG_Buffer CSG_String::to_UTF8(void) const
 {
-	CSG_Buffer	String;
+	CSG_Buffer String;
 
-	if( !is_Empty() )
-	{
-		const wxScopedCharBuffer	Buffer	= m_pString->utf8_str();
+	const wxScopedCharBuffer Buffer = m_pString->utf8_str();
 
-		String.Set_Data(Buffer.data(), Buffer.length() + 1);
-	}
+	String.Set_Data(Buffer.data(), Buffer.length() + 1);
 
 	return( String );
 }
@@ -844,16 +854,16 @@ CSG_Buffer CSG_String::to_UTF8(void) const
 //---------------------------------------------------------
 size_t CSG_String::to_MBChar(char **pString, int Encoding) const
 {
-	CSG_Buffer	String(to_MBChar(Encoding));
+	CSG_Buffer String(to_MBChar(Encoding));
 
-	if( String.Get_Size() && (*pString = (char *)SG_Malloc(String.Get_Size())) != NULL )
+	if( String.Get_Size() > 1 && (*pString = (char *)SG_Malloc(String.Get_Size())) != NULL )
 	{
 		memcpy(*pString, String.Get_Data(), String.Get_Size());
 
-		return( String.Get_Size() );
+		return( String.Get_Size() - 1 );
 	}
 
-	*pString	= NULL;
+	*pString = NULL;
 
 	return( 0 );
 }
@@ -861,27 +871,24 @@ size_t CSG_String::to_MBChar(char **pString, int Encoding) const
 //---------------------------------------------------------
 CSG_Buffer CSG_String::to_MBChar(int Encoding) const
 {
-	CSG_Buffer	String;
+	CSG_Buffer String;
 
-	if( !is_Empty() )
+	wxScopedCharBuffer Buffer;
+
+	switch( Encoding )	// selecting the appropriate wxMBConv class
 	{
-		wxScopedCharBuffer	Buffer;
-
-		switch( Encoding )	// selecting the appropriate wxMBConv class
-		{
-		case SG_FILE_ENCODING_ANSI   : Buffer = m_pString->mb_str(wxConvLibc       ); break;
-		case SG_FILE_ENCODING_UTF7   : Buffer = m_pString->mb_str(wxConvUTF7       ); break;
-		case SG_FILE_ENCODING_UTF8   : Buffer = m_pString->mb_str(wxConvUTF8       ); break;
-		case SG_FILE_ENCODING_UTF16LE: Buffer = m_pString->mb_str(wxMBConvUTF16LE()); break;
-		case SG_FILE_ENCODING_UTF16BE: Buffer = m_pString->mb_str(wxMBConvUTF16BE()); break;
-		case SG_FILE_ENCODING_UTF32LE: Buffer = m_pString->mb_str(wxMBConvUTF32LE()); break;
-		case SG_FILE_ENCODING_UTF32BE: Buffer = m_pString->mb_str(wxMBConvUTF32BE()); break;
-		default                      : Buffer = m_pString->mb_str(wxConvAuto     ()); break;
-		}
-
-		String.Set_Data(Buffer.data(), Buffer.length());
+	case SG_FILE_ENCODING_ANSI   : Buffer = m_pString->mb_str(wxConvLibc       ); break;
+	case SG_FILE_ENCODING_UTF7   : Buffer = m_pString->mb_str(wxConvUTF7       ); break;
+	case SG_FILE_ENCODING_UTF8   : Buffer = m_pString->mb_str(wxConvUTF8       ); break;
+	case SG_FILE_ENCODING_UTF16LE: Buffer = m_pString->mb_str(wxMBConvUTF16LE()); break;
+	case SG_FILE_ENCODING_UTF16BE: Buffer = m_pString->mb_str(wxMBConvUTF16BE()); break;
+	case SG_FILE_ENCODING_UTF32LE: Buffer = m_pString->mb_str(wxMBConvUTF32LE()); break;
+	case SG_FILE_ENCODING_UTF32BE: Buffer = m_pString->mb_str(wxMBConvUTF32BE()); break;
+	default                      : Buffer = m_pString->mb_str(wxConvAuto     ()); break;
 	}
 
+	String.Set_Data(Buffer.data(), Buffer.length() + 1);
+
 	return( String );
 }
 
@@ -901,44 +908,37 @@ CSG_Buffer CSG_String::to_MBChar(int Encoding) const
 //---------------------------------------------------------
 bool CSG_String::to_ASCII(char **pString, char Replace)	const
 {
-	if( !is_Empty() )
+	CSG_Buffer String(to_ASCII());
+
+	if( String.Get_Size() > 1 && (*pString = (char *)SG_Malloc(String.Get_Size())) != NULL )
 	{
-		#if wxCHECK_VERSION(3, 1, 0)
-			const wxScopedCharBuffer	Buffer	= m_pString->ToAscii(Replace);
-		#else
-			const wxScopedCharBuffer	Buffer	= m_pString->ToAscii();
-		#endif
+		memcpy(*pString, String.Get_Data(), String.Get_Size());
 
-		if( (*pString = (char *)SG_Malloc(Buffer.length())) != NULL )
-		{
-			memcpy(*pString, Buffer.data(), Buffer.length());
-
-			return( true );
-		}
+		return( true );
 	}
 
+	*pString = NULL;
+
 	return( false );
 }
 
 //---------------------------------------------------------
 CSG_Buffer CSG_String::to_ASCII(char Replace) const
 {
-	CSG_Buffer	String;
+	CSG_Buffer String;
 
-	if( !is_Empty() )
-	{
-		#if wxCHECK_VERSION(3, 1, 0)
-			const wxScopedCharBuffer	Buffer	= m_pString->ToAscii(Replace);
-		#else
-			const wxScopedCharBuffer	Buffer	= m_pString->ToAscii();
-		#endif
+	#if wxCHECK_VERSION(3, 1, 0)
+		const wxScopedCharBuffer Buffer = m_pString->ToAscii(Replace);
+	#else
+		const wxScopedCharBuffer Buffer = m_pString->ToAscii();
+	#endif
 
-		String.Set_Data(Buffer.data(), Buffer.length());
-	}
+	String.Set_Data(Buffer.data(), Buffer.length() + 1);
 
 	return( String );
 }
 
+
 ///////////////////////////////////////////////////////////
 //														 //
 ///////////////////////////////////////////////////////////
@@ -1108,10 +1108,10 @@ public:
 
 	CSG_Index_Compare_Strings(CSG_String **Values, bool Ascending) : m_Values(Values), m_Ascending(Ascending) {}
 
-	virtual int			Compare		(const int _a, const int _b)
+	virtual int			Compare		(const sLong _a, const sLong _b)
 	{
-		int	a	= m_Ascending ? _a : _b;
-		int	b	= m_Ascending ? _b : _a;
+		sLong a	= m_Ascending ? _a : _b;
+		sLong b	= m_Ascending ? _b : _a;
 
 		return( m_Values[a]->Cmp(*m_Values[b]) );
 	}
@@ -1125,15 +1125,15 @@ bool CSG_Strings::Sort(bool Ascending)
 		return( true );
 	}
 
-	CSG_Index_Compare_Strings	Compare((CSG_String **)m_Strings.Get_Array(), Ascending);
+	CSG_Index_Compare_Strings Compare((CSG_String **)m_Strings.Get_Array(), Ascending);
 
-	CSG_Index	Index(Get_Count(), Compare);
+	CSG_Index Index(Get_Count(), Compare);
 
-	CSG_Array_Pointer	Strings(m_Strings);
+	CSG_Array_Pointer Strings(m_Strings);
 
 	for(size_t i=0; i<Get_Size(); i++)
 	{
-		m_Strings[i]	= Strings[Index[i]];
+		m_Strings[i] = Strings[Index[i]];
 	}
 
 	return( true );
