@@ -351,7 +351,7 @@ bool CSG_Simple_Statistics::Create(bool bHoldValues)
 {
 	Invalidate();
 
-	m_Values.Create(bHoldValues ? sizeof(double) : 0, 0, SG_ARRAY_GROWTH_1);
+	m_Values.Create(bHoldValues ? sizeof(double) : 0, 0, TSG_Array_Growth::SG_ARRAY_GROWTH_1);
 
 	return( true );
 }
@@ -409,7 +409,7 @@ bool CSG_Simple_Statistics::Create(const CSG_Vector &Values, bool bHoldValues)
 {
 	if( Create(bHoldValues) )
 	{
-		for(size_t i=0; i<Values.Get_Size(); i++)
+		for(sLong i=0; i<Values.Get_Size(); i++)
 		{
 			Add_Value(Values[i]);
 		}
@@ -581,7 +581,7 @@ void CSG_Simple_Statistics::_Evaluate(int Level)
 
 		if( Get_StdDev() > 0. && m_Values.Get_Size() > 0 )
 		{
-			for(int i=0; i<Get_Count(); i++)
+			for(sLong i=0; i<Get_Count(); i++)
 			{
 				double	d	= (Get_Value(i) - Get_Mean()) / Get_StdDev();
 
@@ -691,7 +691,7 @@ double CSG_Simple_Statistics::Get_Gini(void)
 
 		m_Gini	= 0.;
 
-		for(int i=0; i<Get_Count(); i++)
+		for(sLong i=0; i<Get_Count(); i++)
 		{
 			m_Gini	+= (i + 1.) * Get_Value(i);
 		}
@@ -714,7 +714,7 @@ sLong CSG_Simple_Statistics::Get_IndexOfMinimum(void)
 	size_t	Index	= 0;
 	double	Value	= Get_Values()[Index];
 
-	for(size_t i=1; i<m_Values.Get_Size(); i++)
+	for(size_t i=1; i<(size_t)m_Values.Get_Size(); i++)
 	{
 		if( Value > Get_Values()[i] )
 		{
@@ -738,7 +738,7 @@ sLong CSG_Simple_Statistics::Get_IndexOfMaximum(void)
 	size_t	Index	= 0;
 	double	Value	= Get_Values()[Index];
 
-	for(size_t i=1; i<m_Values.Get_Size(); i++)
+	for(size_t i=1; i<(size_t)m_Values.Get_Size(); i++)
 	{
 		if( Value < Get_Values()[i] )
 		{
@@ -1017,25 +1017,25 @@ TSG_Data_Type CSG_Category_Statistics::Get_Category_Type(void)	const
 //---------------------------------------------------------
 int CSG_Category_Statistics::Get_Category(int Value)	const
 {
-	CSG_Table_Record	*pRecord	= m_pTable->Find_Record(0, Value, m_pTable->Get_Count() > 10);
+	CSG_Table_Record *pRecord = m_pTable->Find_Record(0, Value, m_pTable->Get_Count() > 10);
 
-	return( pRecord ? pRecord->Get_Index() : -1);
+	return( pRecord ? (int)pRecord->Get_Index() : -1);
 }
 
 //---------------------------------------------------------
 int CSG_Category_Statistics::Get_Category(double Value)	const
 {
-	CSG_Table_Record	*pRecord	= m_pTable->Find_Record(0, Value, m_pTable->Get_Count() > 10);
+	CSG_Table_Record *pRecord = m_pTable->Find_Record(0, Value, m_pTable->Get_Count() > 10);
 
-	return( pRecord ? pRecord->Get_Index() : -1);
+	return( pRecord ? (int)pRecord->Get_Index() : -1);
 }
 
 //---------------------------------------------------------
 int CSG_Category_Statistics::Get_Category(const CSG_String &Value)	const
 {
-	CSG_Table_Record	*pRecord	= m_pTable->Find_Record(0, Value, m_pTable->Get_Count() > 10);
+	CSG_Table_Record *pRecord = m_pTable->Find_Record(0, Value, m_pTable->Get_Count() > 10);
 
-	return( pRecord ? pRecord->Get_Index() : -1);
+	return( pRecord ? (int)pRecord->Get_Index() : -1);
 }
 
 
@@ -1046,13 +1046,13 @@ int CSG_Category_Statistics::Get_Category(const CSG_String &Value)	const
 //---------------------------------------------------------
 int CSG_Category_Statistics::Add_Value(int Value)
 {
-	int		i	= Get_Category(Value);
+	int i = Get_Category(Value);
 
-	CSG_Table_Record	*pRecord	= m_pTable->Get_Record_byIndex(i);
+	CSG_Table_Record *pRecord = m_pTable->Get_Record_byIndex(i);
 
 	if( !pRecord )
 	{
-		i	= m_pTable->Get_Count();
+		i = (int)m_pTable->Get_Count();
 
 		(pRecord = m_pTable->Add_Record())->Set_Value(0, Value);
 	}
@@ -1065,13 +1065,13 @@ int CSG_Category_Statistics::Add_Value(int Value)
 //---------------------------------------------------------
 int CSG_Category_Statistics::Add_Value(double Value)
 {
-	int		i	= Get_Category(Value);
+	int i = Get_Category(Value);
 
-	CSG_Table_Record	*pRecord	= m_pTable->Get_Record_byIndex(i);
+	CSG_Table_Record *pRecord = m_pTable->Get_Record_byIndex(i);
 
 	if( !pRecord )
 	{
-		i	= m_pTable->Get_Count();
+		i = (int)m_pTable->Get_Count();
 
 		(pRecord = m_pTable->Add_Record())->Set_Value(0, Value);
 	}
@@ -1084,13 +1084,13 @@ int CSG_Category_Statistics::Add_Value(double Value)
 //---------------------------------------------------------
 int CSG_Category_Statistics::Add_Value(const CSG_String &Value)
 {
-	int		i	= Get_Category(Value);
+	int i = Get_Category(Value);
 
-	CSG_Table_Record	*pRecord	= m_pTable->Get_Record_byIndex(i);
+	CSG_Table_Record *pRecord = m_pTable->Get_Record_byIndex(i);
 
 	if( !pRecord )
 	{
-		i	= m_pTable->Get_Count();
+		i = (int)m_pTable->Get_Count();
 
 		(pRecord = m_pTable->Add_Record())->Set_Value(0, Value);
 	}
@@ -1116,7 +1116,7 @@ bool CSG_Category_Statistics::Sort(void)
 // returns the number of categories.
 int CSG_Category_Statistics::Get_Count(void)	const
 {
-	return( m_pTable->Get_Count() );
+	return( (int)m_pTable->Get_Count() );
 }
 
 //---------------------------------------------------------
@@ -1589,7 +1589,7 @@ bool CSG_Histogram::Create(size_t nClasses, double Minimum, double Maximum, CSG_
 	}
 
 	//-----------------------------------------------------
-	for(int i=0; i<pTable->Get_Count(); i++)
+	for(sLong i=0; i<pTable->Get_Count(); i++)
 	{
 		double	Value	= pTable->Get_Record(i)->asDouble(Field);
 
@@ -1754,7 +1754,7 @@ bool CSG_Natural_Breaks::Create(CSG_Table *pTable, int Field, int nClasses, int
 	}
 	else if( Field >= 0 && Field < pTable->Get_Field_Count() )
 	{
-		for(int i=0; i<pTable->Get_Count(); i++)
+		for(sLong i=0; i<pTable->Get_Count(); i++)
 		{
 			CSG_Table_Record	*pRecord	= pTable->Get_Record(i);
 
@@ -2013,7 +2013,7 @@ bool CSG_Cluster_Analysis::Create(int nFeatures)
 	{
 		m_nFeatures	= nFeatures;
 
-		m_Features.Create(m_nFeatures * sizeof(double), 0, SG_ARRAY_GROWTH_3);
+		m_Features.Create(m_nFeatures * sizeof(double), 0, TSG_Array_Growth::SG_ARRAY_GROWTH_3);
 
 		return( true );
 	}
