@@ -178,7 +178,7 @@ SAGA_API_DLL_EXPORT	void		SG_Decimal_To_Degree	(double Value, double &Deg, doubl
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-typedef int (* TSG_PFNC_Compare) (const int a, const int b);
+typedef int (* TSG_PFNC_Compare) (const sLong a, const sLong b);
 
 //---------------------------------------------------------
 class SAGA_API_DLL_EXPORT CSG_Index
@@ -190,7 +190,7 @@ public:
 		CSG_Index_Compare(void) {}
 		virtual ~CSG_Index_Compare(void) {}
 
-		virtual int				Compare				(const int a, const int b)	= 0;
+		virtual int				Compare				(const sLong a, const sLong b) = 0;
 	};
 
 
@@ -198,37 +198,37 @@ public:
 	CSG_Index(void);
 	virtual ~CSG_Index(void);
 
-								CSG_Index			(int nValues, CSG_Index_Compare &Compare);
-	bool						Create				(int nValues, CSG_Index_Compare &Compare);
+								CSG_Index			(sLong nValues, CSG_Index_Compare &Compare);
+	bool						Create				(sLong nValues, CSG_Index_Compare &Compare);
 
-								CSG_Index			(int nValues, CSG_Index_Compare *pCompare);
-	bool						Create				(int nValues, CSG_Index_Compare *pCompare);
+								CSG_Index			(sLong nValues, CSG_Index_Compare *pCompare);
+	bool						Create				(sLong nValues, CSG_Index_Compare *pCompare);
 
-								CSG_Index			(int nValues, int    *Values, bool bAscending = true);
-	bool						Create				(int nValues, int    *Values, bool bAscending = true);
+								CSG_Index			(sLong nValues, int    *Values, bool bAscending = true);
+	bool						Create				(sLong nValues, int    *Values, bool bAscending = true);
 
-								CSG_Index			(int nValues, double *Values, bool bAscending = true);
-	bool						Create				(int nValues, double *Values, bool bAscending = true);
+								CSG_Index			(sLong nValues, double *Values, bool bAscending = true);
+	bool						Create				(sLong nValues, double *Values, bool bAscending = true);
 
-								CSG_Index			(int nValues, TSG_PFNC_Compare fCompare);
-	bool						Create				(int nValues, TSG_PFNC_Compare fCompare);
+								CSG_Index			(sLong nValues, TSG_PFNC_Compare fCompare);
+	bool						Create				(sLong nValues, TSG_PFNC_Compare fCompare);
 
 	bool						Destroy				(void);
 
 	void						Show_Progress		(bool bProgress = true);
 
-	bool						Add_Entry			(int Position = -1);
-	bool						Del_Entry			(int Position = -1);
+	bool						Add_Entry			(sLong Position = -1);
+	bool						Del_Entry			(sLong Position = -1);
 
 	bool						is_Okay				(void)	const	{	return( m_nValues > 0 );	}
-	int							Get_Count			(void)	const	{	return( m_nValues     );	}
+	sLong						Get_Count			(void)	const	{	return( m_nValues     );	}
 
-	int							Get_Index			(int Position, bool Ascending = true)	const
+	sLong						Get_Index			(sLong Position, bool Ascending = true)	const
 	{
 		return( Position < 0 || Position >= m_nValues ? -1 : m_Index[Ascending ? Position : m_nValues - 1 - Position] );
 	}
 
-	int							operator []			(int Position)	const
+	sLong						operator []			(sLong Position)	const
 	{
 		return( Position < 0 || Position >= m_nValues ? -1 : m_Index[Position] );
 	}
@@ -238,12 +238,12 @@ private:
 
 	bool						m_bProgress;
 
-	int							m_nValues, *m_Index;
+	sLong						m_nValues, *m_Index;
 
 
 	void						_On_Construction	(void);
 
-	bool						_Set_Array			(int nValues);
+	bool						_Set_Array			(sLong nValues);
 	bool						_Set_Index			(CSG_Index_Compare *pCompare);
 
 };
@@ -350,31 +350,31 @@ public:
 								CSG_Vector			(const CSG_Vector &Vector);
 	bool						Create				(const CSG_Vector &Vector);
 
-								CSG_Vector			(int    n, double *Data = NULL);
-	bool						Create				(int    n, double *Data = NULL);
-
-								CSG_Vector			(size_t n, double *Data = NULL);
-	bool						Create				(size_t n, double *Data = NULL);
+								CSG_Vector			(sLong n, const double *Data = NULL);
+	bool						Create				(sLong n, const double *Data = NULL);
 
 	bool						Destroy				(void);
 
-	bool						Set_Rows			(int    nRows);
-	bool						Set_Rows			(size_t nRows);
-	bool						Add_Rows			(int    nRows);
-	bool						Add_Rows			(size_t nRows);
-	bool						Del_Rows			(int    nRows);
-	bool						Del_Rows			(size_t nRows);
+	bool						Set_Rows			(sLong  nRows);
+	bool						Add_Rows			(sLong  nRows);
+	bool						Del_Rows			(sLong  nRows);
 	bool						Add_Row				(double Value = 0.);
-	bool						Del_Row				(int    Row = -1);
-	bool						Del_Row				(size_t Row);
+	bool						Del_Row				(sLong  Row = -1);
+
+	sLong						Get_Row_Count		(void)		const	{	return( m_Array.Get_Size () );	}
+	sLong						Get_Size			(void)		const	{	return( m_Array.Get_Size () );	}
+	size_t						Get_uSize			(void)		const	{	return( m_Array.Get_uSize() );	}
+	int							Get_N				(void)		const	{	return( (int)m_Array.Get_Size() );	}
 
-	int							Get_N				(void)		const	{	return(    (int)Get_Size() );	}
-	size_t						Get_Size			(void)		const	{	return( m_Array.Get_Size() );	}
 	double *					Get_Data			(void)		const	{	return( (double *)m_Array.Get_Array() );	}
-	double						Get_Data			(int    x)	const	{	return( Get_Data()[x] );	}
-	double						operator ()			(int    x)	const	{	return( Get_Data()[x] );	}
-	double &					operator []			(int    x)			{	return( Get_Data()[x] );	}
+	double						Get_Data			(sLong  x)	const	{	return( Get_Data()[x] );	}
+	double						operator ()			(sLong  x)	const	{	return( Get_Data()[x] );	}
+	double &					operator []			(sLong  x)			{	return( Get_Data()[x] );	}
 	double &					operator []			(size_t x)			{	return( Get_Data()[x] );	}
+	double &					operator []			(int    x)			{	return( Get_Data()[x] );	}
+	const double &				operator []			(sLong  x)	const	{	return( Get_Data()[x] );	}
+	const double &				operator []			(size_t x)	const	{	return( Get_Data()[x] );	}
+	const double &				operator []			(int    x)	const	{	return( Get_Data()[x] );	}
 	operator const double *							(void)		const	{	return( Get_Data() );		}
 
 	CSG_String					to_String			(int Width = -1, int Precision = -1, bool bScientific = false, const SG_Char *Separator = NULL)	const;
@@ -462,52 +462,52 @@ public:
 								CSG_Matrix			(const CSG_Matrix &Matrix);
 	bool						Create				(const CSG_Matrix &Matrix);
 
-								CSG_Matrix			(int nCols, int nRows, double *Data = NULL);
-	bool						Create				(int nCols, int nRows, double *Data = NULL);
+								CSG_Matrix			(sLong nCols, sLong nRows, const double *Data = NULL);
+	bool						Create				(sLong nCols, sLong nRows, const double *Data = NULL);
 
-								CSG_Matrix			(int nCols, int nRows, double **Data);
-	bool						Create				(int nCols, int nRows, double **Data);
+								CSG_Matrix			(sLong nCols, sLong nRows, const double **Data);
+	bool						Create				(sLong nCols, sLong nRows, const double **Data);
 
 	bool						Destroy				(void);
 
-	bool						Set_Size			(int nRows, int nCols);
-	bool						Set_Cols			(int nCols);
-	bool						Set_Rows			(int nRows);
-	bool						Add_Cols			(int nCols);
-	bool						Add_Rows			(int nRows);
-	bool						Del_Cols			(int nCols);
-	bool						Del_Rows			(int nRows);
-	bool						Add_Col				(         double *Data = NULL);
-	bool						Add_Col				(         const CSG_Vector &Data);
-	bool						Add_Row				(         double *Data = NULL);
-	bool						Add_Row				(         const CSG_Vector &Data);
-	bool						Ins_Col				(int Col, double *Data = NULL);
-	bool						Ins_Col				(int Col, const CSG_Vector &Data);
-	bool						Ins_Row				(int Row, double *Data = NULL);
-	bool						Ins_Row				(int Row, const CSG_Vector &Data);
-	bool						Set_Col				(int Col, double *Data);
-	bool						Set_Col				(int Col, const CSG_Vector &Data);
-	bool						Set_Col				(         const CSG_Vector &Data);
-	bool						Set_Row				(int Row, double *Data);
-	bool						Set_Row				(int Row, const CSG_Vector &Data);
-	bool						Set_Row				(         const CSG_Vector &Data);
-	bool						Del_Col				(int Col);
-	bool						Del_Row				(int Row);
-	CSG_Vector					Get_Col				(int Col)	const;
-	CSG_Vector					Get_Row				(int Row)	const;
+	bool						Set_Size			(sLong nRows, sLong nCols);
+	bool						Set_Cols			(sLong nCols);
+	bool						Set_Rows			(sLong nRows);
+	bool						Add_Cols			(sLong nCols);
+	bool						Add_Rows			(sLong nRows);
+	bool						Del_Cols			(sLong nCols);
+	bool						Del_Rows			(sLong nRows);
+	bool						Add_Col				(           const double     *Data = NULL);
+	bool						Add_Col				(           const CSG_Vector &Data);
+	bool						Add_Row				(           const double     *Data = NULL);
+	bool						Add_Row				(           const CSG_Vector &Data);
+	bool						Ins_Col				(sLong Col, const double     *Data = NULL);
+	bool						Ins_Col				(sLong Col, const CSG_Vector &Data);
+	bool						Ins_Row				(sLong Row, const double     *Data = NULL);
+	bool						Ins_Row				(sLong Row, const CSG_Vector &Data);
+	bool						Set_Col				(sLong Col, const double     *Data);
+	bool						Set_Col				(sLong Col, const CSG_Vector &Data);
+	bool						Set_Col				(           const CSG_Vector &Data);
+	bool						Set_Row				(sLong Row, const double     *Data);
+	bool						Set_Row				(sLong Row, const CSG_Vector &Data);
+	bool						Set_Row				(           const CSG_Vector &Data);
+	bool						Del_Col				(sLong Col);
+	bool						Del_Row				(sLong Row);
+	CSG_Vector					Get_Col				(sLong Col) const;
+	CSG_Vector					Get_Row				(sLong Row) const;
 
-	int							Get_NX				(void)		const	{	return( m_nx );			}
-	int							Get_NY				(void)		const	{	return( m_ny );			}
-	int							Get_NCols			(void)		const	{	return( m_nx );			}
-	int							Get_NRows			(void)		const	{	return( m_ny );			}
+	int							Get_NX				(void) const { return( (int)m_nx ); }
+	int							Get_NY				(void) const { return( (int)m_ny ); }
+	sLong						Get_NCols			(void) const { return(      m_nx ); }
+	sLong						Get_NRows			(void) const { return(      m_ny ); }
 
-	operator const double **						(void)		const	{	return( (const double **)m_z );	}
-	double **					Get_Data			(void)		const	{	return( m_z );			}
+	operator const double **						(void) const { return( (const double **)m_z ); }
+	double **					Get_Data			(void) const { return(                  m_z ); }
 
-	double						operator ()			(int    Row, int    Col)	const	{	return( m_z[Row][Col] );	}
-	double						operator ()			(size_t Row, size_t Col)	const	{	return( m_z[Row][Col] );	}
-	double *					operator []			(int    Row)				const	{	return( m_z[Row]      );	}
-	double *					operator []			(size_t Row)				const	{	return( m_z[Row]      );	}
+	double						operator ()			(sLong  Row, sLong Col) const { return( m_z[Row][Col] ); }
+	double *					operator []			(sLong  Row)            const { return( m_z[Row]      ); }
+	double *					operator []			(size_t Row)            const { return( m_z[Row]      ); }
+	double *					operator []			(int    Row)            const { return( m_z[Row]      ); }
 
 	CSG_String					to_String			(int Width = -1, int Precision = -1, bool bScientific = false, const SG_Char *Separator = NULL)	const;
 	bool						from_String			(const CSG_String &String);
@@ -553,7 +553,7 @@ public:
 
 private:
 
-	int							m_nx, m_ny;
+	sLong						m_nx, m_ny;
 
 	double						**m_z;
 
@@ -803,7 +803,7 @@ public:
 
 protected:
 
-	bool					m_bWeights;
+	bool					m_bWeights{false};
 
 	CSG_Array_Int			m_Count;
 
@@ -1403,8 +1403,8 @@ public:
 	void						Add					(double x, double y);
 
 	int							Get_Count			(void)	const	{	return( m_x.Get_N() );	}
-	double						Get_xMin			(void)	const	{	return( m_x.Get_N() > 0 ? m_x(0              ) : 0. );	}
-	double						Get_xMax			(void)	const	{	return( m_x.Get_N() > 0 ? m_x(m_x.Get_N() - 1) : 0. );	}
+	double						Get_xMin			(void)	const	{	return( m_x.Get_N() > 0 ? m_x(0                 ) : 0. );	}
+	double						Get_xMax			(void)	const	{	return( m_x.Get_N() > 0 ? m_x(m_x.Get_Size() - 1) : 0. );	}
 	double						Get_x				(int i)	const	{	return( i >= 0 && i < m_x.Get_N() ? m_x(i) : 0. );	}
 	double						Get_y				(int i)	const	{	return( i >= 0 && i < m_y.Get_N() ? m_y(i) : 0. );	}
 
@@ -1433,9 +1433,9 @@ public:
 	bool					Destroy				(void);
 
 	bool					Set_Point_Count		(int Count)	{	return( m_Points.Set_Count(Count) );	}
-	int						Get_Point_Count		(void)		{	return( m_Points.Get_Count() );			}
+	int						Get_Point_Count		(void)		{	return( (int)m_Points.Get_Count() );	}
 
-	CSG_Points_Z &			Get_Points			(void)		{	return( m_Points );	}
+	CSG_Points_3D &			Get_Points			(void)		{	return( m_Points );	}
 
 	bool					Add_Point			(double x, double y, double z)	{	return( m_Points.Add(  x,   y, z) );	}
 	bool					Add_Point			(const TSG_Point &p, double z)	{	return( m_Points.Add(p.x, p.y, z) );	}
@@ -1465,14 +1465,14 @@ public:
 
 private:
 
-	CSG_Points_Z			m_Points;
+	CSG_Points_3D			m_Points;
 
 	CSG_Vector				m_V;
 
 
-	double					_Get_hDistance		(TSG_Point_Z A, TSG_Point_Z B);
+	double					_Get_hDistance		(TSG_Point_3D A, TSG_Point_3D B);
 	double					_Get_Base_Funtion	(double x);
-	double					_Get_Base_Funtion	(TSG_Point_Z A, double x, double y);
+	double					_Get_Base_Funtion	(TSG_Point_3D A, double x, double y);
 
 };
 
@@ -1484,7 +1484,7 @@ private:
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-typedef enum ESG_Test_Distribution_Type
+typedef enum
 {
 	TESTDIST_TYPE_Left	= 0,
 	TESTDIST_TYPE_Right,
@@ -1542,7 +1542,7 @@ SAGA_API_DLL_EXPORT CSG_Matrix	SG_Get_Correlation_Matrix		(const CSG_Matrix &Val
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-typedef enum ESG_Regression_Correction
+typedef enum
 {
 	REGRESSION_CORR_None	= 0,
 	REGRESSION_CORR_Smith,
@@ -1563,7 +1563,7 @@ SAGA_API_DLL_EXPORT double		SG_Regression_Get_Adjusted_R2	(double R2, int nSampl
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-typedef enum ESG_Regression_Type
+typedef enum
 {
 	REGRESSION_Linear	= 0,	// Y = a + b * X
 	REGRESSION_Rez_X,			// Y = a + b / X
@@ -1776,8 +1776,8 @@ public:
 	bool						Destroy				(void);
 
 	bool						Add_Sample			(double Weight, double Dependent, const CSG_Vector &Predictors);
-	int							Get_Sample_Count	(void)	const	{	return( m_X.Get_NRows()     );	}
-	int							Get_Predictor_Count	(void)	const	{	return( m_X.Get_NCols() - 1 );	}
+	int							Get_Sample_Count	(void)	const	{	return( m_X.Get_NY()     );	}
+	int							Get_Predictor_Count	(void)	const	{	return( m_X.Get_NX() - 1 );	}
 
 	bool						Calculate			(const CSG_Vector &Weights, const CSG_Vector &Dependents, const CSG_Matrix &Predictors, bool bLogistic = false);
 	bool						Calculate			(bool bLogistic = false);
