@@ -231,6 +231,12 @@ CSG_Parameter_Bool::CSG_Parameter_Bool(CSG_Parameters *pOwner, CSG_Parameter *pP
 	m_Value	= false;
 }
 
+//---------------------------------------------------------
+bool CSG_Parameter_Bool::Toggle_Value(void)
+{
+	return( Set_Value(asBool() ? 0 : 1) );
+}
+
 //---------------------------------------------------------
 int CSG_Parameter_Bool::_Set_Value(int Value)
 {
@@ -861,6 +867,12 @@ CSG_Parameter_Choice::CSG_Parameter_Choice(CSG_Parameters *pOwner, CSG_Parameter
 	m_Value	= -1;
 }
 
+//---------------------------------------------------------
+bool CSG_Parameter_Choice::Toggle_Value(void)
+{
+	return( Get_Count() > 1 && Set_Value((asInt() + 1) % Get_Count()) );
+}
+
 //---------------------------------------------------------
 bool CSG_Parameter_Choice::Del_Items(void)
 {
@@ -937,11 +949,6 @@ CSG_String CSG_Parameter_Choice::Get_Items(void) const
 			Items += "|";
 		}
 
-		if( !m_Data[i].is_Empty() )
-		{
-			Items += "{" + m_Data[i] + "}";
-		}
-
 		Items += m_Items[i];
 	}
 
@@ -1122,7 +1129,7 @@ void CSG_Parameter_Choices::_Set_String(void)
 {
 	m_String.Clear();
 
-	for(size_t i=0; i<m_Selection.Get_Size(); i++)
+	for(size_t i=0; i<m_Selection.Get_uSize(); i++)
 	{
 		m_String	+= CSG_String::Format("%d;", m_Selection[i]);
 	}
@@ -1189,7 +1196,7 @@ void CSG_Parameter_Choices::Add_Item(const CSG_String &Item, const CSG_String &D
 //---------------------------------------------------------
 bool CSG_Parameter_Choices::is_Selected(int Index)
 {
-	for(size_t i=0; i<m_Selection.Get_Size(); i++)
+	for(size_t i=0; i<m_Selection.Get_uSize(); i++)
 	{
 		if( Index == m_Selection[i] )
 		{
@@ -1211,11 +1218,11 @@ bool CSG_Parameter_Choices::Select(int Index, bool bSelect)
 		}
 		else if( !bSelect )
 		{
-			for(size_t i=0; i<m_Selection.Get_Size(); i++)
+			for(size_t i=0; i<m_Selection.Get_uSize(); i++)
 			{
 				if( Index == m_Selection[i] )
 				{
-					for(size_t j=i+1; j<m_Selection.Get_Size(); i++, j++)
+					for(size_t j=i+1; j<m_Selection.Get_uSize(); i++, j++)
 					{
 						m_Selection[i]	= m_Selection[j];
 					}
@@ -1710,7 +1717,7 @@ CSG_Parameter_Fixed_Table::CSG_Parameter_Fixed_Table(CSG_Parameters *pOwner, CSG
 //---------------------------------------------------------
 void CSG_Parameter_Fixed_Table::_Set_String(void)
 {
-	m_String.Printf("%s (%s: %d, %s: %d)", m_Table.Get_Name(), _TL("columns"), m_Table.Get_Field_Count(), _TL("rows"), m_Table.Get_Record_Count());
+	m_String.Printf("%s (%s: %d, %s: %lld)", m_Table.Get_Name(), _TL("columns"), m_Table.Get_Field_Count(), _TL("rows"), m_Table.Get_Count());
 }
 
 //---------------------------------------------------------
@@ -1743,7 +1750,7 @@ bool CSG_Parameter_Fixed_Table::_Serialize(CSG_MetaData &Entry, bool bSave)
 
 		pNode	= Entry.Add_Child("RECORDS");
 
-		for(int iRecord=0; iRecord<m_Table.Get_Count(); iRecord++)
+		for(sLong iRecord=0; iRecord<m_Table.Get_Count(); iRecord++)
 		{
 			CSG_MetaData	*pEntry	= pNode->Add_Child("RECORD");
 
@@ -2106,6 +2113,28 @@ bool CSG_Parameter_Table_Field::_Serialize(CSG_MetaData &Entry, bool bSave)
 	}
 }
 
+//---------------------------------------------------------
+CSG_String CSG_Parameter_Table_Field::Get_Choices(const CSG_Table &Table, bool bAllowNone)
+{
+	CSG_String Choices;
+
+	for(int i=0; i<Table.Get_Field_Count(); i++)
+	{
+		if( i > 0 ) { Choices += "|"; }
+
+		Choices += Table.Get_Field_Name(i);
+	}
+
+	if( bAllowNone )
+	{
+		if( Table.Get_Field_Count() ) { Choices += "|"; }
+
+		Choices += _TL("<not set>");
+	}
+
+	return( Choices );
+}
+
 
 ///////////////////////////////////////////////////////////
 //														 //
