@@ -118,7 +118,7 @@
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-typedef enum ESG_Parameter_Type
+typedef enum
 {
 	PARAMETER_TYPE_Node		= 0,
 
@@ -263,6 +263,8 @@ public:	///////////////////////////////////////////////////
 	virtual bool					Set_Value				(void             *Value);
 	virtual bool					Set_Value				(CSG_Parameter    *Value);
 
+	virtual bool					Toggle_Value			(void);
+
 	bool							Set_Default				(int               Value);
 	bool							Set_Default				(double            Value);
 	bool							Set_Default				(const CSG_String &Value);
@@ -392,6 +394,8 @@ public:
 
 	virtual TSG_Parameter_Type	Get_Type				(void)	const	{	return( PARAMETER_TYPE_Bool );	}
 
+	virtual bool				Toggle_Value			(void);
+
 
 protected:
 
@@ -646,6 +650,8 @@ public:
 
 	virtual TSG_Parameter_Type	Get_Type				(void)	const	{	return( PARAMETER_TYPE_Choice );	}
 
+	virtual bool				Toggle_Value			(void);
+
 	bool						Del_Items				(void);
 	bool						Add_Item				(const CSG_String &Item, const CSG_String &Data = "");
 
@@ -1011,6 +1017,8 @@ public:
 
 	CSG_Table *					Get_Table				(void)	const;
 
+	static CSG_String			Get_Choices				(const class CSG_Table &Table, bool bAllowNone = false);
+
 
 protected:
 
@@ -1318,9 +1326,6 @@ public:
 
 	virtual bool				Update_Data				(void)	{	return( true );	}
 
-	virtual int					Get_Data_Count			(void)		const	{	return( Get_Item_Count() );	}
-	virtual CSG_Data_Object *	Get_Data				(int Index)	const	{	return( Get_Item(Index) );	}
-
 
 protected:
 
@@ -1365,11 +1370,8 @@ public:
 
 	virtual bool				Update_Data				(void);
 
-	virtual int					Get_Data_Count			(void)		const	{	return( Get_Grid_Count() );	}
-	virtual CSG_Data_Object *	Get_Data				(int Index)	const	{	return( (CSG_Data_Object *)Get_Grid(Index) );	}
-
 	int							Get_Grid_Count			(void)		const	{	return( (int)m_Grids.Get_Size() );	}
-	CSG_Grid *					Get_Grid				(int Index)	const	{	return( Index >= 0 && Index < Get_Data_Count() ? (CSG_Grid *)m_Grids[Index] : NULL );	}
+	CSG_Grid *					Get_Grid				(int Index)	const	{	return( Index >= 0 && Index < Get_Grid_Count() ? (CSG_Grid *)m_Grids[Index] : NULL );	}
 
 
 protected:
@@ -1562,7 +1564,9 @@ public:
 	bool						Add_Grids					(const CSG_String &ID, const CSG_String &Name, bool bOptional, bool bZLevels = false);
 
 	bool						On_Parameter_Changed		(CSG_Parameters *pParameters, CSG_Parameter *pParameter);
+	static bool					On_Parameter_Changed		(CSG_Parameters *pParameters, CSG_Parameter *pParameter, const CSG_String Prefix);
 	bool						On_Parameters_Enable		(CSG_Parameters *pParameters, CSG_Parameter *pParameter);
+	static bool					On_Parameters_Enable		(CSG_Parameters *pParameters, CSG_Parameter *pParameter, const CSG_String Prefix);
 
 	bool						Set_User_Defined			(CSG_Parameters *pParameters, const TSG_Rect &Extent, int Rows = 0, int Rounding = 2);
 	bool						Set_User_Defined			(CSG_Parameters *pParameters, CSG_Shapes *pPoints, int Scale = 4  , int Rounding = 2);
