@@ -107,15 +107,19 @@ CSG_Table * SG_Create_Table(CSG_Table *pTemplate)
 }
 
 //---------------------------------------------------------
-CSG_Table * SG_Create_Table(const CSG_String &File_Name, TSG_Table_File_Type Format, int Encoding)
+CSG_Table * SG_Create_Table(const char       *File, TSG_Table_File_Type Format, int Encoding) { return( SG_Create_Table(CSG_String(File), Format, Encoding) ); }
+CSG_Table * SG_Create_Table(const wchar_t    *File, TSG_Table_File_Type Format, int Encoding) { return( SG_Create_Table(CSG_String(File), Format, Encoding) ); }
+CSG_Table * SG_Create_Table(const CSG_String &File, TSG_Table_File_Type Format, int Encoding)
 {
-	return( new CSG_Table(File_Name, Format, Encoding) );
+	return( new CSG_Table(File, Format, Encoding) );
 }
 
 //---------------------------------------------------------
-CSG_Table * SG_Create_Table(const CSG_String &File_Name, TSG_Table_File_Type Format, const SG_Char Separator, int Encoding)
+CSG_Table * SG_Create_Table(const char       *File, TSG_Table_File_Type Format, const SG_Char Separator, int Encoding) { return( SG_Create_Table(CSG_String(File), Format, Separator, Encoding) ); }
+CSG_Table * SG_Create_Table(const wchar_t    *File, TSG_Table_File_Type Format, const SG_Char Separator, int Encoding) { return( SG_Create_Table(CSG_String(File), Format, Separator, Encoding) ); }
+CSG_Table * SG_Create_Table(const CSG_String &File, TSG_Table_File_Type Format, const SG_Char Separator, int Encoding)
 {
-	return( new CSG_Table(File_Name, Format, Encoding) );
+	return( new CSG_Table(File, Format, Encoding) );
 }
 
 
@@ -136,9 +140,7 @@ CSG_Table::CSG_Table(void)
 CSG_Table::CSG_Table(const CSG_Table &Table)
 	: CSG_Data_Object()
 {
-	_On_Construction();
-
-	Create(Table);
+	_On_Construction(); Create(Table);
 }
 
 bool CSG_Table::Create(const CSG_Table &Table)
@@ -147,57 +149,58 @@ bool CSG_Table::Create(const CSG_Table &Table)
 }
 
 //---------------------------------------------------------
-CSG_Table::CSG_Table(const CSG_String &File_Name, TSG_Table_File_Type Format, int Encoding)
+CSG_Table::CSG_Table(const char       *File, TSG_Table_File_Type Format, int Encoding) : CSG_Table(CSG_String(File), Format, Encoding) {}
+CSG_Table::CSG_Table(const wchar_t    *File, TSG_Table_File_Type Format, int Encoding) : CSG_Table(CSG_String(File), Format, Encoding) {}
+CSG_Table::CSG_Table(const CSG_String &File, TSG_Table_File_Type Format, int Encoding)
+	: CSG_Data_Object()
 {
-	_On_Construction();
-
-	Create(File_Name, Format, Encoding);
+	_On_Construction(); Create(File, Format, Encoding);
 }
 
-bool CSG_Table::Create(const CSG_String &File_Name, TSG_Table_File_Type Format, int Encoding)
+bool CSG_Table::Create(const char       *File, TSG_Table_File_Type Format, int Encoding) { return( Create(CSG_String(File), Format, Encoding) ); }
+bool CSG_Table::Create(const wchar_t    *File, TSG_Table_File_Type Format, int Encoding) { return( Create(CSG_String(File), Format, Encoding) ); }
+bool CSG_Table::Create(const CSG_String &File, TSG_Table_File_Type Format, int Encoding)
 {
 	Destroy();
 
-	SG_UI_Msg_Add(CSG_String::Format("%s %s: %s...", _TL("Loading"), _TL("table"), File_Name.c_str()), true);
+	SG_UI_Msg_Add(CSG_String::Format("%s %s: %s...", _TL("Loading"), _TL("table"), File.c_str()), true);
 
 	//-----------------------------------------------------
-	bool	bResult	= File_Name.BeforeFirst(':').Cmp("PGSQL") && SG_File_Exists(File_Name) && Load(File_Name, (int)Format, '\0', Encoding);
+	bool bResult = File.BeforeFirst(':').Cmp("PGSQL") && SG_File_Exists(File) && Load(File, (int)Format, '\0', Encoding);
 
 	if( bResult )
 	{
-		Set_File_Name(File_Name, true);
+		Set_File_Name(File, true);
 	}
 
 	//-----------------------------------------------------
-	else if( File_Name.BeforeFirst(':').Cmp("PGSQL") == 0 )	// database source
+	else if( File.BeforeFirst(':').Cmp("PGSQL") == 0 )	// database source
 	{
-		CSG_String	s(File_Name);
+		CSG_String s(File);
 
-		s	= s.AfterFirst(':');	CSG_String	Host  (s.BeforeFirst(':'));
-		s	= s.AfterFirst(':');	CSG_String	Port  (s.BeforeFirst(':'));
-		s	= s.AfterFirst(':');	CSG_String	DBName(s.BeforeFirst(':'));
-		s	= s.AfterFirst(':');	CSG_String	Table (s.BeforeFirst(':'));
+		s = s.AfterFirst(':'); CSG_String Host  (s.BeforeFirst(':'));
+		s = s.AfterFirst(':'); CSG_String Port  (s.BeforeFirst(':'));
+		s = s.AfterFirst(':'); CSG_String DBName(s.BeforeFirst(':'));
+		s = s.AfterFirst(':'); CSG_String Table (s.BeforeFirst(':'));
 
-		CSG_Tool	*pTool	= SG_Get_Tool_Library_Manager().Create_Tool("db_pgsql", 0, true);	// CGet_Connections
+		CSG_Tool *pTool = SG_Get_Tool_Library_Manager().Create_Tool("db_pgsql", 0, true); // CGet_Connections
 
 		if(	pTool != NULL )
 		{
 			SG_UI_ProgressAndMsg_Lock(true);
 
 			//---------------------------------------------
-			CSG_Table	Connections;
-			CSG_String	Connection	= DBName + " [" + Host + ":" + Port + "]";
+			CSG_Table Connections; CSG_String Connection(DBName + " [" + Host + ":" + Port + "]");
 
-			pTool->Set_Manager(NULL);
-			pTool->On_Before_Execution();
+			pTool->Set_Manager(NULL); pTool->On_Before_Execution();
 
-			if( SG_TOOL_PARAMETER_SET("CONNECTIONS", &Connections) && pTool->Execute() )	// CGet_Connections
+			if( SG_TOOL_PARAMETER_SET("CONNECTIONS", &Connections) && pTool->Execute() ) // CGet_Connections
 			{
 				for(int i=0; !bResult && i<Connections.Get_Count(); i++)
 				{
 					if( !Connection.Cmp(Connections[i].asString(0)) )
 					{
-						bResult	= true;
+						bResult = true;
 					}
 				}
 			}
@@ -205,14 +208,13 @@ bool CSG_Table::Create(const CSG_String &File_Name, TSG_Table_File_Type Format,
 			SG_Get_Tool_Library_Manager().Delete_Tool(pTool);
 
 			//---------------------------------------------
-			if( bResult && (bResult = (pTool = SG_Get_Tool_Library_Manager().Create_Tool("db_pgsql", 12, true)) != NULL) == true )	// CPGIS_Table_Load
+			if( bResult && (bResult = (pTool = SG_Get_Tool_Library_Manager().Create_Tool("db_pgsql", 12, true)) != NULL) == true ) // CPGIS_Table_Load
 			{
-				pTool->Set_Manager(NULL);
-				pTool->On_Before_Execution();
+				pTool->Set_Manager(NULL); pTool->On_Before_Execution();
 
-				bResult	=  SG_TOOL_PARAMETER_SET("CONNECTION", Connection)
-						&& SG_TOOL_PARAMETER_SET("TABLES"    , Table)
-						&& SG_TOOL_PARAMETER_SET("TABLE"     , this)
+				bResult =  SG_TOOL_PARAMETER_SET("CONNECTION", Connection)
+						&& SG_TOOL_PARAMETER_SET("DB_TABLE"  , Table     )
+						&& SG_TOOL_PARAMETER_SET("TABLE"     , this      )
 						&& pTool->Execute();
 
 				SG_Get_Tool_Library_Manager().Delete_Tool(pTool);
@@ -241,26 +243,26 @@ bool CSG_Table::Create(const CSG_String &File_Name, TSG_Table_File_Type Format,
 }
 
 //---------------------------------------------------------
-CSG_Table::CSG_Table(const CSG_String &File_Name, TSG_Table_File_Type Format, const SG_Char Separator, int Encoding)
+CSG_Table::CSG_Table(const char       *File, TSG_Table_File_Type Format, const SG_Char Separator, int Encoding) : CSG_Table(CSG_String(File), Format, Separator, Encoding) {}
+CSG_Table::CSG_Table(const wchar_t    *File, TSG_Table_File_Type Format, const SG_Char Separator, int Encoding) : CSG_Table(CSG_String(File), Format, Separator, Encoding) {}
+CSG_Table::CSG_Table(const CSG_String &File, TSG_Table_File_Type Format, const SG_Char Separator, int Encoding)
 	: CSG_Data_Object()
 {
-	_On_Construction();
-
-	Create(File_Name, Format, Separator, Encoding);
+	_On_Construction(); Create(File, Format, Separator, Encoding);
 }
 
-bool CSG_Table::Create(const CSG_String &File_Name, TSG_Table_File_Type Format, const SG_Char Separator, int Encoding)
+bool CSG_Table::Create(const char       *File, TSG_Table_File_Type Format, const SG_Char Separator, int Encoding) { return( Create(CSG_String(File), Format, Separator, Encoding) ); }
+bool CSG_Table::Create(const wchar_t    *File, TSG_Table_File_Type Format, const SG_Char Separator, int Encoding) { return( Create(CSG_String(File), Format, Separator, Encoding) ); }
+bool CSG_Table::Create(const CSG_String &File, TSG_Table_File_Type Format, const SG_Char Separator, int Encoding)
 {
-	return( Load(File_Name, (int)Format, Separator, Encoding) );
+	return( Load(File, (int)Format, Separator, Encoding) );
 }
 
 //---------------------------------------------------------
 CSG_Table::CSG_Table(const CSG_Table *pTemplate)
 	: CSG_Data_Object()
 {
-	_On_Construction();
-
-	Create(pTemplate);
+	_On_Construction(); Create(pTemplate);
 }
 
 bool CSG_Table::Create(const CSG_Table *pTemplate)
@@ -276,7 +278,7 @@ bool CSG_Table::Create(const CSG_Table *pTemplate)
 	Set_Description       (pTemplate->Get_Description());
 	Set_NoData_Value_Range(pTemplate->Get_NoData_Value(), pTemplate->Get_NoData_Value(true));
 
-	m_Encoding	= pTemplate->m_Encoding;
+	m_Encoding = pTemplate->m_Encoding;
 
 	for(int i=0; i<pTemplate->Get_Field_Count(); i++)
 	{
@@ -294,18 +296,18 @@ bool CSG_Table::Create(const CSG_Table *pTemplate)
 //---------------------------------------------------------
 void CSG_Table::_On_Construction(void)
 {
-	m_nFields		= 0;
-	m_Field_Name	= NULL;
-	m_Field_Type	= NULL;
-	m_Field_Stats	= NULL;
+	m_nFields     = 0;
+	m_Field_Name  = NULL;
+	m_Field_Type  = NULL;
+	m_Field_Stats = NULL;
 
-	m_Records		= NULL;
-	m_nRecords		= 0;
-	m_nBuffer		= 0;
+	m_Records     = NULL;
+	m_nRecords    = 0;
+	m_nBuffer     = 0;
 
-	m_Encoding		= SG_FILE_ENCODING_UTF8;
+	m_Encoding    = SG_FILE_ENCODING_UTF8;
 
-	m_Selection.Create(sizeof(size_t), 0, SG_ARRAY_GROWTH_3);
+	m_Selection.Create(sizeof(sLong), 0, TSG_Array_Growth::SG_ARRAY_GROWTH_3);
 
 	Set_Update_Flag();
 }
@@ -336,15 +338,14 @@ bool CSG_Table::Destroy(void)
 			delete(m_Field_Stats[i]);
 		}
 
-		m_nFields		= 0;
-
-		SG_Free(m_Field_Name);
-		SG_Free(m_Field_Type);
+		SG_Free(m_Field_Name );
 		SG_Free(m_Field_Stats);
+		SG_Free(m_Field_Type );
 
-		m_Field_Name	= NULL;
-		m_Field_Type	= NULL;
-		m_Field_Stats	= NULL;
+		m_nFields     = 0;
+		m_Field_Name  = NULL;
+		m_Field_Type  = NULL;
+		m_Field_Stats = NULL;
 	}
 
 	CSG_Data_Object::Destroy();
@@ -371,27 +372,27 @@ CSG_Table & CSG_Table::operator = (const CSG_Table &Table)
 bool CSG_Table::Assign(CSG_Data_Object *pObject)
 {
 	if( !pObject || !pObject->is_Valid()
-	||	(	pObject->Get_ObjectType() != SG_DATAOBJECT_TYPE_Table
-		&&	pObject->Get_ObjectType() != SG_DATAOBJECT_TYPE_Shapes
-		&&	pObject->Get_ObjectType() != SG_DATAOBJECT_TYPE_PointCloud	) )
+	|| ( pObject->Get_ObjectType() != SG_DATAOBJECT_TYPE_Table
+	  && pObject->Get_ObjectType() != SG_DATAOBJECT_TYPE_Shapes
+	  && pObject->Get_ObjectType() != SG_DATAOBJECT_TYPE_PointCloud	) )
 	{
 		return( false );
 	}
 
-	CSG_Table	*pTable	= (CSG_Table *)pObject;
+	CSG_Table *pTable = (CSG_Table *)pObject;
 
 	if( !Create(pTable) )
 	{
 		return( false );
 	}
 
-	for(int i=0; i<pTable->Get_Count(); i++)
+	for(sLong i=0; i<pTable->Get_Count(); i++)
 	{
 		Add_Record(pTable->Get_Record(i));
 	}
 
-	Get_MetaData()	= pTable->Get_MetaData();
-//	Get_History ()	= pTable->Get_History ();
+	Get_MetaData() = pTable->Get_MetaData();
+//	Get_History () = pTable->Get_History ();
 
 	return( true );
 }
@@ -401,7 +402,7 @@ bool CSG_Table::Assign_Values(const CSG_Table &Table)
 {
 	if( is_Compatible(Table) && Set_Count(Table.Get_Count()) )
 	{
-		for(int i=0; i<Table.Get_Count(); i++)
+		for(sLong i=0; i<Table.Get_Count(); i++)
 		{
 			Get_Record(i)->Assign(Table.Get_Record(i));
 		}
@@ -511,7 +512,7 @@ bool CSG_Table::Add_Field(const CSG_String &Name, TSG_Data_Type Type, int Positi
 	m_Field_Stats[Position]	= new CSG_Simple_Statistics();
 
 	//-----------------------------------------------------
-	for(int i=0; i<m_nRecords; i++)
+	for(sLong i=0; i<m_nRecords; i++)
 	{
 		m_Records[i]->_Add_Field(Position);
 	}
@@ -549,7 +550,7 @@ bool CSG_Table::Del_Field(int del_Field)
 	m_Field_Stats	= (CSG_Simple_Statistics **)SG_Realloc(m_Field_Stats, m_nFields * sizeof(CSG_Simple_Statistics *));
 
 	//-------------------------------------------------
-	for(int i=0; i<m_nRecords; i++)
+	for(sLong i=0; i<m_nRecords; i++)
 	{
 		m_Records[i]->_Del_Field(del_Field);
 	}
@@ -593,7 +594,7 @@ bool CSG_Table::Mov_Field(int iField, int Position)
 	}
 
 	#pragma omp parallel for
-	for(int i=0; i<m_nRecords; i++)
+	for(sLong i=0; i<m_nRecords; i++)
 	{
 		*m_Records[i]->Get_Value(Position) = *m_Records[i]->Get_Value(iField);
 	}
@@ -640,19 +641,19 @@ bool CSG_Table::Set_Field_Type(int iField, TSG_Data_Type Type)
 		return( true );
 	}
 
-	m_Field_Type[iField]	= Type;
+	m_Field_Type[iField] = Type;
 
-	for(int i=0; i<m_nRecords; i++)
+	for(sLong i=0; i<m_nRecords; i++)
 	{
-		CSG_Table_Record	*pRecord	= m_Records[i];
+		CSG_Table_Record *pRecord = m_Records[i];
 
-		CSG_Table_Value	*pNew	= CSG_Table_Record::_Create_Value(Type);
+		CSG_Table_Value  *pValue  = CSG_Table_Record::_Create_Value(Type);
 
-		(*pNew)	= *pRecord->m_Values[iField];
+		(*pValue) = *pRecord->m_Values[iField];
 
 		delete(pRecord->m_Values[iField]);
 
-		pRecord->m_Values[iField]	= pNew;
+		pRecord->m_Values[iField] = pValue;
 
 		pRecord->Set_Modified();
 	}
@@ -663,30 +664,30 @@ bool CSG_Table::Set_Field_Type(int iField, TSG_Data_Type Type)
 //---------------------------------------------------------
 int CSG_Table::Get_Field_Length(int iField, int Encoding)	const
 {
-	size_t	Length	= 0;
+	size_t Length = 0;
 
 	if( iField >= 0 && iField < m_nFields && m_Field_Type[iField] == SG_DATATYPE_String )
 	{
-		for(int i=0; i<m_nRecords; i++)
+		for(sLong i=0; i<m_nRecords; i++)
 		{
-			CSG_String	s(m_Records[i]->asString(iField));
+			CSG_String s(m_Records[i]->asString(iField));
 
-			size_t	n;
+			size_t nBytes;
 
 			switch( Encoding )
 			{
 			default                      :
-			case SG_FILE_ENCODING_UTF7   : n = s.Length()            ; break;
-			case SG_FILE_ENCODING_UTF8   : n = s.to_UTF8().Get_Size(); break;
+			case SG_FILE_ENCODING_UTF7   : nBytes = s.Length()            ; break;
+			case SG_FILE_ENCODING_UTF8   : nBytes = s.to_UTF8().Get_Size(); break;
 			case SG_FILE_ENCODING_UTF16LE:
-			case SG_FILE_ENCODING_UTF16BE: n = s.Length() * 2        ; break;
+			case SG_FILE_ENCODING_UTF16BE: nBytes = s.Length() * 2        ; break;
 			case SG_FILE_ENCODING_UTF32LE:
-			case SG_FILE_ENCODING_UTF32BE: n = s.Length() * 4        ; break;
+			case SG_FILE_ENCODING_UTF32BE: nBytes = s.Length() * 4        ; break;
 			}
 
-			if( Length < n )
+			if( Length < nBytes )
 			{
-				Length	= n;
+				Length = nBytes;
 			}
 		}
 	}
@@ -708,6 +709,34 @@ int CSG_Table::Get_Field(const CSG_String &Name) const
 	return( -1 );
 }
 
+//---------------------------------------------------------
+/**
+* Returns the index of the field with the requested name or
+* -1 if it does not exists.
+*/
+int CSG_Table::Find_Field(const CSG_String &Name) const
+{
+	for(int i=0; i<Get_Field_Count(); i++)
+	{
+		if( !Name.Cmp(Get_Field_Name(i)) )
+		{
+			return( i );
+		}
+	}
+
+	return( -1 );
+}
+
+//---------------------------------------------------------
+/**
+* Returns true if a field with the requested name exists and
+* sets the Index parameter to its position.
+*/
+bool CSG_Table::Find_Field(const CSG_String &Name, int &Index) const
+{
+	return( (Index = Find_Field(Name)) >= 0 );
+}
+
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -726,15 +755,15 @@ bool CSG_Table::_Inc_Array(void)
 		return( true );
 	}
 
-	CSG_Table_Record	**pRecords	= (CSG_Table_Record **)SG_Realloc(m_Records, (m_nBuffer + GET_GROW_SIZE(m_nBuffer)) * sizeof(CSG_Table_Record *));
+	CSG_Table_Record **pRecords = (CSG_Table_Record **)SG_Realloc(m_Records, (m_nBuffer + GET_GROW_SIZE(m_nBuffer)) * sizeof(CSG_Table_Record *));
 
 	if( pRecords == NULL )
 	{
 		return( false );
 	}
 
-	m_Records	= pRecords;
-	m_nBuffer	+= GET_GROW_SIZE(m_nBuffer);
+	m_Records  = pRecords;
+	m_nBuffer += GET_GROW_SIZE(m_nBuffer);
 
 	return( true );
 }
@@ -761,7 +790,7 @@ bool CSG_Table::_Dec_Array(void)
 }
 
 //---------------------------------------------------------
-CSG_Table_Record * CSG_Table::_Get_New_Record(int Index)
+CSG_Table_Record * CSG_Table::_Get_New_Record(sLong Index)
 {
 	return( new CSG_Table_Record(this, Index) );
 }
@@ -773,24 +802,17 @@ CSG_Table_Record * CSG_Table::Add_Record(CSG_Table_Record *pCopy)
 }
 
 //---------------------------------------------------------
-CSG_Table_Record * CSG_Table::Ins_Record(int iRecord, CSG_Table_Record *pCopy)
+CSG_Table_Record * CSG_Table::Ins_Record(sLong iRecord, CSG_Table_Record *pCopy)
 {
 	if( iRecord < 0 ) { iRecord = 0; } else if( iRecord > m_nRecords ) { iRecord = m_nRecords; }
 
-	CSG_Table_Record	*pRecord	= _Inc_Array() ? _Get_New_Record(m_nRecords) : NULL;
+	CSG_Table_Record *pRecord = _Inc_Array() ? _Get_New_Record(m_nRecords) : NULL;
 
 	if( pRecord )
 	{
 		if( pCopy )
 		{
-			if( Get_ObjectType() == SG_DATAOBJECT_TYPE_Shapes && pCopy->Get_Table()->Get_ObjectType() == SG_DATAOBJECT_TYPE_Shapes )
-			{
-				((CSG_Shape *)pRecord)->Assign((CSG_Shape *)pCopy, true);
-			}
-			else
-			{
-				pRecord->Assign(pCopy);
-			}
+			pRecord->Assign(pCopy);
 		}
 
 		//-------------------------------------------------
@@ -798,26 +820,26 @@ CSG_Table_Record * CSG_Table::Ins_Record(int iRecord, CSG_Table_Record *pCopy)
 		{
 			if( Get_Selection_Count() > 0 )	// update selection index
 			{
-				size_t	*Selection	= (size_t *)m_Selection.Get_Array();
+				sLong *Selection = (sLong *)m_Selection.Get_Array();
 
-				for(size_t i=0; i<m_Selection.Get_Size(); i++)
+				for(sLong i=0; i<m_Selection.Get_Size(); i++)
 				{
-					if( Selection[i] > (size_t)iRecord )
+					if( Selection[i] > iRecord )
 					{
 						Selection[i]++;
 					}
 				}
 			}
 
-			for(int i=m_nRecords; i>iRecord; i--)
+			for(sLong i=m_nRecords; i>iRecord; i--)
 			{
 				m_Records[i] = m_Records[i - 1]; m_Records[i]->m_Index = i;
 			}
 
-			pRecord->m_Index	= iRecord;
+			pRecord->m_Index = iRecord;
 		}
 
-		m_Records[iRecord]	= pRecord;
+		m_Records[iRecord] = pRecord;
 		m_nRecords++;
 
 		//-------------------------------------------------
@@ -837,7 +859,7 @@ CSG_Table_Record * CSG_Table::Ins_Record(int iRecord, CSG_Table_Record *pCopy)
 }
 
 //---------------------------------------------------------
-bool CSG_Table::Set_Record(int iRecord, CSG_Table_Record *pCopy)
+bool CSG_Table::Set_Record(sLong iRecord, CSG_Table_Record *pCopy)
 {
 	if( iRecord >= 0 && iRecord < m_nRecords && pCopy )
 	{
@@ -848,7 +870,7 @@ bool CSG_Table::Set_Record(int iRecord, CSG_Table_Record *pCopy)
 }
 
 //---------------------------------------------------------
-bool CSG_Table::Del_Record(int iRecord)
+bool CSG_Table::Del_Record(sLong iRecord)
 {
 	if( iRecord >= 0 && iRecord < m_nRecords )
 	{
@@ -861,7 +883,7 @@ bool CSG_Table::Del_Record(int iRecord)
 
 		m_nRecords--;
 
-		for(int i=iRecord; i<m_nRecords; i++)
+		for(sLong i=iRecord; i<m_nRecords; i++)
 		{
 			m_Records[i] = m_Records[i + 1]; m_Records[i]->m_Index = i;
 		}
@@ -891,21 +913,21 @@ bool CSG_Table::Del_Records(void)
 {
 	Del_Index();
 
-	for(int iRecord=0; iRecord<m_nRecords; iRecord++)
+	for(sLong iRecord=0; iRecord<m_nRecords; iRecord++)
 	{
 		delete(m_Records[iRecord]);
 	}
 
 	SG_FREE_SAFE(m_Records);
 
-	m_nRecords	= 0;
-	m_nBuffer	= 0;
+	m_nRecords = 0;
+	m_nBuffer  = 0;
 
 	return( true );
 }
 
 //---------------------------------------------------------
-bool CSG_Table::Set_Count(int nRecords)
+bool CSG_Table::Set_Count(sLong nRecords)
 {
 	if( m_nRecords < nRecords )
 	{
@@ -919,21 +941,13 @@ bool CSG_Table::Set_Count(int nRecords)
 	return( m_nRecords == nRecords );
 }
 
-//---------------------------------------------------------
-bool CSG_Table::Set_Record_Count(int nRecords)
-{
-	return( Set_Count(nRecords) );
-}
-
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CSG_Table::Find_Record(int &iRecord, int iField, const CSG_String &Value, bool bCreateIndex)
+bool CSG_Table::Find_Record(sLong &iRecord, int iField, const CSG_String &Value, bool bCreateIndex)
 {
 	if( iField < 0 || iField >= m_nFields || m_nRecords < 1 )
 	{
@@ -965,36 +979,34 @@ bool CSG_Table::Find_Record(int &iRecord, int iField, const CSG_String &Value, b
 	//-----------------------------------------------------
 	else	// indexed search
 	{
-		#define GET_RECORD(i)	Get_Record_byIndex(bAscending ? (iRecord = i) : m_nRecords - 1 - (iRecord = i))
+		#define GET_RECORD(i) Get_Record_byIndex(bAscending ? (iRecord = i) : m_nRecords - 1 - (iRecord = i))
 
-		bool	bAscending	= Get_Index_Order(0) == TABLE_INDEX_Ascending;
-
-		double	d;
+		double d; bool bAscending = Get_Index_Order(0) == TABLE_INDEX_Ascending;
 
 		if( (d = Value.Cmp(GET_RECORD(0             )->asString(iField))) < 0 ) { return( false ); } else if( d == 0 ) { return( true ); }
 		if( (d = Value.Cmp(GET_RECORD(m_nRecords - 1)->asString(iField))) > 0 ) { return( false ); } else if( d == 0 ) { return( true ); }
 
-		for(int a=0, b=m_nRecords-1; b-a > 1; )
+		for(sLong a=0, b=m_nRecords-1; b-a > 1; )
 		{
-			d	= Value.Cmp(GET_RECORD(a + (b - a) / 2)->asString(iField));
+			d = Value.Cmp(GET_RECORD(a + (b - a) / 2)->asString(iField));
 
-			if( d > 0.0 )
+			if( d > 0. )
 			{
-				a	= iRecord;
+				a = iRecord;
 			}
-			else if( d < 0.0 )
+			else if( d < 0. )
 			{
-				b	= iRecord;
+				b = iRecord;
 			}
 			else
 			{
-				iRecord	= Get_Record_byIndex(bAscending ? iRecord : m_nRecords - 1 - iRecord)->Get_Index();
+				iRecord = Get_Record_byIndex(bAscending ? iRecord : m_nRecords - 1 - iRecord)->Get_Index();
 
 				return( true );
 			}
 		}
 
-		iRecord	= Get_Record_byIndex(bAscending ? iRecord : m_nRecords - 1 - iRecord)->Get_Index();
+		iRecord = Get_Record_byIndex(bAscending ? iRecord : m_nRecords - 1 - iRecord)->Get_Index();
 	}
 
 	//-----------------------------------------------------
@@ -1004,7 +1016,7 @@ bool CSG_Table::Find_Record(int &iRecord, int iField, const CSG_String &Value, b
 //---------------------------------------------------------
 CSG_Table_Record * CSG_Table::Find_Record(int iField, const CSG_String &Value, bool bCreateIndex)
 {
-	int	iRecord;
+	sLong iRecord;
 
 	if( Find_Record(iRecord, iField, Value, bCreateIndex) )
 	{
@@ -1015,7 +1027,7 @@ CSG_Table_Record * CSG_Table::Find_Record(int iField, const CSG_String &Value, b
 }
 
 //---------------------------------------------------------
-bool CSG_Table::Find_Record(int &iRecord, int iField, double Value, bool bCreateIndex)
+bool CSG_Table::Find_Record(sLong &iRecord, int iField, double Value, bool bCreateIndex)
 {
 	if( iField < 0 || iField >= m_nFields || m_nRecords < 1 )
 	{
@@ -1049,34 +1061,32 @@ bool CSG_Table::Find_Record(int &iRecord, int iField, double Value, bool bCreate
 	{
 		#define GET_RECORD(i)	Get_Record_byIndex(bAscending ? (iRecord = i) : m_nRecords - 1 - (iRecord = i))
 
-		bool	bAscending	= Get_Index_Order(0) == TABLE_INDEX_Ascending;
-
-		double	d;
+		double d; bool bAscending = Get_Index_Order(0) == TABLE_INDEX_Ascending;
 
 		if( (d = Value - GET_RECORD(0             )->asDouble(iField)) < 0. ) { return( false ); } else if( d == 0. ) { return( true ); }
 		if( (d = Value - GET_RECORD(m_nRecords - 1)->asDouble(iField)) > 0. ) { return( false ); } else if( d == 0. ) { return( true ); }
 
-		for(int a=0, b=m_nRecords-1; b-a > 1; )
+		for(sLong a=0, b=m_nRecords-1; b-a > 1; )
 		{
-			d	= Value - GET_RECORD(a + (b - a) / 2)->asDouble(iField);
+			d = Value - GET_RECORD(a + (b - a) / 2)->asDouble(iField);
 
 			if( d > 0. )
 			{
-				a	= iRecord;
+				a = iRecord;
 			}
 			else if( d < 0. )
 			{
-				b	= iRecord;
+				b = iRecord;
 			}
 			else
 			{
-				iRecord	= Get_Record_byIndex(bAscending ? iRecord : m_nRecords - 1 - iRecord)->Get_Index();
+				iRecord = Get_Record_byIndex(bAscending ? iRecord : m_nRecords - 1 - iRecord)->Get_Index();
 
 				return( true );
 			}
 		}
 
-		iRecord	= Get_Record_byIndex(bAscending ? iRecord : m_nRecords - 1 - iRecord)->Get_Index();
+		iRecord = Get_Record_byIndex(bAscending ? iRecord : m_nRecords - 1 - iRecord)->Get_Index();
 	}
 
 	//-----------------------------------------------------
@@ -1086,7 +1096,7 @@ bool CSG_Table::Find_Record(int &iRecord, int iField, double Value, bool bCreate
 //---------------------------------------------------------
 CSG_Table_Record * CSG_Table::Find_Record(int iField, double Value, bool bCreateIndex)
 {
-	int	iRecord;
+	sLong iRecord;
 
 	if( Find_Record(iRecord, iField, Value, bCreateIndex) )
 	{
@@ -1113,7 +1123,7 @@ void CSG_Table::Set_Modified(bool bModified)
 		if( bModified == false )
 		{
 			#pragma omp parallel for
-			for(int iRecord=0; iRecord<m_nRecords; iRecord++)
+			for(sLong iRecord=0; iRecord<m_nRecords; iRecord++)
 			{
 				m_Records[iRecord]->Set_Modified(false);
 			}
@@ -1122,56 +1132,62 @@ void CSG_Table::Set_Modified(bool bModified)
 }
 
 //---------------------------------------------------------
-bool CSG_Table::Set_Value(int iRecord, int iField, const SG_Char  *Value)
+bool CSG_Table::Set_Value(sLong iRecord, int iField, const SG_Char  *Value)
 {
-	CSG_Table_Record	*pRecord;
-
-	if( iField >= 0 && iField < m_nFields && (pRecord = Get_Record(iRecord)) != NULL )
+	if( iField >= 0 && iField < m_nFields )
 	{
-		return( pRecord->Set_Value(iField, Value) );
+		CSG_Table_Record *pRecord = Get_Record(iRecord);
+
+		return( pRecord && pRecord->Set_Value(iField, Value) );
 	}
 
 	return( false );
 }
 
 //---------------------------------------------------------
-bool CSG_Table::Set_Value(int iRecord, int iField, double       Value)
+bool CSG_Table::Set_Value(sLong iRecord, int iField, double       Value)
 {
-	CSG_Table_Record	*pRecord;
-
-	if( iField >= 0 && iField < m_nFields && (pRecord = Get_Record(iRecord)) != NULL )
+	if( iField >= 0 && iField < m_nFields )
 	{
-		return( pRecord->Set_Value(iField, Value) );
+		CSG_Table_Record *pRecord = Get_Record(iRecord);
+
+		return( pRecord && pRecord->Set_Value(iField, Value) );
 	}
 
 	return( false );
 }
 
 //---------------------------------------------------------
-bool CSG_Table::Get_Value(int iRecord, int iField, CSG_String &Value) const
+bool CSG_Table::Get_Value(sLong iRecord, int iField, CSG_String &Value) const
 {
-	CSG_Table_Record	*pRecord;
-
-	if( iField >= 0 && iField < m_nFields && (pRecord = Get_Record(iRecord)) != NULL )
+	if( iField >= 0 && iField < m_nFields )
 	{
-		Value	= pRecord->asString(iField);
+		CSG_Table_Record *pRecord = Get_Record(iRecord);
 
-		return( true );
+		if( pRecord )
+		{
+			Value = pRecord->asString(iField);
+
+			return( true );
+		}
 	}
 
 	return( false );
 }
 
 //---------------------------------------------------------
-bool CSG_Table::Get_Value(int iRecord, int iField, double      &Value) const
+bool CSG_Table::Get_Value(sLong iRecord, int iField, double      &Value) const
 {
-	CSG_Table_Record	*pRecord;
-
-	if( iField >= 0 && iField < m_nFields && (pRecord = Get_Record(iRecord)) != NULL )
+	if( iField >= 0 && iField < m_nFields )
 	{
-		Value	= pRecord->asDouble(iField);
+		CSG_Table_Record *pRecord = Get_Record(iRecord);
 
-		return( true );
+		if( pRecord )
+		{
+			Value = pRecord->asDouble(iField);
+
+			return( true );
+		}
 	}
 
 	return( false );
@@ -1216,7 +1232,7 @@ bool CSG_Table::_Stats_Update(int iField) const
 		return( false );
 	}
 
-	CSG_Simple_Statistics	&Statistics	= *m_Field_Stats[iField];
+	CSG_Simple_Statistics &Statistics = *m_Field_Stats[iField];
 
 	if( Statistics.is_Evaluated() )
 	{
@@ -1225,15 +1241,15 @@ bool CSG_Table::_Stats_Update(int iField) const
 
 	if( Get_Max_Samples() > 0 && Get_Max_Samples() < Get_Count() )
 	{
-		double	d	= (double)Get_Count() / (double)Get_Max_Samples();
+		double d = (double)Get_Count() / (double)Get_Max_Samples();
 
 		for(double i=0; i<(double)Get_Count(); i+=d)
 		{
-			CSG_Table_Record	*pRecord	= m_Records[(int)i];
+			CSG_Table_Record *pRecord = m_Records[(sLong)i];
 
 			if( !pRecord->is_NoData(iField) )
 			{
-				Statistics	+= pRecord->asDouble(iField);
+				Statistics += pRecord->asDouble(iField);
 			}
 		}
 
@@ -1243,13 +1259,13 @@ bool CSG_Table::_Stats_Update(int iField) const
 	}
 	else
 	{
-		for(int i=0; i<Get_Count(); i++)
+		for(sLong i=0; i<Get_Count(); i++)
 		{
-			CSG_Table_Record	*pRecord	= m_Records[i];
+			CSG_Table_Record *pRecord = m_Records[i];
 
 			if( !pRecord->is_NoData(iField) )
 			{
-				Statistics	+= pRecord->asDouble(iField);
+				Statistics += pRecord->asDouble(iField);
 			}
 		}
 	}
@@ -1355,21 +1371,21 @@ bool CSG_Table::Toggle_Index(int iField)
 //---------------------------------------------------------
 void CSG_Table::_Index_Update(void)
 {
-	CSG_Array_Int	Fields;
+	CSG_Array_Int Fields;
 
-	bool	Ascending	= Get_Index_Order(0) != TABLE_INDEX_Descending;
+	bool Ascending = Get_Index_Order(0) != TABLE_INDEX_Descending;
 
-	for(size_t i=0; i<m_Index_Fields.Get_Size(); i++)
+	for(size_t i=0; i<m_Index_Fields.Get_uSize(); i++)
 	{
-		int	Field	= abs(m_Index_Fields[i]) - 1;
+		int Field = abs(m_Index_Fields[i]) - 1;
 
 		if( Ascending )
 		{
-			Fields	+= m_Index_Fields[i] > 0 ? Field : -Field;
+			Fields += m_Index_Fields[i] > 0 ? Field : -Field;
 		}
 		else
 		{
-			Fields	+= m_Index_Fields[i] < 0 ? Field : -Field;
+			Fields += m_Index_Fields[i] < 0 ? Field : -Field;
 		}
 	}
 
@@ -1388,7 +1404,7 @@ void CSG_Table::_Index_Update(void)
 class CSG_Table_Record_Compare_Field : public CSG_Index::CSG_Index_Compare
 {
 public:
-	CSG_Table_Record_Compare_Field(CSG_Table *pTable, int Field, bool Ascending)
+	CSG_Table_Record_Compare_Field(const CSG_Table *pTable, int Field, bool Ascending)
 	{
 		m_pTable	= pTable;
 		m_Field		= Field;
@@ -1402,15 +1418,15 @@ public:
 
 	bool				is_Okay		(void)	const	{	return( m_pTable != NULL );	}
 
-	virtual int			Compare		(const int _a, const int _b)
+	virtual int			Compare		(const sLong _a, const sLong _b)
 	{
-		int	a	= m_Ascending ? _a : _b;
-		int	b	= m_Ascending ? _b : _a;
+		sLong a = m_Ascending ? _a : _b;
+		sLong b = m_Ascending ? _b : _a;
 
 		switch( m_pTable->Get_Field_Type(m_Field) )
 		{
 		default: {
-			double	d	=
+			double d =
 				m_pTable->Get_Record(a)->asDouble(m_Field) -
 				m_pTable->Get_Record(b)->asDouble(m_Field);
 
@@ -1433,14 +1449,14 @@ private:
 
 	int					m_Field;
 
-	CSG_Table			*m_pTable;
+	const CSG_Table		*m_pTable;
 
 };
 
 //---------------------------------------------------------
-bool CSG_Table::Set_Index(CSG_Index &Index, int Field, bool bAscending)
+bool CSG_Table::Set_Index(CSG_Index &Index, int Field, bool bAscending) const
 {
-	CSG_Table_Record_Compare_Field	Compare(this, Field, bAscending);
+	CSG_Table_Record_Compare_Field Compare(this, Field, bAscending);
 
 	return( Compare.is_Okay() && Index.Create(Get_Count(), &Compare) );
 }
@@ -1454,12 +1470,12 @@ bool CSG_Table::Set_Index(CSG_Index &Index, int Field, bool bAscending)
 class CSG_Table_Record_Compare_Fields : public CSG_Index::CSG_Index_Compare
 {
 public:
-	CSG_Table_Record_Compare_Fields(CSG_Table *pTable, int Fields[], int nFields, bool Ascending)
+	CSG_Table_Record_Compare_Fields(const CSG_Table *pTable, int Fields[], int nFields, bool Ascending)
 	{
-		m_pTable	= pTable;
-		m_Fields	= Fields;
-		m_nFields	= nFields;
-		m_Ascending	= Ascending;
+		m_pTable    = pTable;
+		m_Fields    = Fields;
+		m_nFields   = nFields;
+		m_Ascending = Ascending;
 
 		for(int i=0; m_pTable && i<m_nFields; i++)
 		{
@@ -1472,34 +1488,34 @@ public:
 
 	bool				is_Okay		(void)	const	{	return( m_pTable != NULL );	}
 
-	virtual int			Compare		(const int _a, const int _b)
+	virtual int			Compare		(const sLong _a, const sLong _b)
 	{
 		int	Difference	= 0;
 
 		for(int i=0; !Difference && i<m_nFields; i++)
 		{
-			int	Field	= m_Fields[i];
+			int	Field = m_Fields[i];
 
-			bool	Ascending	= i == 0 ? m_Ascending : m_Ascending ? Field > 0 : Field < 0;
+			bool Ascending = i == 0 ? m_Ascending : m_Ascending ? Field > 0 : Field < 0;
 
-			int	a	= Ascending ? _a : _b;
-			int	b	= Ascending ? _b : _a;
+			sLong a = Ascending ? _a : _b;
+			sLong b = Ascending ? _b : _a;
 
-			Field	= abs(Field);
+			Field = abs(Field);
 
 			switch( m_pTable->Get_Field_Type(Field) )
 			{
 			default: {
-				double	d	=
+				double d =
 					m_pTable->Get_Record(a)->asDouble(Field) -
 					m_pTable->Get_Record(b)->asDouble(Field);
 
-				Difference	= d < 0. ? -1 : d > 0. ? 1 : 0;
+				Difference = d < 0. ? -1 : d > 0. ? 1 : 0;
 			}	break;
 
 			case SG_DATATYPE_String:
 			case SG_DATATYPE_Date  :
-				Difference	= SG_STR_CMP(
+				Difference = SG_STR_CMP(
 					m_pTable->Get_Record(a)->asString(Field),
 					m_pTable->Get_Record(b)->asString(Field)
 				);
@@ -1517,12 +1533,12 @@ private:
 
 	int					*m_Fields, m_nFields;
 
-	CSG_Table			*m_pTable;
+	const CSG_Table		*m_pTable;
 
 };
 
 //---------------------------------------------------------
-bool CSG_Table::Set_Index(CSG_Index &Index, int Fields[], int nFields, bool bAscending)
+bool CSG_Table::Set_Index(CSG_Index &Index, int Fields[], int nFields, bool bAscending) const
 {
 	CSG_Table_Record_Compare_Fields	Compare(this, Fields, nFields, bAscending);
 
@@ -1530,7 +1546,7 @@ bool CSG_Table::Set_Index(CSG_Index &Index, int Fields[], int nFields, bool bAsc
 }
 
 //---------------------------------------------------------
-bool CSG_Table::Set_Index(CSG_Index &Index, const CSG_Array_Int &Fields, bool bAscending)
+bool CSG_Table::Set_Index(CSG_Index &Index, const CSG_Array_Int &Fields, bool bAscending) const
 {
 	return( Set_Index(Index, Fields.Get_Array(), (int)Fields.Get_Size(), bAscending) );
 }
