@@ -62,8 +62,11 @@
 //---------------------------------------------------------
 #define GET_XML_CONTENT(XML, ID, DEFAULT, TRANSLATE)	(!XML(ID) ? CSG_String(DEFAULT) : !TRANSLATE ? XML[ID].Get_Content() : CSG_String(SG_Translate(XML[ID].Get_Content())))
 
-#define IS_TRUE_STRING(String)				(!String.CmpNoCase("true") || !String.CmpNoCase("1"))
-#define IS_TRUE_PROPERTY(Item, Property)	(Item.Cmp_Property(Property, "true", true) || Item.Cmp_Property(Property, "1"))
+#define IS_TRUE_STRING(String)           (!String.CmpNoCase("true") || !String.CmpNoCase("1"))
+#define IS_TRUE_PROPERTY(Item, Property) (Item.Cmp_Property(Property, "true", true) || Item.Cmp_Property(Property, "1"))
+
+#define Get_List_Count(p)   (p->asGridList() ? p->asGridList()->Get_Grid_Count() : p->asList() ? p->asList()->Get_Item_Count() : 0)
+#define Get_List_Item(p, i) (p->asGridList() ? p->asGridList()->Get_Grid     (i) : p->asList() ? p->asList()->Get_Item     (i) : NULL)
 
 
 ///////////////////////////////////////////////////////////
@@ -551,10 +554,10 @@ bool CSG_Tool_Chain::Data_Add(const CSG_String &ID, CSG_Parameter *pData)
 	}
 	else if( pData->is_DataObject_List() && pParameter->is_DataObject_List() )
 	{
-		for(int i=0; i<pData->asList()->Get_Data_Count(); i++)
+		for(int i=0; i<Get_List_Count(pData); i++)
 		{
-			pParameter->asList()->Add_Item(pData->asList()->Get_Data(i));
-			m_Data_Manager.Add            (pData->asList()->Get_Data(i));
+			pParameter->asList()->Add_Item(Get_List_Item(pData, i));
+			m_Data_Manager.Add            (Get_List_Item(pData, i));
 		}
 	}
 
@@ -600,9 +603,9 @@ bool CSG_Tool_Chain::Data_Del_Temp(const CSG_String &ID, bool bData)
 			}
 			else if( pData->is_DataObject_List() )
 			{
-				for(int i=0; i<pData->asList()->Get_Data_Count(); i++)
+				for(int i=0; i<Get_List_Count(pData); i++)
 				{
-					m_Data_Manager.Delete(pData->asList()->Get_Data(i));
+					m_Data_Manager.Delete(Get_List_Item(pData, i));
 				}
 			}
 		}
@@ -632,9 +635,9 @@ bool CSG_Tool_Chain::Data_Exists(CSG_Data_Object *pData)
 		}
 		else if( m_Data(i)->is_DataObject_List() )
 		{
-			for(int j=0; j<m_Data(i)->asList()->Get_Data_Count(); j++)
+			for(int j=0; j<Get_List_Count(m_Data(i)); j++)
 			{
-				if( pData == m_Data(i)->asList()->Get_Data(j) )
+				if( pData == Get_List_Item(m_Data(i), j) )
 				{
 					return( true );
 				}
@@ -687,15 +690,15 @@ bool CSG_Tool_Chain::Data_Finalize(void)
 			{
 				CSG_Parameter	*pData	= m_Data(Parameters(i)->Get_Identifier());
 
-				for(int j=0; j<pData->asList()->Get_Data_Count(); j++)	// csg_parameter::assign() will not work, if parameters data manager is the standard data manager because it checks for existing data sets
+				for(int j=0; j<Get_List_Count(pData); j++)	// csg_parameter::assign() will not work, if parameters data manager is the standard data manager because it checks for existing data sets
 				{
-					Parameters(i)->asList()->Add_Item(pData->asList()->Get_Data(j));
+					Parameters(i)->asList()->Add_Item(Get_List_Item(pData, j));
 				}
 			}
 
-			for(int j=0; j<Parameters(i)->asList()->Get_Data_Count(); j++)
+			for(int j=0; j<Get_List_Count(Parameters(i)); j++)
 			{
-				m_Data_Manager.Delete(Parameters(i)->asList()->Get_Data(j), true);
+				m_Data_Manager.Delete(Get_List_Item(Parameters(i), j), true);
 			}
 		}
 	}
@@ -816,7 +819,7 @@ bool CSG_Tool_Chain::Check_Condition(const CSG_MetaData &Condition, CSG_Paramete
 	{
 		CSG_Parameter	*pParameter	= (*pData)(Variable);
 
-		return( pParameter && ((pParameter->is_DataObject() && pParameter->asDataObject()) || (pParameter->is_DataObject_List() && pParameter->asList()->Get_Data_Count())) );
+		return( pParameter && ((pParameter->is_DataObject() && pParameter->asDataObject()) || (pParameter->is_DataObject_List() && pParameter->asList()->Get_Item_Count())) );
 	}
 
 	if( !Type.CmpNoCase("not_exists") )	// data object does not exist
@@ -1041,7 +1044,7 @@ bool CSG_Tool_Chain::ForEach_Object(const CSG_MetaData &Commands, const CSG_Stri
 
 	if( pList->is_DataObject_List() )
 	{
-		for(int iObject=0; bResult && iObject<pList->asList()->Get_Data_Count(); iObject++)
+		for(int iObject=0; bResult && iObject<Get_List_Count(pList->asList()); iObject++)
 		{
 			for(int iTool=0; bResult && iTool<Commands.Get_Children_Count(); iTool++)
 			{
@@ -1141,7 +1144,7 @@ bool CSG_Tool_Chain::ForEach_File(const CSG_MetaData &Commands, const CSG_String
 
 				bResult	= Tool_Run(Tool, bIgnoreErrors);
 
-				for(size_t i=0; i<Input.Get_Size(); i++)
+				for(size_t i=0; i<Input.Get_uSize(); i++)
 				{
 					Tool(Input[i])->Set_Content(ListVarName);
 					Tool(Input[i])->Set_Property("varname", "true");
@@ -1375,9 +1378,9 @@ bool CSG_Tool_Chain::Tool_Initialize(const CSG_MetaData &Tool, CSG_Tool *pTool)
 	//-----------------------------------------------------
 	for(int i=0; i<Tool.Get_Children_Count(); i++)	// check for invalid parameters...
 	{
-		const CSG_MetaData	&Parameter	= Tool[i];	if( Parameter.Cmp_Name("comment") )	{	continue;	}
+		const CSG_MetaData &Parameter = Tool[i]; if( Parameter.Cmp_Name("comment") ) { continue; }
 
-		CSG_Parameter	*pParameter, *pOwner;
+		CSG_Parameter *pParameter, *pOwner;
 
 		if( !Tool_Get_Parameter(Parameter, pTool, &pParameter, &pOwner) )
 		{
@@ -1387,146 +1390,166 @@ bool CSG_Tool_Chain::Tool_Initialize(const CSG_MetaData &Tool, CSG_Tool *pTool)
 		}
 	}
 
+	//-----------------------------------------------------
+	if( Tool.Get_Property("grid_system") )
+	{
+		CSG_Parameter *pParameter = Parameters(Tool.Get_Property("grid_system"));
+
+		if( pParameter && pParameter->asGrid_System() )
+		{
+			if( !pTool->Set_Grid_System(*pParameter->asGrid_System()) )
+			{
+				Message_Fmt("\n%s: %s\n", _TL("Warning"), _TL("failed to set tool's grid system"));
+			}
+		}
+	}
+
+	//-----------------------------------------------------
+	for(int i=0; i<Tool.Get_Children_Count(); i++)	// initialize all options first, some might en-/disable mandatory input data
+	{
+		const CSG_MetaData &Parameter = Tool[i]; if( !Parameter.Cmp_Name("option") ) { continue; }
+
+		CSG_Parameter *pParameter, *pOwner; Tool_Get_Parameter(Parameter, pTool, &pParameter, &pOwner);
+
+		if( IS_TRUE_PROPERTY(Parameter, "varname")
+			? pParameter->Set_Value(Parameters(Parameter.Get_Content()))
+			: pParameter->Set_Value(           Parameter.Get_Content() ) )
+		{
+			pParameter->has_Changed(); if( pOwner ) { pOwner->has_Changed(); }
+		}
+	}
+
 	//-----------------------------------------------------
 	for(int i=0; i<Tool.Get_Children_Count(); i++)	// set data input first
 	{
-		const CSG_MetaData	&Parameter	= Tool[i];	if( Parameter.Cmp_Name("comment") )	{	continue;	}
+		const CSG_MetaData &Parameter = Tool[i]; if( !Parameter.Cmp_Name("input") ) { continue; }
 
-		if( Parameter.Cmp_Name("input") )
+		CSG_Parameter *pParameter, *pOwner; Tool_Get_Parameter(Parameter, pTool, &pParameter, &pOwner);
+
+		int	Index;
+
+		if( Parameter.Get_Content().Find('[') < 1 || !Parameter.Get_Content().AfterFirst('[').asInt(Index) )
 		{
-			CSG_Parameter	*pParameter, *pOwner;	Tool_Get_Parameter(Parameter, pTool, &pParameter, &pOwner);
+			Index = -1;
+		}
 
-			int	Index;
+		bool bResult = false;
 
-			if( Parameter.Get_Content().Find('[') < 1 || !Parameter.Get_Content().AfterFirst('[').asInt(Index) )
+		CSG_Parameter *pData = m_Data(Index < 0 ? Parameter.Get_Content() : Parameter.Get_Content().BeforeFirst('['));
+
+		if( pData && (pParameter->is_DataObject() || pParameter->is_DataObject_List()) )
+		{
+			if( Index >= 0 && (pData->is_DataObject_List() || pData->asGrids()) )
 			{
-				Index	= -1;
-			}
+				CSG_Data_Object	*pObject = pData->is_DataObject_List()
+					? pData->asList ()->Get_Item    (Index)
+					: pData->asGrids()->Get_Grid_Ptr(Index);
 
-			bool	bResult	= false;
-
-			CSG_Parameter	*pData	= m_Data(Index < 0 ? Parameter.Get_Content() : Parameter.Get_Content().BeforeFirst('['));
-
-			if( pData && (pParameter->is_DataObject() || pParameter->is_DataObject_List()) )
-			{
-				if( Index >= 0 && (pData->is_DataObject_List() || pData->asGrids()) )
+				if( pParameter->is_DataObject() )
 				{
-					CSG_Data_Object	*pObject	= pData->is_DataObject_List()
-						? pData->asList ()->Get_Item    (Index)
-						: pData->asGrids()->Get_Grid_Ptr(Index);
-
-					if( pParameter->is_DataObject() )
-					{
-						bResult	= pParameter->Set_Value(pObject);
-					}
-					if( pParameter->is_DataObject_List() )
-					{
-						bResult	= pParameter->asList()->Add_Item(pObject);
-					}
+					bResult = pParameter->Set_Value(pObject);
 				}
-				else if( pParameter->Get_Type() == pData->Get_Type() )
+				if( pParameter->is_DataObject_List() )
 				{
-					if( pParameter->is_DataObject_List() && pParameter->asList()->Get_Item_Count() > 0 )
-					{
-						bResult	= true;
-
-						for(int i=0; bResult && i<pData->asList()->Get_Item_Count(); i++)
-						{
-							bResult	= pParameter->asList()->Add_Item(pData->asList()->Get_Item(i));
-						}
-					}
-					else
-					{
-						bResult	= pParameter->Assign(pData);
-					}
-				}
-				else if( pParameter->is_DataObject_List() && pData->is_DataObject() )
-				{
-					bResult	= pParameter->asList()->Add_Item(pData->asDataObject());
-				}
-				else if( Parameter_isCompatible(pParameter->Get_Type(), pData->Get_Type()) )
-				{
-					bResult = pParameter->Set_Value(pData->asDataObject());
+					bResult = pParameter->asList()->Add_Item(pObject);
 				}
 			}
-
-			if( !bResult )
+			else if( pParameter->Get_Type() == pData->Get_Type() )
 			{
-				Error_Fmt("%s: %s", _TL("set input"), Parameter.Get_Property("id"));
+				if( pParameter->is_DataObject_List() && pParameter->asList()->Get_Item_Count() > 0 )
+				{
+					bResult = true;
 
-				return( false );
+					for(int i=0; bResult && i<pData->asList()->Get_Item_Count(); i++)
+					{
+						bResult = pParameter->asList()->Add_Item(pData->asList()->Get_Item(i));
+					}
+				}
+				else
+				{
+					bResult = pParameter->Assign(pData);
+				}
 			}
-
-			pParameter->has_Changed();
-
-			if( pOwner )
+			else if( pParameter->is_DataObject_List() && pData->is_DataObject() )
 			{
-				pOwner->has_Changed();
+				bResult = pParameter->asList()->Add_Item(pData->asDataObject());
 			}
+			else if( Parameter_isCompatible(pParameter->Get_Type(), pData->Get_Type()) )
+			{
+				bResult = pParameter->Set_Value(pData->asDataObject());
+			}
+		}
+
+		if( bResult )
+		{
+			pParameter->has_Changed(); if( pOwner ) { pOwner->has_Changed(); }
+		}
+		else if( pParameter->is_Enabled() )
+		{
+			Error_Fmt("%s: %s", _TL("set input"), Parameter.Get_Property("id"));
+
+			return( false );
 		}
 	}
 
 	//-----------------------------------------------------
 	for(int i=0; i<Tool.Get_Children_Count(); i++)	// now set all options
 	{
-		const CSG_MetaData	&Parameter	= Tool[i];	if( Parameter.Cmp_Name("comment") )	{	continue;	}
+		const CSG_MetaData &Parameter = Tool[i]; if( !Parameter.Cmp_Name("option") ) { continue; }
 
-		if( Parameter.Cmp_Name("option") )
-		{
-			CSG_Parameter	*pParameter, *pOwner;	Tool_Get_Parameter(Parameter, pTool, &pParameter, &pOwner);
+		CSG_Parameter *pParameter, *pOwner; Tool_Get_Parameter(Parameter, pTool, &pParameter, &pOwner);
 
-			if( IS_TRUE_PROPERTY(Parameter, "varname") )
-			{	// does option want a value from tool chain parameters and do these provide one ?
-				switch( pParameter->Get_Type() )
-				{
-				default:
-					pParameter->Set_Value(Parameters(Parameter.Get_Content()));
-					break;
-
-				case PARAMETER_TYPE_FixedTable:
-					if( m_Data(Parameter.Get_Content()) && pParameter->asTable()->Assign_Values(m_Data(Parameter.Get_Content())->asTable()) )
-					{
-						pParameter->has_Changed();
-					}
-					break;
-				}
-			}
-			else
+		if( IS_TRUE_PROPERTY(Parameter, "varname") )
+		{	// does option want a value from tool chain parameters and do these provide one ?
+			switch( pParameter->Get_Type() )
 			{
-				switch( pParameter->Get_Type() )
+			default:
+				pParameter->Set_Value(Parameters(Parameter.Get_Content()));
+				break;
+
+			case PARAMETER_TYPE_FixedTable:
+				if( m_Data(Parameter.Get_Content()) && pParameter->asTable()->Assign_Values(m_Data(Parameter.Get_Content())->asTable()) )
 				{
-				case PARAMETER_TYPE_FixedTable:
-					if( Parameter("OPTION") )
-					{
-						pParameter->Serialize(*Parameter("OPTION"), false);
-					}
-					break;
+					pParameter->has_Changed();
+				}
+				break;
+			}
+		}
+		else
+		{
+			switch( pParameter->Get_Type() )
+			{
+			case PARAMETER_TYPE_FixedTable:
+				if( Parameter("OPTION") )
+				{
+					pParameter->Serialize(*Parameter("OPTION"), false);
+				}
+				break;
 
-				default: {
-					CSG_String	Value(Parameter.Get_Content());
+			default: {
+				CSG_String	Value(Parameter.Get_Content());
 
-					if( Value.Find("$(") >= 0 )
+				if( Value.Find("$(") >= 0 )
+				{
+					for(int j=0; j<Parameters.Get_Count(); j++)
 					{
-						for(int j=0; j<Parameters.Get_Count(); j++)
+						CSG_String	Var; Var.Printf("$(%s)", Parameters(j)->Get_Identifier());
+
+						if( Value.Find(Var) >= 0 )
 						{
-							CSG_String	Var; Var.Printf("$(%s)", Parameters(j)->Get_Identifier());
-
-							if( Value.Find(Var) >= 0 )
-							{
-								Value.Replace(Var, Parameters(j)->asString());
-							}
+							Value.Replace(Var, Parameters(j)->asString());
 						}
 					}
-
-					pParameter->Set_Value(Value);
-
-					if( pOwner )
-					{
-						pOwner->has_Changed();
-					}
-
-					break; }
 				}
+
+				pParameter->Set_Value(Value);
+
+				if( pOwner )
+				{
+					pOwner->has_Changed();
+				}
+
+				break; }
 			}
 		}
 	}
@@ -1534,26 +1557,23 @@ bool CSG_Tool_Chain::Tool_Initialize(const CSG_MetaData &Tool, CSG_Tool *pTool)
 	//-----------------------------------------------------
 	for(int i=0; i<Tool.Get_Children_Count(); i++)	// finally set the data output
 	{
-		const CSG_MetaData	&Parameter	= Tool[i];	if( Parameter.Cmp_Name("comment") )	{	continue;	}
+		const CSG_MetaData &Parameter = Tool[i]; if( !Parameter.Cmp_Name("output") ) { continue; }
 
-		if( Parameter.Cmp_Name("output") )
+		CSG_Parameter *pParameter, *pOwner; Tool_Get_Parameter(Parameter, pTool, &pParameter, &pOwner);
+
+		if( !pParameter->Assign(m_Data(Parameter.Get_Content())) )
 		{
-			CSG_Parameter	*pParameter, *pOwner;	Tool_Get_Parameter(Parameter, pTool, &pParameter, &pOwner);
-
-			if( !pParameter->Assign(m_Data(Parameter.Get_Content())) )
+			if( pParameter->is_DataObject() )
 			{
-				if( pParameter->is_DataObject() )
-				{
-					pParameter->Set_Value(DATAOBJECT_CREATE);
-				}
-				else if( pParameter->is_DataObject_List() )
-				{
-					pParameter->asList()->Del_Items();
-				}
-				else if( pParameter->asGrids() )
-				{
-					pParameter->asGrids()->Del_Grids();
-				}
+				pParameter->Set_Value(DATAOBJECT_CREATE);
+			}
+			else if( pParameter->is_DataObject_List() )
+			{
+				pParameter->asList()->Del_Items();
+			}
+			else if( pParameter->asGrids() )
+			{
+				pParameter->asGrids()->Del_Grids();
 			}
 		}
 	}
@@ -1605,11 +1625,11 @@ bool CSG_Tool_Chain::Tool_Finalize(const CSG_MetaData &Tool, CSG_Tool *pTool)
 				}
 				else if( pParameter->is_DataObject_List() )
 				{
-					for(int k=0; k<pParameter->asList()->Get_Data_Count(); k++)
+					for(int k=0; k<Get_List_Count(pParameter); k++)
 					{
-						if( !Data_Exists(pParameter->asList()->Get_Data(k)) )
+						if( !Data_Exists(Get_List_Item(pParameter, k)) )
 						{
-							m_Data_Manager.Delete(pParameter->asList()->Get_Data(k));
+							m_Data_Manager.Delete(Get_List_Item(pParameter, k));
 						}
 					}
 				}
@@ -1962,7 +1982,7 @@ CSG_Tool_Chains::~CSG_Tool_Chains(void)
 {
 	Delete_Tools();
 
-	for(size_t i=0; i<m_Tools.Get_Size(); i++)
+	for(size_t i=0; i<m_Tools.Get_uSize(); i++)
 	{
 		delete((CSG_Tool_Chain *)m_Tools[i]);
 	}
@@ -2041,7 +2061,7 @@ bool CSG_Tool_Chains::Delete_Tool(CSG_Tool *pTool)
 //---------------------------------------------------------
 bool CSG_Tool_Chains::Delete_Tools(void)
 {
-	for(size_t i=0; i<m_xTools.Get_Size(); i++)
+	for(size_t i=0; i<m_xTools.Get_uSize(); i++)
 	{
 		delete((CSG_Tool_Chain *)m_xTools[i]);
 	}
