@@ -88,7 +88,7 @@
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-typedef enum ESG_Tool_Library_Type
+typedef enum
 {
 	TOOL_LIBRARY		= 0,
 	TOOL_CHAINS
