@@ -128,7 +128,7 @@ void CCMD_Tool::Usage(void)
 {
 	if( !m_Usage.is_Empty() )
 	{
-		SG_Printf(m_Usage);
+		CMD_Print(m_Usage);
 	}
 }
 
@@ -297,7 +297,7 @@ bool CCMD_Tool::_has_Unused(void)
 
 	if( !Unused.is_Empty() )
 	{
-		SG_Printf("%s: %s!\n[%s]\n\n", _TL("Error"), _TL("The tool does not know some of the provided arguments!"), Unused.c_str());
+		CMD_Print(CSG_String::Format("%s: %s!\n[%s]\n\n", _TL("Error"), _TL("The tool does not know some of the provided arguments!"), Unused.c_str()));
 
 		return( true );
 	}
@@ -359,7 +359,7 @@ bool CCMD_Tool::_Found(const CSG_String &Name, CSG_DateTime &Value)
 {
 	CSG_String Argument;
 	
-	return( _Found(Name, Argument) && Value.Parse_DateTime(Argument) );
+	return( _Found(Name, Argument) && Value.Parse_Date(Argument) );
 }
 
 
@@ -530,7 +530,9 @@ bool CCMD_Tool::_Get_Options(CSG_Parameters *pParameters, bool bInitialize)
 
 		else if( pParameter->is_Output() )
 		{
-			if( pParameter->is_DataObject() && pParameter->is_Optional() && !pParameter->asDataObject() && _Found(ID) )
+			CSG_String FileName; // only create optional output if a valid output file name is given!
+
+			if( pParameter->is_DataObject() && pParameter->is_Optional() && !pParameter->asDataObject() && _Found(ID, FileName) && FileName.Length() > 0 )
 			{
 				pParameter->Set_Value(DATAOBJECT_CREATE);
 			}
