@@ -88,8 +88,8 @@ CSG_3DView_Canvas::CSG_3DView_Canvas(void)
 	m_BoxBuffer   = 0.01;
 	m_bStereo     = false;
 	m_dStereo     = 1.;
-	m_bNorth      = false;
-
+	m_North       = 1;
+	m_North_Size  = 15.;
 	m_Labels      = 0;
 	m_Label_Res   = 50;
 	m_Label_Scale = 1.;
@@ -132,7 +132,7 @@ bool CSG_3DView_Canvas::Draw(void)
 
 	_Draw_Background();
 
-	//-------------------------------------------------
+	//-----------------------------------------------------
 	if( m_Data_Min.x >= m_Data_Max.x
 	||  m_Data_Min.y >= m_Data_Max.y
 	||  m_Data_Min.z >  m_Data_Max.z )
@@ -142,7 +142,7 @@ bool CSG_3DView_Canvas::Draw(void)
 		return( false );
 	}
 
-	//-------------------------------------------------
+	//-----------------------------------------------------
 	if( !On_Before_Draw() )
 	{
 		bDrawing = false;
@@ -150,7 +150,7 @@ bool CSG_3DView_Canvas::Draw(void)
 		return( false );
 	}
 
-	//-------------------------------------------------
+	//-----------------------------------------------------
 	m_Projector.Set_Center( // rotation center set to data's center
 		m_Data_Min.x + (m_Data_Max.x - m_Data_Min.x) / 2.,
 		m_Data_Min.y + (m_Data_Max.y - m_Data_Min.y) / 2.,
@@ -159,12 +159,14 @@ bool CSG_3DView_Canvas::Draw(void)
 
 	m_Projector.Set_Scale(SG_Get_Length(m_Data_Max.x - m_Data_Min.x, m_Data_Max.y - m_Data_Min.y));
 
-	//-------------------------------------------------
+	int Front = _Draw_Get_Box_Front();
+
+	//-----------------------------------------------------
 	if( m_bStereo == false )
 	{
 		m_Color_Mode = COLOR_MODE_RGB;
 
-		m_Image_zMax.Assign(999999.); On_Draw(); _Draw_Box(); _Draw_Labels();
+		m_Image_zMax.Assign(999999.); On_Draw(); _Draw_Box(); _Draw_Labels(Front); _Draw_North();
 	}
 
 	//-----------------------------------------------------
@@ -180,7 +182,7 @@ bool CSG_3DView_Canvas::Draw(void)
 
 		m_Color_Mode = COLOR_MODE_RED;
 
-		m_Image_zMax.Assign(999999.); On_Draw(); _Draw_Box(); _Draw_Labels();
+		m_Image_zMax.Assign(999999.); On_Draw(); _Draw_Box(); _Draw_Labels(Front); _Draw_North();
 
 		//-------------------------------------------------
 		m_Projector.Set_yRotation(ry + dy);
@@ -188,7 +190,7 @@ bool CSG_3DView_Canvas::Draw(void)
 
 		m_Color_Mode = COLOR_MODE_CYAN;
 
-		m_Image_zMax.Assign(999999.); On_Draw(); _Draw_Box(); _Draw_Labels();
+		m_Image_zMax.Assign(999999.); On_Draw(); _Draw_Box(); _Draw_Labels(Front); _Draw_North();
 
 		//-------------------------------------------------
 		m_Projector.Set_yRotation(ry);
@@ -241,9 +243,9 @@ void CSG_3DView_Canvas::_Draw_Background(void)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-void CSG_3DView_Canvas::_Draw_Get_Box(TSG_Point_Z Box[8], bool bProjected)
+void CSG_3DView_Canvas::_Draw_Get_Box(TSG_Point_3D Box[8], bool bProjected)
 {
-	TSG_Point_Z	Buffer;
+	TSG_Point_3D	Buffer;
 
 	Buffer.x = m_BoxBuffer * (m_Data_Max.x - m_Data_Min.x);
 	Buffer.y = m_BoxBuffer * (m_Data_Max.y - m_Data_Min.y);
@@ -265,6 +267,22 @@ void CSG_3DView_Canvas::_Draw_Get_Box(TSG_Point_Z Box[8], bool bProjected)
 	}
 }
 
+//---------------------------------------------------------
+int CSG_3DView_Canvas::_Draw_Get_Box_Front(void)
+{
+	int Front = 0; TSG_Point_3D b[8]; _Draw_Get_Box(b, true);
+
+	for(int i=1; i<4; i++)
+	{
+		if( b[i].z < b[Front].z )
+		{
+			Front = i;
+		}
+	}
+
+	return( Front );
+}
+
 //---------------------------------------------------------
 void CSG_3DView_Canvas::_Draw_Box(void)
 {
@@ -275,7 +293,7 @@ void CSG_3DView_Canvas::_Draw_Box(void)
 
 	int Color = SG_GET_RGB(SG_GET_R(m_bgColor) + 128, SG_GET_G(m_bgColor) + 128, SG_GET_B(m_bgColor) + 128);
 
-	TSG_Point_Z	Box[8]; _Draw_Get_Box(Box, true);
+	TSG_Point_3D	Box[8]; _Draw_Get_Box(Box, true);
 
 	for(int i=0; i<8; i+=4)
 	{
@@ -292,6 +310,85 @@ void CSG_3DView_Canvas::_Draw_Box(void)
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+void CSG_3DView_Canvas::_Draw_North(void)
+{
+	if( !m_North )
+	{
+		return;
+	}
+
+	const double Arrow[9][2] = { { 0., 0. }, { 0., 1. }, { 0., -0.5 }, { -0.5, -1. }, { 0.5, -1. }, { -1, -1. }, { 1, -1. }, { 1, 1. }, { -1, 1. } };
+
+	TSG_Point_3D A[9]; CSG_Rect r; double Scale = M_GET_MIN(m_Data_Max.x - m_Data_Min.x, m_Data_Max.y - m_Data_Min.y) / 2.;
+
+	for(int i=0; i<9; i++)
+	{
+		A[i].x = m_Projector.Get_xCenter() + Arrow[i][0] * Scale;
+		A[i].y = m_Projector.Get_yCenter() + Arrow[i][1] * Scale;
+		A[i].z = m_Data_Min.z;
+
+		m_Projector.Get_Projection(A[i]);
+
+		if( i == 2 )
+		{
+			r.Assign(A[1].x, A[1].y, A[2].x, A[2].y);
+		}
+		else if( i > 2 )
+		{
+			r.Union(CSG_Point(A[i].x, A[i].y));
+		}
+	}
+
+	double Size = m_North_Size * M_GET_MIN(m_Image_NX, m_Image_NY) / 100.;
+
+	A[0].z = Size / r.Get_Diameter();
+
+	for(int i=1; i<9; i++)
+	{
+		A[i].x = (A[i].x - A[0].x) * A[0].z +              0.6 * Size;
+		A[i].y = (A[i].y - A[0].y) * A[0].z + m_Image_NY - 0.6 * Size;
+		A[i].z = 0.;
+	}
+
+	Draw_Line(A[1], A[2], SG_COLOR_BLACK);
+	Draw_Line(A[1], A[3], SG_COLOR_BLACK);
+	Draw_Line(A[1], A[4], SG_COLOR_BLACK);
+	Draw_Line(A[2], A[3], SG_COLOR_BLACK);
+	Draw_Line(A[2], A[4], SG_COLOR_BLACK);
+
+	if( m_North == 1 )
+	{
+		int Color = SG_GET_RGB(SG_GET_R(m_bgColor) + 128, SG_GET_G(m_bgColor) + 128, SG_GET_B(m_bgColor) + 128);
+
+		Draw_Line(A[5], A[6], Color);
+		Draw_Line(A[6], A[7], Color);
+		Draw_Line(A[7], A[8], Color);
+		Draw_Line(A[8], A[5], Color);
+	}
+
+	TSG_Triangle_Node p[3]; CSG_Grid *pDrape = m_pDrape; m_pDrape = NULL;
+
+	#define SET_NODE(n, i, color) { p[n].x = A[i].x; p[n].y = A[i].y; p[n].z = A[i].z; p[n].c = color; }
+
+	SET_NODE(0, 1, SG_COLOR_BLACK);
+	SET_NODE(1, 2, SG_COLOR_BLACK);
+	SET_NODE(2, 3, SG_COLOR_BLACK);
+	Draw_Triangle(p, true);
+
+	SET_NODE(0, 1, SG_COLOR_WHITE);
+	SET_NODE(1, 2, SG_COLOR_WHITE);
+	SET_NODE(2, 4, SG_COLOR_WHITE);
+	Draw_Triangle(p, true);
+
+	m_pDrape = pDrape;
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 ///////////////////////////////////////////////////////////
@@ -300,16 +397,14 @@ void CSG_3DView_Canvas::_Draw_Box(void)
 #define LABEL_SCALE 50.
 
 //---------------------------------------------------------
-void CSG_3DView_Canvas::_Draw_Labels(void)
+void CSG_3DView_Canvas::_Draw_Labels(int Front)
 {
 	if( m_Labels == 2 ) // none
 	{
 		return;
 	}
 
-	TSG_Point_Z Box[8]; _Draw_Get_Box(Box, false);
-
-	int Front = 0; { TSG_Point_Z b[8]; _Draw_Get_Box(b, true); for(int j=1; j<4; j++) { if( b[j].z < b[Front].z ) Front = j; } }
+	TSG_Point_3D Box[8]; _Draw_Get_Box(Box, false);
 
 	switch( m_Labels )
 	{
@@ -320,24 +415,28 @@ void CSG_3DView_Canvas::_Draw_Labels(void)
 			_Draw_Labels(0, Box[0], Box[1],   0,   0,   0, LABEL_ALIGN_LEFT , m_Label_Res, m_Label_Scale);
 			_Draw_Labels(1, Box[3], Box[0], 270,   0,   0, LABEL_ALIGN_RIGHT, m_Label_Res, m_Label_Scale);
 			_Draw_Labels(2, Box[3], Box[7], 180,  90, 270, LABEL_ALIGN_RIGHT, m_Label_Res, m_Label_Scale);
+			_Draw_Labels(2, Box[1], Box[5], 180, 270, 180, LABEL_ALIGN_LEFT , m_Label_Res, m_Label_Scale);
 			break;
 
 		case  1:
 			_Draw_Labels(0, Box[0], Box[1],   0,   0,   0, LABEL_ALIGN_RIGHT, m_Label_Res, m_Label_Scale);
 			_Draw_Labels(1, Box[1], Box[2],  90,   0,   0, LABEL_ALIGN_LEFT , m_Label_Res, m_Label_Scale);
 			_Draw_Labels(2, Box[0], Box[4], 180,  90, 180, LABEL_ALIGN_RIGHT, m_Label_Res, m_Label_Scale);
+			_Draw_Labels(2, Box[2], Box[6], 180, 270, 270, LABEL_ALIGN_LEFT , m_Label_Res, m_Label_Scale);
 			break;
 
 		case  2:
 			_Draw_Labels(0, Box[2], Box[3], 180,   0,   0, LABEL_ALIGN_LEFT , m_Label_Res, m_Label_Scale);
 			_Draw_Labels(1, Box[1], Box[2],  90,   0,   0, LABEL_ALIGN_RIGHT, m_Label_Res, m_Label_Scale);
 			_Draw_Labels(2, Box[1], Box[5], 180,  90,  90, LABEL_ALIGN_RIGHT, m_Label_Res, m_Label_Scale);
+			_Draw_Labels(2, Box[3], Box[7], 180, 270,   0, LABEL_ALIGN_LEFT , m_Label_Res, m_Label_Scale);
 			break;
 
 		case  3:
 			_Draw_Labels(0, Box[2], Box[3], 180,   0,   0, LABEL_ALIGN_RIGHT, m_Label_Res, m_Label_Scale);
 			_Draw_Labels(1, Box[3], Box[0], 270,   0,   0, LABEL_ALIGN_LEFT , m_Label_Res, m_Label_Scale);
 			_Draw_Labels(2, Box[2], Box[6], 180,  90,   0, LABEL_ALIGN_RIGHT, m_Label_Res, m_Label_Scale);
+			_Draw_Labels(2, Box[0], Box[4], 180, 270,  90, LABEL_ALIGN_LEFT , m_Label_Res, m_Label_Scale);
 			break;
 		}
 		break;
@@ -352,7 +451,7 @@ void CSG_3DView_Canvas::_Draw_Labels(void)
 }
 
 //---------------------------------------------------------
-void CSG_3DView_Canvas::_Draw_Labels(double Min, double Max, const TSG_Point_Z &Point, double Rx, double Ry, double Rz, int Resolution, double Scale)
+void CSG_3DView_Canvas::_Draw_Labels(double Min, double Max, const TSG_Point_3D &Point, double Rx, double Ry, double Rz, int Resolution, double Scale)
 {
 	bool bAscending = Max > Min; if( !bAscending ) { double val = Min; Min = Max; Max = val; }
 
@@ -382,7 +481,7 @@ void CSG_3DView_Canvas::_Draw_Labels(double Min, double Max, const TSG_Point_Z &
 }
 
 //---------------------------------------------------------
-void CSG_3DView_Canvas::_Draw_Labels(int Axis, const TSG_Point_Z &A, const TSG_Point_Z &B, double Rx, double Ry, double Rz, int Align, int Resolution, double Scale)
+void CSG_3DView_Canvas::_Draw_Labels(int Axis, const TSG_Point_3D &A, const TSG_Point_3D &B, double Rx, double Ry, double Rz, int Align, int Resolution, double Scale)
 {
 	double Min = Axis == 0 ? A.x : Axis == 1 ? A.y : A.z;
 	double Max = Axis == 0 ? B.x : Axis == 1 ? B.y : B.z;
@@ -423,7 +522,7 @@ void CSG_3DView_Canvas::_Draw_Labels(int Axis, const TSG_Point_Z &A, const TSG_P
 	}
 
 	//-----------------------------------------------------
-	CSG_Point_Z D(B.x - A.x, B.y - A.y, B.z - A.z);
+	CSG_Point_3D D(B.x - A.x, B.y - A.y, B.z - A.z);
 
 	for(double Value=Step*floor(Min/Step); !std::isinf(Value) && Value<=Max; Value+=Step)
 	{
@@ -432,14 +531,14 @@ void CSG_3DView_Canvas::_Draw_Labels(int Axis, const TSG_Point_Z &A, const TSG_P
 			continue;
 		}
 
-		double i = (Value - Min) / (Max - Min); CSG_Point_Z p(A.x + i * D.x, A.y + i * D.y, A.z + i * D.z);
+		double i = (Value - Min) / (Max - Min); CSG_Point_3D p(A.x + i * D.x, A.y + i * D.y, A.z + i * D.z);
 
 		_Draw_Label(SG_Get_String(Value, -Decimals), p, Rx, Ry, Rz, Align, Resolution, Scale);
 	}
 }
 
 //---------------------------------------------------------
-void CSG_3DView_Canvas::_Draw_Label(const CSG_String &Text, const TSG_Point_Z &Point, double Rx, double Ry, double Rz, int Align, int Resolution, double Scale)
+void CSG_3DView_Canvas::_Draw_Label(const CSG_String &Text, const TSG_Point_3D &Point, double Rx, double Ry, double Rz, int Align, int Resolution, double Scale)
 {
 	if( Text.is_Empty() || Resolution < 20 )
 	{
@@ -712,7 +811,7 @@ void CSG_3DView_Canvas::Draw_Line(double ax, double ay, double az, double bx, do
 	}
 }
 
-void CSG_3DView_Canvas::Draw_Line(const TSG_Point_Z &a, const TSG_Point_Z &b, int aColor, int bColor)
+void CSG_3DView_Canvas::Draw_Line(const TSG_Point_3D &a, const TSG_Point_3D &b, int aColor, int bColor)
 {
 	Draw_Line(a.x, a.y, a.z, b.x, b.y, b.z, aColor, bColor);
 }
@@ -755,7 +854,7 @@ void CSG_3DView_Canvas::Draw_Line(double ax, double ay, double az, double bx, do
 	}
 }
 
-void CSG_3DView_Canvas::Draw_Line(const TSG_Point_Z &a, const TSG_Point_Z &b, int Color)
+void CSG_3DView_Canvas::Draw_Line(const TSG_Point_3D &a, const TSG_Point_3D &b, int Color)
 {
 	Draw_Line(a.x, a.y, a.z, b.x, b.y, b.z, Color);
 }
