@@ -119,12 +119,21 @@ CWKSP_Data_Manager::CWKSP_Data_Manager(void)
 	m_pMenu_Files	= new CWKSP_Data_Menu_Files;
 
 	//-----------------------------------------------------
-	m_Parameters.Add_Node("",
-		"NODE_GENERAL"			, _TL("General"),
-		_TL("")
+	m_Parameters.Add_Bool (""          , "THUMBNAILS"        , _TL("Thumbnails"     ), _TL(""), true);
+	m_Parameters.Add_Int  ("THUMBNAILS", "THUMBNAIL_SIZE"    , _TL("Size"           ), _TL(""), 50, 10, true);
+	m_Parameters.Add_Color("THUMBNAILS", "THUMBNAIL_SELCOLOR", _TL("Selection Color"), _TL(""), Get_Color_asInt(SYS_Get_Color(wxSYS_COLOUR_BTNSHADOW)));
+	m_Parameters.Add_Bool ("THUMBNAILS", "THUMBNAIL_CATEGORY", _TL("Show Categories"), _TL(""), true);
+
+	m_Parameters.Add_Bool("",
+		"SHOW_FILE_SOURCES"		, _TL("Show Data File Source Browser"),
+		_TL("Show data sources tab for file system. Disabling might speed up start-up. Changes take effect after restart."),
+		true
 	);
 
-	m_Parameters.Add_Choice("NODE_GENERAL",
+	//-----------------------------------------------------
+	m_Parameters.Add_Node("", "NODE_PROJECT", _TL("Projects"), _TL(""));
+
+	m_Parameters.Add_Choice("NODE_PROJECT",
 		"PROJECT_START"			, _TL("Startup Project"),
 		_TL(""),
 		CSG_String::Format("%s|%s|%s",
@@ -134,22 +143,29 @@ CWKSP_Data_Manager::CWKSP_Data_Manager(void)
 		), 2
 	);
 
-	m_Parameters.Add_Choice("NODE_GENERAL",
-		"PROJECT_MAP_ARRANGE"	, _TL("Map Window Arrangement"),
-		_TL("initial map window arrangement after a project is loaded"),
-		CSG_String::Format("%s|%s|%s|%s",
+	m_Parameters.Add_Choice("NODE_PROJECT",
+		"PROJECT_MAP_ARRANGE", _TL("Map Window Arrangement"),
+		_TL("Initial map window arrangement after a project has been loaded."),
+		#ifdef _SAGA_MSW
+		CSG_String::Format("%s|%s|%s|%s|%s",
 			_TL("Cascade"),
 			_TL("Tile Horizontally"),
 			_TL("Tile Vertically"),
-			_TL("Maximized")
+			_TL("Maximized"),
+			_TL("Don't Restore")
 		), 2
+		#else
+		CSG_String::Format("%s|%s",
+			_TL("Open Map Windows"),
+			_TL("Don't Open Map Windows")
+		), 0
+		#endif
 	);
 
-#ifndef _SAGA_MSW
-	m_Parameters.Set_Enabled("PROJECT_MAP_ARRANGE", false);
-#endif
+	//-----------------------------------------------------
+	m_Parameters.Add_Node("", "NODE_DB", _TL("Databases"), _TL(""));
 
-	m_Parameters.Add_Choice("NODE_GENERAL",
+	m_Parameters.Add_Choice("NODE_DB",
 		"PROJECT_DB_REOPEN"		, _TL("Reopen Database Connections"),
 		_TL("Reopen PostgreSQL database connections. Warning: if set to true account information including unencrypted passwords for automatic connection will be stored."),
 		CSG_String::Format("%s|%s",
@@ -164,100 +180,72 @@ CWKSP_Data_Manager::CWKSP_Data_Manager(void)
 		2, 0, true
 	);
 
-	m_Parameters.Add_Bool("NODE_GENERAL",
-		"SHOW_FILE_SOURCES"		, _TL("Show Data File Sources"),
-		_TL("Show data sources tab for file system. Disabling might speed up start-up. Changes take effect after restart."),
-		true
+	//-----------------------------------------------------
+	m_Parameters.Add_Node("", "NODE_DATA", _TL("Data"), _TL(""));
+
+	m_Parameters.Add_Int("NODE_DATA",
+		"NUMBERING"				, _TL("Data Set Numeration"),
+		_TL("Minimum width of data set numbering. If set to 0 no numbering is applied at all. Negative values will prepend zeros. Numbering helps in data set sorting and selection."),
+		3
 	);
 
-	m_Parameters.Add_Int("NODE_GENERAL",
-		"NUMBERING"				, _TL("Numbering of Data Sets"),
-		_TL("Leading zeros for data set numbering. Set to -1 for not using numbers at all."),
-		m_Numbering = 2, -1, true
-	);
-
-	m_Parameters.Add_Bool("NODE_GENERAL",
+	m_Parameters.Add_Bool("NODE_DATA",
 		"NAME_BY_FILE"			, _TL("Use File Name for Naming"),
 		_TL("Use file name to name a data set, when it has been loaded from file."),
 		false
 	);
 
-	m_Parameters.Add_Colors("NODE_GENERAL",
-		"COLORS_DEFAULT"		, _TL("Default Colors"),
-		_TL("")
-	);
-
-	m_Parameters.Add_Bool("NODE_GENERAL",
+	m_Parameters.Add_Bool("NODE_DATA",
 		"COLORS_FROM_TOOL"		, _TL("Tool Set Colors"),
 		_TL("Allow tools to change data set colors programmatically."),
 		true
 	);
 
+	m_Parameters.Add_Colors("NODE_DATA",
+		"COLORS_DEFAULT"		, _TL("Default Colors"),
+		_TL("")
+	);
+
+	m_Parameters.Add_Choice("NODE_DATA"      , "STRETCH_DEFAULT", _TL("Histogram Stretch Defaults"), _TL("Histogram stretch applied by default to new data."), CSG_String::Format("%s|%s|%s",
+		_TL("Linear"), _TL("Standard Deviation"), _TL("Percent Clip")), 1
+	);
+
+	m_Parameters.Add_Range ("STRETCH_DEFAULT", "STRETCH_LINEAR" , _TL("Linear"                    ), _TL("Linear percent stretch allows you to trim extreme values from both ends of the histogram using the percentage specified here."), 5., 95., 0., true, 100., true);
+	m_Parameters.Add_Double("STRETCH_DEFAULT", "STRETCH_STDDEV" , _TL("Standard Deviation"        ), _TL(""), 2., 0., true);
+	m_Parameters.Add_Range ("STRETCH_DEFAULT", "STRETCH_PCTL"   , _TL("Percent Clip"              ), _TL(""), 2., 98., 0., true, 100., true);
+
 	SG_DataObject_Set_Max_Samples(1000000);	// default to one million
 
-	m_Parameters.Add_Int("NODE_GENERAL",
+	m_Parameters.Add_Int("NODE_DATA",
 		"DATA_SAMPLE_MAX"		, _TL("Maximum Samples"),
 		_TL("Default maximum number of samples used to build statistics and histograms. Set to zero to use all data records."),
 		(int)SG_DataObject_Get_Max_Samples(), 0, true
 	);
 
 	//-----------------------------------------------------
-	m_Parameters.Add_Node("",
-		"NODE_HISTORY"			, _TL("History"),
-		_TL("")
-	);
-
-	m_Parameters.Add_Int("NODE_HISTORY",
+	m_Parameters.Add_Int("NODE_DATA",
 		"HISTORY_DEPTH"			, _TL("History Depth"),
 		_TL("Depth to which data history is stored. Set -1 keeps all history entries (default), 0 switches history option off."),
 		SG_Get_History_Depth(), -1, true
 	);
 
-	m_Parameters.Add_Bool("NODE_HISTORY",
+	m_Parameters.Add_Bool("HISTORY_DEPTH",
 		"HISTORY_LISTS"			, _TL("Ignore Input Lists"),
 		_TL(""),
 		SG_Get_History_Ignore_Lists() != 0
 	);
 
 	//-----------------------------------------------------
-	m_Parameters.Add_Node("",
-		"NODE_THUMBNAILS"		, _TL("Thumbnails"),
-		_TL("")
-	);
-
-	m_Parameters.Add_Int("NODE_THUMBNAILS",
-		"THUMBNAIL_SIZE"		, _TL("Thumbnail Size"),
-		_TL(""),
-		50, 10, true
-	);
-
-	m_Parameters.Add_Bool("NODE_THUMBNAILS",
-		"THUMBNAIL_CATEGORY"	, _TL("Show Categories"),
-		_TL(""),
-		true
-	);
-
-	m_Parameters.Add_Color("NODE_THUMBNAILS",
-		"THUMBNAIL_SELCOLOR"	, _TL("Selection Color"),
-		_TL(""),
-		Get_Color_asInt(SYS_Get_Color(wxSYS_COLOUR_BTNSHADOW))
-	);
-
-	//-----------------------------------------------------
-	m_Parameters.Add_Node("",
-		"NODE_GRID"				, _TL("Grids"),
-		_TL("")
-	);
+	m_Parameters.Add_Node("", "NODE_GRID", _TL("Grids"), _TL(""));
 
 	m_Parameters.Add_Choice("NODE_GRID",
 		"GRID_FMT_DEFAULT"		, _TL("Default Output Format"),
 		_TL(""),
-		CSG_String::Format("%s (*.sg-grd-z)|%s (*.sg-grd)|%s (*.sgrd)|%s (*.tif)",
+		CSG_String::Format("%s (*.sg-grd-z)|%s (*.sg-grd)|%s (*.tif)",
 			_TL("SAGA Compressed Grid Files"),
 			_TL("SAGA Grid Files"),
-			_TL("SAGA Grid Files (old extension)"),
 			_TL("GeoTIFF")
-		), 2
+		), 0
 	);
 
 	m_Parameters.Add_Int("NODE_GRID",
@@ -266,24 +254,12 @@ CWKSP_Data_Manager::CWKSP_Data_Manager(void)
 		10, 0, true
 	);
 
-	m_Parameters.Add_Choice("NODE_GRID",
-		"GRID_STRETCH_DEFAULT"	, _TL("Histogram Stretch"),
-		_TL("Histogram stretch appolied by default to new grids."),
-		CSG_String::Format("%s|%s|%s",
-			_TL("Minimum/Maximum"),
-			_TL("Standard Deviation"),
-			_TL("Percentile")
-		), 1
-	);
-
-	//-----------------------------------------------------
 	m_Parameters.Add_Int("NODE_GRID",
 		"GRID_SELECT_MAX"		, _TL("Maximum Selection"),
 		_TL("Maximum number of rows/columns in selection of grid cells."),
 		100, 1, true
 	);
 
-	//-----------------------------------------------------
 	m_Parameters.Add_Choice("NODE_GRID",
 		"GRID_CACHE_MODE"		, _TL("File Cache"),
 		_TL("Activate file caching automatically, if memory size exceeds the threshold value."),
@@ -297,7 +273,7 @@ CWKSP_Data_Manager::CWKSP_Data_Manager(void)
 	m_Parameters.Add_Double("GRID_CACHE_MODE",
 		"GRID_CACHE_THRSHLD"	, _TL("Threshold for automatic mode [MB]"),
 		_TL(""),
-		SG_Grid_Cache_Get_Threshold_MB(), 0.0, true
+		SG_Grid_Cache_Get_Threshold_MB(), 0., true
 	);
 
 	m_Parameters.Add_FilePath("GRID_CACHE_MODE",
@@ -307,10 +283,7 @@ CWKSP_Data_Manager::CWKSP_Data_Manager(void)
 	);
 
 	//-----------------------------------------------------
-	m_Parameters.Add_Node("",
-		"NODE_TABLE"			, _TL("Tables"),
-		_TL("")
-	);
+	m_Parameters.Add_Node("", "NODE_TABLE", _TL("Tables"), _TL(""));
 
 	m_Parameters.Add_Choice("NODE_TABLE",
 		"TABLE_FLT_STYLE"		, _TL("Floating Point Numbers"),
@@ -329,10 +302,7 @@ CWKSP_Data_Manager::CWKSP_Data_Manager(void)
 	);
 
 	//-----------------------------------------------------
-	m_Parameters.Add_Node("",
-		"NODE_SHAPES"			, _TL("Shapes"),
-		_TL("")
-	);
+	m_Parameters.Add_Node("", "NODE_SHAPES", _TL("Shapes"), _TL(""));
 
 	m_Parameters.Add_Choice("NODE_SHAPES",
 		"SHAPES_FMT_DEFAULT"	, _TL("Default Output Format"),
@@ -362,8 +332,7 @@ CWKSP_Data_Manager::CWKSP_Data_Manager(void)
 	{
 	default: SG_Grid_Set_File_Format_Default(GRID_FILE_FORMAT_Compressed); break;
 	case  1: SG_Grid_Set_File_Format_Default(GRID_FILE_FORMAT_Binary    ); break;
-	case  2: SG_Grid_Set_File_Format_Default(GRID_FILE_FORMAT_Binary_old); break;
-	case  3: SG_Grid_Set_File_Format_Default(GRID_FILE_FORMAT_GeoTIFF   ); break;
+	case  2: SG_Grid_Set_File_Format_Default(GRID_FILE_FORMAT_GeoTIFF   ); break;
 	}
 
 	switch( m_Parameters("SHAPES_FMT_DEFAULT")->asInt() )
@@ -372,8 +341,6 @@ CWKSP_Data_Manager::CWKSP_Data_Manager(void)
 	case  1: SG_Shapes_Set_File_Format_Default(SHAPE_FILE_FORMAT_GeoPackage); break;
 	case  2: SG_Shapes_Set_File_Format_Default(SHAPE_FILE_FORMAT_GeoJSON   ); break;
 	}
-
-	m_Numbering	= m_Parameters("NUMBERING")->asInt();
 }
 
 //---------------------------------------------------------
@@ -761,8 +728,7 @@ void CWKSP_Data_Manager::Parameters_Changed(void)
 	{
 	default: SG_Grid_Set_File_Format_Default(GRID_FILE_FORMAT_Compressed); break;
 	case  1: SG_Grid_Set_File_Format_Default(GRID_FILE_FORMAT_Binary    ); break;
-	case  2: SG_Grid_Set_File_Format_Default(GRID_FILE_FORMAT_Binary_old); break;
-	case  3: SG_Grid_Set_File_Format_Default(GRID_FILE_FORMAT_GeoTIFF   ); break;
+	case  2: SG_Grid_Set_File_Format_Default(GRID_FILE_FORMAT_GeoTIFF   ); break;
 	}
 
 	switch( m_Parameters("SHAPES_FMT_DEFAULT")->asInt() )
@@ -772,9 +738,7 @@ void CWKSP_Data_Manager::Parameters_Changed(void)
 	case  2: SG_Shapes_Set_File_Format_Default(SHAPE_FILE_FORMAT_GeoJSON   ); break;
 	}
 
-	m_Numbering	= m_Parameters("NUMBERING")->asInt();
-
-	g_pData_Buttons->Update_Buttons();
+	if( g_pData_Buttons ) { g_pData_Buttons->Update_Buttons(); }
 
 	CWKSP_Base_Manager::Parameters_Changed();
 }
@@ -790,11 +754,31 @@ int CWKSP_Data_Manager::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Pa
 	//-----------------------------------------------------
 	if( Flags & PARAMETER_CHECK_VALUES )
 	{
+		if( g_pSAGA_Frame && g_pData )
+		{
+			if( pParameter->Cmp_Identifier("THUMBNAILS") )
+			{
+				if( DLG_Message_Confirm(_TL("Close now ?"), _TL("Restart SAGA to apply the changes")) )
+				{
+					m_Parameters.Assign_Values(pParameters);
+
+					if( g_pData->Close(true) )
+					{
+						g_pSAGA_Frame->Close();
+					}
+				}
+			}
+		}
 	}
 
 	//-----------------------------------------------------
 	if( Flags & PARAMETER_CHECK_ENABLE )
 	{
+		if(	pParameter->Cmp_Identifier("THUMBNAILS") )
+		{
+			pParameter->Set_Children_Enabled(pParameter->asBool());
+		}
+
 		if(	pParameter->Cmp_Identifier("GRID_CACHE_MODE") )
 		{
 			pParameters->Set_Enabled("GRID_CACHE_THRSHLD", pParameter->asInt() != 0);
@@ -817,33 +801,33 @@ int CWKSP_Data_Manager::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Pa
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-CWKSP_Base_Item * CWKSP_Data_Manager::Open(const wxString &File, int DataType)
+CWKSP_Data_Item * CWKSP_Data_Manager::Open(const wxString &File, int DataType)
 {
-	CSG_Data_Object	*pObject	= NULL;
+	CSG_Data_Object *pObject = NULL;
 
 //	SG_Get_Data_Manager().Add(File, DataType);
 
 	switch( DataType )
 	{
-	case SG_DATAOBJECT_TYPE_Table     :	pObject	= SG_Create_Table     (&File);	break;
-	case SG_DATAOBJECT_TYPE_Shapes    :	pObject	= SG_Create_Shapes    (&File);	break;
-	case SG_DATAOBJECT_TYPE_TIN       :	pObject	= SG_Create_TIN       (&File);	break;
-	case SG_DATAOBJECT_TYPE_PointCloud:	pObject	= SG_Create_PointCloud(&File);	break;
-	case SG_DATAOBJECT_TYPE_Grid      :	pObject	= SG_Create_Grid      (&File);	break;
-	case SG_DATAOBJECT_TYPE_Grids     :	pObject	= SG_Create_Grids     (&File);	break;
+	case SG_DATAOBJECT_TYPE_Table     : pObject = SG_Create_Table     (&File); break;
+	case SG_DATAOBJECT_TYPE_Shapes    : pObject = SG_Create_Shapes    (&File); break;
+	case SG_DATAOBJECT_TYPE_TIN       : pObject = SG_Create_TIN       (&File); break;
+	case SG_DATAOBJECT_TYPE_PointCloud: pObject = SG_Create_PointCloud(&File); break;
+	case SG_DATAOBJECT_TYPE_Grid      : pObject = SG_Create_Grid      (&File); break;
+	case SG_DATAOBJECT_TYPE_Grids     : pObject = SG_Create_Grids     (&File); break;
 	}
 
 	if( pObject )
 	{
-		CWKSP_Data_Item	*pItem;
+		CWKSP_Data_Item *pItem = Add(pObject);
 
-		if( pObject->is_Valid() && (pItem = Add(pObject)) != NULL )
+		if( pItem )
 		{
 			m_pMenu_Files->Recent_Add(pObject->Get_ObjectType(), File);
 
 			SG_Get_Data_Manager().Add(pObject);
 
-			return( (CWKSP_Base_Item *)pItem );
+			return( pItem );
 		}
 
 		delete(pObject);
@@ -1281,6 +1265,27 @@ bool CWKSP_Data_Manager::Save_Modified_Sel(void)
 //---------------------------------------------------------
 bool CWKSP_Data_Manager::Close(bool bSilent)
 {
+	if( g_pTool )
+	{
+		if( !g_pTool->is_Executing() )
+		{
+			g_pTool->Finish(false, false);
+		}
+		else
+		{
+			if( bSilent )
+			{
+				MSG_General_Add(_TL("Stop running tool or wait until its exection has been finished!"), true, false);
+			}
+			else
+			{
+				DLG_Message_Show(_TL("Stop running tool or wait until its exection has been finished!"), g_pTool->Get_Name());
+			}
+
+			return( false );
+		}
+	}
+
 	if( Get_Count() == 0 || ((bSilent || DLG_Message_Confirm(_TL("Close all data sets"), _TL("Close"))) && Save_Modified(this)) )
 	{
 		m_pProject->Clr_File_Name();
@@ -1336,13 +1341,68 @@ CWKSP_Data_Item * CWKSP_Data_Manager::Get(CSG_Data_Object *pObject)
 		case SG_DATAOBJECT_TYPE_Shapes    : return( (CWKSP_Data_Item *)m_pShapes     ->Get_Data((CSG_Shapes     *)pObject) );
 		case SG_DATAOBJECT_TYPE_Grid      : return( (CWKSP_Data_Item *)m_pGrids      ->Get_Data((CSG_Grid       *)pObject) );
 		case SG_DATAOBJECT_TYPE_Grids     : return( (CWKSP_Data_Item *)m_pGrids      ->Get_Data((CSG_Grids      *)pObject) );
-		default:	break;
+		default: break;
 		}
 	}
 
 	return( NULL );
 }
 
+//---------------------------------------------------------
+CWKSP_Data_Item * CWKSP_Data_Manager::Get_byFile(const wxString &File)
+{
+	return( Get(SG_Get_Data_Manager().Find(&File, false)) );
+}
+
+//---------------------------------------------------------
+CWKSP_Data_Item * CWKSP_Data_Manager::Get_byID(const wxString &ID)
+{
+	CWKSP_Data_Item *pItem;
+
+	if( m_pTables      && (pItem = (CWKSP_Data_Item *)m_pTables     ->Get_Item_byID(ID)) != NULL ) { return( pItem ); }
+	if( m_pTINs        && (pItem = (CWKSP_Data_Item *)m_pTINs       ->Get_Item_byID(ID)) != NULL ) { return( pItem ); }
+	if( m_pPointClouds && (pItem = (CWKSP_Data_Item *)m_pPointClouds->Get_Item_byID(ID)) != NULL ) { return( pItem ); }
+	if( m_pShapes      && (pItem = (CWKSP_Data_Item *)m_pShapes     ->Get_Item_byID(ID)) != NULL ) { return( pItem ); }
+	if( m_pGrids       && (pItem = (CWKSP_Data_Item *)m_pGrids      ->Get_Item_byID(ID)) != NULL ) { return( pItem ); }
+
+	return( NULL );
+}
+
+//---------------------------------------------------------
+CWKSP_Data_Item * CWKSP_Data_Manager::Get_byID_or_File(const SG_Char *ID, const SG_Char *File, const SG_Char *Root)
+{
+	CWKSP_Data_Item *pItem = NULL;
+
+	if( ID )
+	{
+		pItem = Get_byID(ID);
+	}
+
+	if( File )
+	{
+		wxString _File(File);
+
+		if( _File.Find("PGSQL") != 0 && Root && SG_Dir_Exists(Root) )
+		{
+			_File = Get_FilePath_Absolute(Root, _File);
+		}
+
+		if( pItem )
+		{
+			if( _File.CmpNoCase(pItem->Get_Object()->Get_File_Name(false)) != 0 )
+			{
+				pItem = NULL;
+			}
+		}
+		else
+		{
+			pItem = Get_byFile(_File);
+		}
+	}
+
+	return( pItem );
+}
+
 //---------------------------------------------------------
 CWKSP_Data_Item * CWKSP_Data_Manager::Add(CSG_Data_Object *pObject)
 {
@@ -1356,7 +1416,7 @@ CWKSP_Data_Item * CWKSP_Data_Manager::Add(CSG_Data_Object *pObject)
 		case SG_DATAOBJECT_TYPE_Shapes    : return( (CWKSP_Data_Item *)m_pShapes     ->Add_Data((CSG_Shapes     *)pObject) );
 		case SG_DATAOBJECT_TYPE_Grid      : return( (CWKSP_Data_Item *)m_pGrids      ->Add_Data((CSG_Grid       *)pObject) );
 		case SG_DATAOBJECT_TYPE_Grids     : return( (CWKSP_Data_Item *)m_pGrids      ->Add_Data((CSG_Grids      *)pObject) );
-		default:	break;
+		default: break;
 		}
 	}
 
@@ -1375,7 +1435,7 @@ CWKSP_Layer * CWKSP_Data_Manager::Get_Layer(CSG_Data_Object *pObject)
 		case SG_DATAOBJECT_TYPE_Shapes    : return( (CWKSP_Layer *)m_pShapes     ->Get_Data((CSG_Shapes     *)pObject) );
 		case SG_DATAOBJECT_TYPE_Grid      : return( (CWKSP_Layer *)m_pGrids      ->Get_Data((CSG_Grid       *)pObject) );
 		case SG_DATAOBJECT_TYPE_Grids     : return( (CWKSP_Layer *)m_pGrids      ->Get_Data((CSG_Grids      *)pObject) );
-		default:	break;
+		default: break;
 		}
 	}
 
@@ -1492,6 +1552,23 @@ bool CWKSP_Data_Manager::Set_Parameters(CSG_Data_Object *pObject, CSG_Parameters
 	return( false );
 }
 
+//---------------------------------------------------------
+bool CWKSP_Data_Manager::Show_Diagram(CSG_Table *pTable, CSG_Parameters *pParameters)
+{
+	CWKSP_Data_Item *pItem = Get(pTable);
+
+	switch( pItem->Get_Type() )
+	{
+	case WKSP_ITEM_Table     : ((CWKSP_Table      *)pItem)->Set_Diagram(true, pParameters); return( true );
+	case WKSP_ITEM_Shapes    : ((CWKSP_Shapes     *)pItem)->Set_Diagram(true, pParameters); return( true );
+//	case WKSP_ITEM_PointCloud: ((CWKSP_PointCloud *)pItem)->Set_Diagram(true, pParameters); return( true );
+
+	default: break;
+	}
+
+	return( false );
+}
+
 
 ///////////////////////////////////////////////////////////
 //														 //
