@@ -352,7 +352,7 @@ void CWKSP_Grid::On_Create_Parameters(void)
 	m_Parameters.Add_Choice("NODE_DISPLAY", "DISPLAY_SHADING"	, _TL("Shading"),
 		_TL(""),
 		CSG_String::Format("%s|%s|%s",
-			_TL("none"), _TL("normal"), _TL("inverse")
+			_TL("none"), _TL("traditional"), _TL("multidirectional")
 		), 0
 	);
 
@@ -491,7 +491,7 @@ void CWKSP_Grid::On_Create_Parameters(void)
 	);
 
 	//-----------------------------------------------------
-	m_Fit_Colors	= g_pData->Get_Parameter("GRID_STRETCH_DEFAULT")->asInt();
+	m_Fit_Colors = g_pData->Get_Parameter("STRETCH_DEFAULT")->asInt();
 }
 
 
@@ -544,7 +544,7 @@ void CWKSP_Grid::On_Parameters_Changed(void)
 	//-----------------------------------------------------
 	if( m_Parameters("STRETCH_DEFAULT")->asInt() < 3 )	// not manual, remember last state...
 	{
-		m_Fit_Colors	= m_Parameters("STRETCH_DEFAULT")->asInt();
+		m_Fit_Colors = m_Parameters("STRETCH_DEFAULT")->asInt();
 	}
 }
 
@@ -878,14 +878,14 @@ void CWKSP_Grid::_LUT_Create(void)
 //---------------------------------------------------------
 wxString CWKSP_Grid::Get_Value(CSG_Point ptWorld, double Epsilon)
 {
-	wxString	s;	double	Value;
+	wxString s; double Value;
 
 	if( Get_Grid()->Get_Value(ptWorld, Value, GRID_RESAMPLING_NearestNeighbour) )
 	{
 		switch( m_pClassify->Get_Mode() )
 		{
 		default:
-			s	= SG_Get_String(Value, -12).c_str();
+			s = SG_Get_String(Value, -12).c_str();
 
 			if( !Get_Grid()->Get_Unit().is_Empty() )
 			{
@@ -898,7 +898,7 @@ wxString CWKSP_Grid::Get_Value(CSG_Point ptWorld, double Epsilon)
 			break;
 
 		case CLASSIFY_LUT:
-			s	= m_pClassify->Get_Class_Name_byValue(Value);
+			s = m_pClassify->Get_Class_Name_byValue(Value);
 			break;
 		}
 	}
@@ -952,7 +952,7 @@ TSG_Rect CWKSP_Grid::Edit_Get_Extent(void)
 		);
 	}
 
-	return( Get_Grid()->Get_Extent().m_rect );
+	return( Get_Grid()->Get_Extent() );
 }
 
 //---------------------------------------------------------
@@ -1047,7 +1047,7 @@ bool CWKSP_Grid::Edit_Set_Attributes(void)
 {
 	if( m_Edit_Attributes.Get_Count() > 0 )
 	{
-		for(int y=0; y<m_Edit_Attributes.Get_Count(); y++)
+		for(sLong y=0; y<m_Edit_Attributes.Get_Count(); y++)
 		{
 			CSG_Table_Record	*pRecord	= m_Edit_Attributes.Get_Record(m_Edit_Attributes.Get_Count() - 1 - y);
 
@@ -1099,7 +1099,7 @@ bool CWKSP_Grid::_Edit_Del_Selection(void)
 {
 	if( m_Edit_Attributes.Get_Count() > 0 && DLG_Message_Confirm(_TL("Set selected values to no data."), _TL("Delete")) )
 	{
-		for(int y=0; y<m_Edit_Attributes.Get_Count(); y++)
+		for(sLong y=0; y<m_Edit_Attributes.Get_Count(); y++)
 		{
 			for(int x=0, xx=1; xx<m_Edit_Attributes.Get_Field_Count(); x++, xx++)
 			{
@@ -1353,11 +1353,7 @@ void CWKSP_Grid::On_Draw(CWKSP_Map_DC &dc_Map, int Flags)
 	//-----------------------------------------------------
 	TSG_Grid_Resampling	Resampling;
 
-	if( m_pClassify->Get_Mode() == CLASSIFY_LUT )
-	{
-		Resampling	= GRID_RESAMPLING_NearestNeighbour;
-	}
-	else switch( m_Parameters("DISPLAY_RESAMPLING")->asInt() )
+	switch( m_Parameters(m_pClassify->Get_Mode() == CLASSIFY_LUT ? "LUT_RESAMPLING" : "DISPLAY_RESAMPLING")->asInt() )
 	{
 	default: Resampling = GRID_RESAMPLING_NearestNeighbour; break;
 	case  1: Resampling = GRID_RESAMPLING_Bilinear        ; break;
@@ -1583,13 +1579,7 @@ void CWKSP_Grid::_Draw_Grid_Cells(CWKSP_Map_DC &dc_Map)
 //---------------------------------------------------------
 inline void CWKSP_Grid::_Set_Shading(double Shade, int &Color)
 {
-	switch( m_Shade_Mode )
-	{
-	default: Shade =      Shade / M_PI_090; break;
-	case  1: Shade = 1. - Shade / M_PI_090; break;
-	}
-
-	Shade	= m_Shade_Parms[5] * (Shade - m_Shade_Parms[4]);
+	Shade	= m_Shade_Parms[5] * ((1. - Shade / M_PI_090) - m_Shade_Parms[4]);
 
 	int	r	= (int)(Shade * SG_GET_R(Color)); if( r < 0 ) r = 0; else if( r > 255 ) r = 255;
 	int	g	= (int)(Shade * SG_GET_G(Color)); if( g < 0 ) g = 0; else if( g > 255 ) g = 255;
@@ -1609,7 +1599,21 @@ inline int CWKSP_Grid::_Get_Shading(int x, int y, int Color)
 		{
 			s	= M_PI_090 - atan(m_Shade_Parms[0] * tan(s));
 
-			_Set_Shading(acos(sin(s) * m_Shade_Parms[1] + cos(s) * m_Shade_Parms[2] * cos(a - m_Shade_Parms[3])), Color);
+			if( m_Shade_Mode == 1 )
+			{
+				_Set_Shading(acos(sin(s) * m_Shade_Parms[1] + cos(s) * m_Shade_Parms[2] * cos(a - m_Shade_Parms[3])), Color);
+			}
+			else
+			{
+				double sum = 0., dir = m_Shade_Parms[3] - M_PI_180 / 2.;
+
+				for(int i=0; i<6; i++, dir+=M_PI_180/5.)
+				{
+					sum += acos(sin(s) * m_Shade_Parms[1] + cos(s) * m_Shade_Parms[2] * cos(a - dir));
+				}
+
+				_Set_Shading(sum / 6., Color);
+			}
 		}
 	}
 
@@ -1636,7 +1640,21 @@ inline int CWKSP_Grid::_Get_Shading(double x, double y, int Color, TSG_Grid_Resa
 		{
 			s	= M_PI_090 - atan(m_Shade_Parms[0] * tan(s));
 
-			_Set_Shading(acos(sin(s) * m_Shade_Parms[1] + cos(s) * m_Shade_Parms[2] * cos(a - m_Shade_Parms[3])), Color);
+			if( m_Shade_Mode == 1 )
+			{
+				_Set_Shading(acos(sin(s) * m_Shade_Parms[1] + cos(s) * m_Shade_Parms[2] * cos(a - m_Shade_Parms[3])), Color);
+			}
+			else
+			{
+				double sum = 0., dir = m_Shade_Parms[3] - M_PI_180 / 2.;
+
+				for(int i=0; i<6; i++, dir+=M_PI_180/5.)
+				{
+					sum += acos(sin(s) * m_Shade_Parms[1] + cos(s) * m_Shade_Parms[2] * cos(a - dir));
+				}
+
+				_Set_Shading(sum / 6., Color);
+			}
 		}
 	}
 
