@@ -146,14 +146,14 @@ bool CWKSP_Map_Extents::Add_Extent(const CSG_Rect &Extent, bool bReset)
 	{
 		if( bReset )
 		{
-			m_iExtent	= -1;
-			m_nExtents	= 0;
+			m_iExtent  = -1;
+			m_nExtents = 0;
 		}
 
 		if( Extent != Get_Extent() )
 		{
-			m_iExtent	++;
-			m_nExtents	= m_iExtent + 1;
+			m_iExtent++;
+			m_nExtents = m_iExtent + 1;
 
 			if( m_nExtents > Get_Count() )
 			{
@@ -161,7 +161,7 @@ bool CWKSP_Map_Extents::Add_Extent(const CSG_Rect &Extent, bool bReset)
 			}
 			else
 			{
-				Get_Rect(m_iExtent)	= Extent;
+				Get_Rect(m_iExtent) = Extent;
 			}
 
 			return( true );
@@ -181,17 +181,15 @@ bool CWKSP_Map_Extents::Add_Extent(const CSG_Rect &Extent, bool bReset)
 //---------------------------------------------------------
 CWKSP_Map::CWKSP_Map(void)
 {
-	static int	iMap	= 0;
+	m_Name         = _TL("Map");
 
-	m_Name.Printf("%02d. %s", ++iMap, _TL("Map"));
+	m_pView        = NULL;
+	m_pView_3D     = NULL;
+	m_pLayout      = NULL;
+	m_pLayout_Info = new CVIEW_Layout_Info(this);
 
-	m_pView			= NULL;
-	m_pView_3D		= NULL;
-	m_pLayout		= NULL;
-	m_pLayout_Info	= new CVIEW_Layout_Info(this);
-
-	m_Img_bSave		= false;
-	m_Sync_bLock	= 0;
+	m_Img_bSave    = false;
+	m_Sync_bLock   = 0;
 
 	On_Create_Parameters();
 }
@@ -220,7 +218,7 @@ wxString CWKSP_Map::Get_Name(void)
 //---------------------------------------------------------
 wxString CWKSP_Map::Get_Description(void)
 {
-	wxString	s;
+	wxString s;
 
 	//-----------------------------------------------------
 	s	+= wxString::Format("<h4>%s</h4>", _TL("Map"));
@@ -231,6 +229,15 @@ wxString CWKSP_Map::Get_Description(void)
 	DESC_ADD_INT(_TL("Layers"           ), Get_Count());
 	DESC_ADD_STR(_TL("Coordinate System"), m_Projection.Get_Description().c_str());
 
+	int n = Get_Extent().Get_Diameter() > 1 ? 0 : -99;
+
+	DESC_ADD_STR(_TL("West"             ), SG_Get_String(Get_Extent().Get_XMin  (), n).c_str());
+	DESC_ADD_STR(_TL("East"             ), SG_Get_String(Get_Extent().Get_XMax  (), n).c_str());
+	DESC_ADD_STR(_TL("West-East"        ), SG_Get_String(Get_Extent().Get_XRange(), n).c_str());
+	DESC_ADD_STR(_TL("South"            ), SG_Get_String(Get_Extent().Get_YMin  (), n).c_str());
+	DESC_ADD_STR(_TL("North"            ), SG_Get_String(Get_Extent().Get_YMax  (), n).c_str());
+	DESC_ADD_STR(_TL("South-North"      ), SG_Get_String(Get_Extent().Get_YRange(), n).c_str());
+
 	s	+= "</table>";
 
 	//-----------------------------------------------------
@@ -622,7 +629,9 @@ bool CWKSP_Map::Serialize(CSG_MetaData &Root, const wxString &ProjectDir, bool b
 {
 	if( bSave )
 	{
-		CSG_MetaData	&Map	= *Root.Add_Child("MAP");
+		CSG_MetaData &Map = *Root.Add_Child("MAP");
+
+		Map.Add_Property("SHOWN", View_Get() != NULL ? 1 : 0);
 
 		if( Get_Projection().is_Okay() )
 		{
@@ -638,7 +647,7 @@ bool CWKSP_Map::Serialize(CSG_MetaData &Root, const wxString &ProjectDir, bool b
 
 		m_pLayout_Info->Save(*Map.Add_Child("LAYOUT"));
 
-		CSG_MetaData	&Layers	= *Map.Add_Child("LAYERS");
+		CSG_MetaData &Layers = *Map.Add_Child("LAYERS");
 
 		for(int i=Get_Count()-1; i>=0; i--)
 		{
@@ -648,24 +657,27 @@ bool CWKSP_Map::Serialize(CSG_MetaData &Root, const wxString &ProjectDir, bool b
 			case WKSP_ITEM_Map_BaseMap  : ((CWKSP_Map_BaseMap   *)Get_Item(i))->Save(Layers); break;
 			case WKSP_ITEM_Map_Layer    :
 			{
-				CWKSP_Map_Layer	*pLayer  = (CWKSP_Map_Layer     *)Get_Item(i);
-				CSG_Data_Object	*pObject = pLayer->Get_Layer()->Get_Object();
+				CWKSP_Map_Layer *pLayer  = (CWKSP_Map_Layer     *)Get_Item(i);
+				CSG_Data_Object *pObject = pLayer->Get_Layer()->Get_Object();
 
 				if( pObject && pObject->Get_File_Name(false) && *pObject->Get_File_Name(false) )
 				{
-					wxString	FileName(pObject->Get_File_Name(false));
+					wxString FileName(pObject->Get_File_Name(false)); CSG_MetaData *pEntry = NULL;
 
 					if( FileName.Find("PGSQL") == 0 )
 					{
-						pLayer->Save_Settings(
-							Layers.Add_Child("FILE", &FileName)
-						);
+						pEntry = Layers.Add_Child("FILE", &FileName);
 					}
 					else if( wxFileExists(FileName) )
 					{
-						pLayer->Save_Settings(
-							Layers.Add_Child("FILE", SG_File_Get_Path_Relative(&ProjectDir, &FileName))
-						);
+						pEntry = Layers.Add_Child("FILE", SG_File_Get_Path_Relative(&ProjectDir, &FileName));
+					}
+
+					if( pEntry )
+					{
+						pEntry->Add_Property("dataset_id", pLayer->Get_Layer()->Get_Unique_ID().wc_str());
+
+						pLayer->Save_Settings(pEntry);
 					}
 				}
 				break; }
@@ -679,7 +691,7 @@ bool CWKSP_Map::Serialize(CSG_MetaData &Root, const wxString &ProjectDir, bool b
 	//-----------------------------------------------------
 	else // if( bSave == false )
 	{
-		TSG_Rect	Extent;
+		TSG_Rect Extent;
 
 		if( !Root.Cmp_Name("MAP") || !Root("LAYERS") || Root["LAYERS"].Get_Children_Count() < 1
 		||	!Root("XMIN") || !Root("XMIN")->Get_Content().asDouble(Extent.xMin)
@@ -698,24 +710,17 @@ bool CWKSP_Map::Serialize(CSG_MetaData &Root, const wxString &ProjectDir, bool b
 		//-------------------------------------------------
         SG_UI_Process_Set_Busy(true);
 
-		CSG_MetaData	&Layers	= *Root.Get_Child("LAYERS");	int	nLayers	= 0;
+		CSG_MetaData &Layers = *Root.Get_Child("LAYERS"); int nLayers = 0;
 
 		m_Parameters["CRS_CHECK"].Set_Value(false);
 
 		for(int i=0; i<Layers.Get_Children_Count(); i++)
 		{
-			CSG_MetaData	&Layer	= *Layers(i);
+			CSG_MetaData &Layer = *Layers(i);
 
 			if( Layer.Cmp_Name("FILE") )
 			{
-				wxString	FileName(Layer.Get_Content().w_str());
-
-				if( FileName.Find("PGSQL") != 0 )
-				{
-					FileName	= Get_FilePath_Absolute(ProjectDir, FileName);
-				}
-
-				CWKSP_Base_Item	*pItem	= g_pData->Get(SG_Get_Data_Manager().Find(&FileName, false));
+				CWKSP_Data_Item *pItem = g_pData->Get_byID_or_File(Layer.Get_Property("dataset_id"), Layer.Get_Content(), ProjectDir);
 
 				if(	pItem &&
 				(   pItem->Get_Type() == WKSP_ITEM_Grid
@@ -724,7 +729,7 @@ bool CWKSP_Map::Serialize(CSG_MetaData &Root, const wxString &ProjectDir, bool b
 				||  pItem->Get_Type() == WKSP_ITEM_PointCloud
 				||  pItem->Get_Type() == WKSP_ITEM_Shapes) )
 				{
-					CWKSP_Map_Layer	*pLayer	= Add_Layer((CWKSP_Layer *)pItem);
+					CWKSP_Map_Layer *pLayer = Add_Layer((CWKSP_Layer *)pItem);
 
 					if( pLayer )
 					{
@@ -736,15 +741,8 @@ bool CWKSP_Map::Serialize(CSG_MetaData &Root, const wxString &ProjectDir, bool b
 			}
 			else if( Layer.Cmp_Name("PARAMETERS") )
 			{
-				if( Layer.Cmp_Property("name", "GRATICULE") )
-				{
-					Add_Graticule(&Layer);
-				}
-
-				if( Layer.Cmp_Property("name", "BASEMAP") )
-				{
-					Add_BaseMap  (&Layer);
-				}
+				if( Layer.Cmp_Property("name", "GRATICULE") ) { Add_Graticule(&Layer); }
+				if( Layer.Cmp_Property("name", "BASEMAP"  ) ) { Add_BaseMap  (&Layer); }
 			}
 		}
 
@@ -769,8 +767,6 @@ bool CWKSP_Map::Serialize(CSG_MetaData &Root, const wxString &ProjectDir, bool b
 		}
 
 		Set_Extent(Extent, true);
-
-		View_Show(true);
 	}
 
 	//-----------------------------------------------------
@@ -785,13 +781,18 @@ bool CWKSP_Map::Serialize(CSG_MetaData &Root, const wxString &ProjectDir, bool b
 //---------------------------------------------------------
 bool CWKSP_Map::Update(CWKSP_Layer *pLayer, bool bMapOnly)
 {
-	bool	bRefresh	= false;
+	if( !m_Projection.is_Okay() && pLayer->Get_Object()->Get_Projection().is_Okay() )
+	{
+		m_Projection.Create(pLayer->Get_Object()->Get_Projection());
+	}
+
+	bool bRefresh = false;
 
 	for(int i=0; i<Get_Count(); i++)
 	{
 		if( Get_Item(i)->Get_Type() == WKSP_ITEM_Map_Layer && ((CWKSP_Map_Layer *)Get_Item(i))->Get_Layer()->Update(pLayer) )
 		{
-			bRefresh	= true;
+			bRefresh = true;
 
 			if( !bMapOnly )
 			{
@@ -846,7 +847,7 @@ CWKSP_Map_Layer * CWKSP_Map::Get_Map_Layer(CWKSP_Layer *pLayer)
 //---------------------------------------------------------
 CWKSP_Map_Layer * CWKSP_Map::Get_Map_Layer_Active(bool bEditable)
 {
-	CWKSP_Map_Layer	*pLayer	= Get_Map_Layer(Get_Active_Layer());
+	CWKSP_Map_Layer *pLayer = Get_Map_Layer(Get_Active_Layer());
 
 	return( pLayer && !(bEditable && pLayer->is_Projecting()) ? pLayer : NULL );
 }
@@ -862,27 +863,27 @@ CWKSP_Map_Layer * CWKSP_Map::Add_Layer(CWKSP_Layer *pLayer)
 	}
 
 	//-----------------------------------------------------
-	bool	bProject	= false;
+	bool bProject = false;
 
 	if( m_Parameters("CRS_CHECK")->asBool()
 	&&  m_Projection.is_Okay() && pLayer->Get_Object()->Get_Projection().is_Okay()
 	&&  m_Projection.is_Equal(    pLayer->Get_Object()->Get_Projection()) == false )
 	{
-		wxString	s;
+		wxString s;
 
-		s	+= _TL("The coordinate system used by the layer is not identical with the one of the map!");
-		s	+= "\n";
-		s	+= wxString::Format("\n%s:\n  [%s]", _TL("Map"  ),                         m_Projection  .Get_Proj4().c_str());
-		s	+= wxString::Format("\n%s:\n  [%s]", _TL("Layer"), pLayer->Get_Object()->Get_Projection().Get_Proj4().c_str());
-		s	+= "\n\n";
-		s	+= _TL("Do you want to activate on-the-fly projection for this layer in the map?");
-		s	+= "\n";
-		s	+= _TL("(Press cancel if you decide not to add the layer at all!)");
+		s += _TL("The coordinate system used by the layer is not identical with the one of the map!");
+		s += "\n";
+		s += wxString::Format("\n%s:\n  [%s]", _TL("Map"  ),                         m_Projection  .Get_Proj4().c_str());
+		s += wxString::Format("\n%s:\n  [%s]", _TL("Layer"), pLayer->Get_Object()->Get_Projection().Get_Proj4().c_str());
+		s += "\n\n";
+		s += _TL("Do you want to activate on-the-fly projection for this layer in the map?");
+		s += "\n";
+		s += _TL("(Press cancel if you decide not to add the layer at all!)");
 
 		switch( DLG_Message_YesNoCancel(s, _TL("Add Layer to Map")) )
 		{
 		case  0: // yes
-			bProject	= true;
+			bProject = true;
 			break;
 
 		case  1: // no
@@ -895,11 +896,11 @@ CWKSP_Map_Layer * CWKSP_Map::Add_Layer(CWKSP_Layer *pLayer)
 
 	if( !m_Projection.is_Okay() && pLayer->Get_Object()->Get_Projection().is_Okay() )
 	{
-		m_Projection	= pLayer->Get_Object()->Get_Projection();
+		m_Projection = pLayer->Get_Object()->Get_Projection();
 	}
 
 	//-----------------------------------------------------
-	CWKSP_Map_Layer	*pMapLayer	= new CWKSP_Map_Layer(pLayer);
+	CWKSP_Map_Layer *pMapLayer = new CWKSP_Map_Layer(pLayer);
 
 	Add_Item(pMapLayer);
 
@@ -907,7 +908,14 @@ CWKSP_Map_Layer * CWKSP_Map::Add_Layer(CWKSP_Layer *pLayer)
 
 	if( Get_Count() == 1 )
 	{
-		m_Parameters("NAME")->Set_Value(pLayer->Get_Name().wx_str());
+		if( pLayer->Get_Object() )
+		{
+			m_Name = pLayer->Get_Object()->Get_Name();
+		}
+
+		m_Name.Prepend(g_pMaps->Get_Numbering());
+
+		m_Parameters("NAME")->Set_Value(m_Name.wx_str());
 
 		Parameters_Changed();
 	}
@@ -1035,6 +1043,8 @@ bool CWKSP_Map::_Set_Extent(const CSG_Rect &Extent)
 	{
 		View_Refresh(true);
 
+		g_pActive->Update_Description();
+
 		for(int i=0; i<Get_Count(); i++)
 		{
 			if( Get_Item(i)->Get_Type() == WKSP_ITEM_Map_Layer )
@@ -1061,14 +1071,14 @@ bool CWKSP_Map::Set_Extent(const CSG_Rect &_Extent, bool bReset, bool bPan)
 
 	if( Extent.Get_XRange() == 0. )
 	{
-		Extent.m_rect.xMin	-= 1.;
-		Extent.m_rect.xMax	+= 1.;
+		Extent.xMin	-= 1.;
+		Extent.xMax	+= 1.;
 	}
 
 	if( Extent.Get_YRange() == 0. )
 	{
-		Extent.m_rect.yMin	-= 1.;
-		Extent.m_rect.yMax	+= 1.;
+		Extent.yMin	-= 1.;
+		Extent.yMax	+= 1.;
 	}
 
 	if( bPan )
@@ -1106,13 +1116,13 @@ bool CWKSP_Map::Set_Extent(const CSG_Rect &Extent, const CSG_Projection &Project
 //---------------------------------------------------------
 bool CWKSP_Map::Set_Extent_Full(void)
 {
-	CSG_Rect	Extent;
+	CSG_Rect Extent;
 
 	for(int i=0, n=0; i<Get_Count(); i++)
 	{
 		if( Get_Item(i)->Get_Type() == WKSP_ITEM_Map_Layer )
 		{
-			CWKSP_Map_Layer	*pLayer	= (CWKSP_Map_Layer *)Get_Item(i);
+			CWKSP_Map_Layer *pLayer = (CWKSP_Map_Layer *)Get_Item(i);
 
 			if( n++ == 0 )
 			{
@@ -1131,11 +1141,11 @@ bool CWKSP_Map::Set_Extent_Full(void)
 //---------------------------------------------------------
 bool CWKSP_Map::Set_Extent_Active(bool bPan)
 {
-	CWKSP_Layer	*pLayer	= Get_Active_Layer();
+	CWKSP_Layer *pLayer = Get_Active_Layer();
 
 	if( pLayer )
 	{
-		CWKSP_Map_Layer	*pMapLayer	= Get_Map_Layer(pLayer);
+		CWKSP_Map_Layer *pMapLayer = Get_Map_Layer(pLayer);
 
 		if( pMapLayer )
 		{
@@ -1151,11 +1161,11 @@ bool CWKSP_Map::Set_Extent_Active(bool bPan)
 //---------------------------------------------------------
 bool CWKSP_Map::Set_Extent_Selection(bool bPan)
 {
-	CWKSP_Layer	*pLayer	= Get_Active_Layer();
+	CWKSP_Layer *pLayer = Get_Active_Layer();
 
 	if( pLayer )
 	{
-		CWKSP_Map_Layer	*pMapLayer	= Get_Map_Layer(Get_Active_Layer());
+		CWKSP_Map_Layer *pMapLayer = Get_Map_Layer(Get_Active_Layer());
 
 		if( pMapLayer && !pMapLayer->do_Project() )
 		{
@@ -1340,13 +1350,15 @@ void CWKSP_Map::Set_CrossHair_Off(void)
 //---------------------------------------------------------
 void CWKSP_Map::Set_Projection(void)
 {
-	CSG_Tool	*pTool	= SG_Get_Tool_Library_Manager().Create_Tool("pj_proj4", 15, true);	// CCRS_Picker
+	CSG_Tool *pTool = SG_Get_Tool_Library_Manager().Create_Tool("pj_proj4", 15, true);	// CCRS_Picker
 
 	if(	pTool )
 	{
-		CSG_Parameters	P(*pTool->Get_Parameters());
+		CSG_Parameters P(*pTool->Get_Parameters());
 
-		P.Set_Parameter("CRS_PROJ4", m_Projection.Get_Proj4());
+		P.Set_Parameter("CRS_PROJ4"    , m_Projection.Get_Proj4       ());
+		P.Set_Parameter("CRS_EPSG"     , m_Projection.Get_Authority_ID());
+		P.Set_Parameter("CRS_EPSG_AUTH", m_Projection.Get_Authority   ());
 
 		P.Add_Bool("", "ONTHEFLY", _TL("On-The-Fly Projection"),
 			_TL("Turn on the on-the-fly projection for all layers in the map."),
@@ -1356,7 +1368,12 @@ void CWKSP_Map::Set_Projection(void)
 		if(	DLG_Parameters(&P) && pTool->Get_Parameters()->Assign_Values(&P)
 		&&  pTool->Set_Manager(NULL) && pTool->On_Before_Execution() && pTool->Execute() )
 		{
-			CSG_Projection	Projection(pTool->Get_Parameter("CRS_PROJ4")->asString(), SG_PROJ_FMT_Proj4);
+			CSG_Projection Projection; CSG_Parameters &_P = *pTool->Get_Parameters();
+
+			if( _P["CRS_EPSG"].asInt() < 0 || !Projection.Create(_P["CRS_EPSG"].asInt(), _P["CRS_EPSG_AUTH"].asString()) )
+			{
+				Projection.Create(_P["CRS_PROJ4"].asString(), SG_PROJ_FMT_Proj4);
+			}
 
 			if( P("ONTHEFLY")->asBool() )
 			{
@@ -1368,9 +1385,9 @@ void CWKSP_Map::Set_Projection(void)
 					}
 				}
 
-				CSG_Rect	r(Get_Extent());
+				CSG_Rect r(Get_Extent());
 
-				SG_Get_Projected(m_Projection, Projection, r.m_rect);
+				SG_Get_Projected(m_Projection, Projection, r);
 
 				Set_Extent(r, true);
 			}
@@ -1426,10 +1443,7 @@ void CWKSP_Map::View_Refresh(bool bMapOnly)
 
 	_Set_Thumbnail();
 
-	if( g_pMap_Buttons )
-	{
-		g_pMap_Buttons->Refresh();
-	}
+	if( g_pMap_Buttons ) { g_pMap_Buttons->Refresh(); }
 }
 
 //---------------------------------------------------------
@@ -1506,36 +1520,31 @@ void CWKSP_Map::View_Layout_Toggle(void)
 //---------------------------------------------------------
 CSG_Rect CWKSP_Map::Get_World(wxRect rClient)
 {
-	double		d, dWorld, dClient;
-	TSG_Rect	Extent;
+	CSG_Rect Extent(Get_Extent());
 
-	Extent	= Get_Extent().m_rect;
-
-	dClient	= (double)rClient.GetHeight() / (double)rClient.GetWidth();
-	dWorld	= Get_Extent().Get_YRange() / Get_Extent().Get_XRange();
+	double dClient = (double)rClient.GetHeight() / (double)rClient.GetWidth();
+	double dWorld  = Get_Extent().Get_YRange() / Get_Extent().Get_XRange();
 
 	if( dWorld > dClient )
 	{
-		d			= (Get_Extent().Get_XRange() - Get_Extent().Get_YRange() / dClient) / 2.;
-		Extent.xMin	+= d;
-		Extent.xMax	-= d;
+		double d = (Get_Extent().Get_XRange() - Get_Extent().Get_YRange() / dClient) / 2.;
+		Extent.xMin += d;
+		Extent.xMax -= d;
 	}
 	else
 	{
-		d			= (Get_Extent().Get_YRange() - Get_Extent().Get_XRange() * dClient) / 2.;
-		Extent.yMin	+= d;
-		Extent.yMax	-= d;
+		double d = (Get_Extent().Get_YRange() - Get_Extent().Get_XRange() * dClient) / 2.;
+		Extent.yMin += d;
+		Extent.yMax -= d;
 	}
 
-	return( CSG_Rect(Extent) );
+	return( Extent );
 }
 
 //---------------------------------------------------------
 CSG_Point CWKSP_Map::Get_World(wxRect rClient, wxPoint Point)
 {
-	CSG_Rect	rWorld(Get_World(rClient));
-
-	double	d	= rWorld.Get_XRange() / (double)rClient.GetWidth();
+	CSG_Rect rWorld(Get_World(rClient)); double d = rWorld.Get_XRange() / (double)rClient.GetWidth();
 
 	return( CSG_Point(
 		rWorld.Get_XMin() + d *                        Point.x,
@@ -1554,19 +1563,14 @@ CSG_Point CWKSP_Map::Get_World(wxRect rClient, wxPoint Point)
 #define MASK_B	255
 
 //---------------------------------------------------------
-bool CWKSP_Map::Get_Image(wxImage &Image, CSG_Rect &rWorld)
+bool CWKSP_Map::Get_Image(wxImage &Image, const CSG_Grid_System &System)
 {
-	if( Image.GetWidth() > 0 && Image.GetHeight() > 0 )
+	if( System.is_Valid() && Image.Create(System.Get_NX(), System.Get_NY()) )
 	{
-		wxBitmap	BMP(Image);
-		wxMemoryDC	dc;
-
-		dc.SelectObject(BMP);
-		Draw_Map(dc, 1., wxRect(0, 0, Image.GetWidth(), Image.GetHeight()), LAYER_DRAW_FLAG_NOEDITS, SG_GET_RGB(MASK_R, MASK_G, MASK_B));
+		wxBitmap BMP(Image); wxMemoryDC dc(BMP);
+		Draw_Map(dc, 1., Image.GetSize(), LAYER_DRAW_FLAG_NOEDITS, SG_GET_RGB(MASK_R, MASK_G, MASK_B));
 		dc.SelectObject(wxNullBitmap);
-
-		rWorld	= Get_World(wxRect(0, 0, Image.GetWidth(), Image.GetHeight()));
-		Image	= BMP.ConvertToImage();
+		Image = BMP.ConvertToImage();
 		Image.SetMaskColour(MASK_R, MASK_G, MASK_B);
 
 		return( true );
@@ -1576,12 +1580,30 @@ bool CWKSP_Map::Get_Image(wxImage &Image, CSG_Rect &rWorld)
 }
 
 //---------------------------------------------------------
-void CWKSP_Map::SaveAs_Image(void)
+bool CWKSP_Map::Get_Image(wxImage &Image, CSG_Rect &rWorld)
+{
+	if( Image.GetWidth() > 0 && Image.GetHeight() > 0 )
+	{
+		wxBitmap BMP(Image); wxMemoryDC dc(BMP);
+		Draw_Map(dc, 1., Image.GetSize(), LAYER_DRAW_FLAG_NOEDITS, SG_GET_RGB(MASK_R, MASK_G, MASK_B));
+		dc.SelectObject(wxNullBitmap);
+		Image = BMP.ConvertToImage();
+		Image.SetMaskColour(MASK_R, MASK_G, MASK_B);
+
+		rWorld = Get_World(Image.GetSize());
+
+		return( true );
+	}
+
+	return( false );
+}
+
+//---------------------------------------------------------
+bool CWKSP_Map::SaveAs_Image(void)
 {
-	//-----------------------------------------------------
 	if( View_Get() && View_Get()->Get_Map_Control() )
 	{
-		wxSize	s(View_Get()->Get_Map_Control()->GetClientSize());
+		wxSize s(View_Get()->Get_Map_Control()->GetClientSize());
 
 		m_Img_Parms("WIDTH" )->Set_Value(s.x);
 		m_Img_Parms("HEIGHT")->Set_Value(s.y);
@@ -1590,22 +1612,24 @@ void CWKSP_Map::SaveAs_Image(void)
 	if( DLG_Image_Save(m_Img_File, m_Img_Type) && DLG_Parameters(&m_Img_Parms) )
 	{
 		_Img_Save(m_Img_File, m_Img_Type);
+
+		return( true );
 	}
+
+	return( false );
 }
 
 //---------------------------------------------------------
-void CWKSP_Map::SaveAs_Image_Clipboard(bool bLegend)
+bool CWKSP_Map::SaveAs_Image_Clipboard(bool bLegend)
 {
 	if( bLegend == false )
 	{
-		SaveAs_Image_Clipboard(
+		return( SaveAs_Image_Clipboard(
 			Get_Manager()->Get_Parameter("CLIP_NX"         )->asInt (),
 			Get_Manager()->Get_Parameter("CLIP_NY"         )->asInt (),
 			Get_Manager()->Get_Parameter("CLIP_FRAME_SHOW" )->asBool() ?
-			Get_Manager()->Get_Parameter("CLIP_FRAME_WIDTH")->asInt () : 0
+			Get_Manager()->Get_Parameter("CLIP_FRAME_WIDTH")->asInt () : 0)
 		);
-
-		return;
 	}
 
 	//-----------------------------------------------------
@@ -1613,21 +1637,18 @@ void CWKSP_Map::SaveAs_Image_Clipboard(bool bLegend)
 
 	Set_Buisy_Cursor(true);
 
-	int			Frame	= Get_Manager()->Get_Parameter("CLIP_LEGEND_FRAME")->asInt();
-	double		Scale	= Get_Manager()->Get_Parameter("CLIP_LEGEND_SCALE")->asDouble();
-	wxSize		s;
-	wxBitmap	BMP;
-	wxMemoryDC	dc;
+	int    Frame = Get_Manager()->Get_Parameter("CLIP_LEGEND_FRAME")->asInt   ();
+	double Scale = Get_Manager()->Get_Parameter("CLIP_LEGEND_SCALE")->asDouble();
+
+	wxSize s;
 
 	if( Get_Legend_Size(s, 1., Scale) )
 	{
-		s.x	+= 2 * Frame;
-		s.y	+= 2 * Frame;
+		s.x += 2 * Frame; s.y += 2 * Frame;
 
-		BMP.Create(s.GetWidth(), s.GetHeight());
-		dc.SelectObject(BMP);
-		dc.SetBackground(*wxWHITE_BRUSH);
-		dc.Clear();
+		wxBitmap BMP(s.GetWidth(), s.GetHeight());
+
+		wxMemoryDC dc(BMP); dc.SetBackground(*wxWHITE_BRUSH); dc.Clear();
 
 		if( Frame > 0 )
 		{
@@ -1641,7 +1662,7 @@ void CWKSP_Map::SaveAs_Image_Clipboard(bool bLegend)
 
 		if( wxTheClipboard->Open() )
 		{
-			wxBitmapDataObject	*pBMP	= new wxBitmapDataObject;
+			wxBitmapDataObject *pBMP = new wxBitmapDataObject;
 			pBMP->SetBitmap(BMP);
 			wxTheClipboard->SetData(pBMP);
 			wxTheClipboard->Close();
@@ -1649,28 +1670,24 @@ void CWKSP_Map::SaveAs_Image_Clipboard(bool bLegend)
 	}
 
 	Set_Buisy_Cursor(false);
+
+	return( true );
 }
 
 //---------------------------------------------------------
-void CWKSP_Map::SaveAs_Image_Clipboard(int nx, int ny, int frame)
+bool CWKSP_Map::SaveAs_Image_Clipboard(int nx, int ny, int frame)
 {
 	Set_Buisy_Cursor(true);
 
-	wxSize		s;
-	wxRect		r;
-	wxBitmap	BMP;
-	wxMemoryDC	dc;
+	if( frame < 10 ) frame = 0; // if( frame <  0 ) frame = Get_Frame_Width();
 
-//	if( frame <  0 ) frame = Get_Frame_Width();
-	if( frame < 10 ) frame = 0;
+	wxRect r = wxRect(0, 0, nx + 2 * frame, ny + 2 * frame);
 
-	r		= wxRect(0, 0, nx + 2 * frame, ny + 2 * frame);
+	wxBitmap BMP(r.GetWidth(), r.GetHeight());
 
-	BMP.Create(r.GetWidth(), r.GetHeight());
 	r.Deflate(frame);
-	dc.SelectObject(BMP);
-	dc.SetBackground(*wxWHITE_BRUSH);
-	dc.Clear();
+
+	wxMemoryDC dc(BMP); dc.SetBackground(*wxWHITE_BRUSH); dc.Clear();
 
 	Draw_Map(dc, 1., r, LAYER_DRAW_FLAG_NOEDITS);
 
@@ -1683,38 +1700,63 @@ void CWKSP_Map::SaveAs_Image_Clipboard(int nx, int ny, int frame)
 
 	if( wxTheClipboard->Open() )
 	{
-		wxBitmapDataObject	*pBMP	= new wxBitmapDataObject;
+		wxBitmapDataObject *pBMP = new wxBitmapDataObject;
 		pBMP->SetBitmap(BMP);
 		wxTheClipboard->SetData(pBMP);
 		wxTheClipboard->Close();
 	}
 
 	Set_Buisy_Cursor(false);
+
+	return( true );
 }
 
 //---------------------------------------------------------
-void CWKSP_Map::SaveAs_Image_To_KMZ(int nx, int ny)
+int _On_Parameter_Changed(CSG_Parameter *pParameter, int Flags)
+{
+	CSG_Parameters *pParameters = pParameter ? pParameter->Get_Parameters() : NULL; if( !pParameters ) { return( 0 ); }
+
+	if( Flags & PARAMETER_CHECK_VALUES )
+	{
+		CSG_Parameters_Grid_Target::On_Parameter_Changed(pParameters, pParameter, "TARGET_");
+	}
+
+	if( Flags & PARAMETER_CHECK_ENABLE )
+	{
+		CSG_Parameters_Grid_Target::On_Parameters_Enable(pParameters, pParameter, "TARGET_");
+	}
+
+	return( 1 );
+}
+
+//---------------------------------------------------------
+bool CWKSP_Map::SaveAs_Image_To_KMZ(int nx, int ny)
 {
 	if( nx < 1 || ny < 1 )
 	{
-		return;
+		return( false );
 	}
 
-	//-----------------------------------------------------
-	CSG_Rect		Extent(Get_Extent());
+	CSG_Grid_System System(Get_Extent().Get_XRange() / nx, Get_Extent());
 
-	CSG_Parameters	P(_TL("Export Map to Google Earth"));
+	CSG_Parameters P(_TL("Export Map to Google Earth"), SG_T(""), SG_T("SAVE_IMAGE"));
+	P.Set_Callback_On_Parameter_Changed(_On_Parameter_Changed);
 
 	P.Add_FilePath("", "FILE"    , _TL("File"    ), _TL(""), CSG_String::Format("%s|*.kmz|%s|*.*", _TL("KMZ Files"), _TL("All Files")), NULL, true);
-	P.Add_Double  ("", "CELLSIZE", _TL("Cellsize"), _TL(""), SG_Get_Rounded_To_SignificantFigures(Extent.Get_XRange() / (double)nx, 2), 0., true);
 	P.Add_Bool    ("", "LOAD"    , _TL("Load"    ), _TL(""), true);
 
-	if( !DLG_Parameters(&P) || P("CELLSIZE")->asDouble() <= 0. )
+	CSG_Parameters_Grid_Target Target;
+	Target.Create(&P, true, "", "TARGET_");
+	Target.Set_User_Defined(&P, System);
+
+	if( !DLG_Parameters(&P) )
 	{
-		return;
+		return( false );
 	}
 
-	wxFileName	FileName(P("FILE")->asString());
+	System = Target.Get_System();
+
+	wxFileName FileName(P["FILE"].asString());
 
 	if( !FileName.IsOk() )
 	{
@@ -1725,23 +1767,20 @@ void CWKSP_Map::SaveAs_Image_To_KMZ(int nx, int ny)
 		{
 			DLG_Message_Show_Error(_TL("invalid file name"), _TL("Export Map to Google Earth"));
 
-			return;
+			return( false );
 		}
 	}
 
 	//-----------------------------------------------------
-	nx	= Extent.Get_XRange() / P("CELLSIZE")->asDouble();
-	ny	= Extent.Get_YRange() / P("CELLSIZE")->asDouble();
+	wxImage Image;
 
-	wxImage		Image(nx, ny);
-
-	if( !Get_Image(Image, Extent) )
+	if( !Get_Image(Image, System) )
 	{
-		return;
+		return( false );
 	}
 
 	//-----------------------------------------------------
-	CSG_Grid	Map(SG_DATATYPE_Int, Image.GetWidth(), Image.GetHeight(), Extent.Get_XRange() / (double)Image.GetWidth(), Extent.Get_XMin(), Extent.Get_YMin());
+	CSG_Grid Map(System, SG_DATATYPE_Int);
 
 	Map.Set_Name(Get_Name().wx_str());
 	Map.Set_NoData_Value(SG_GET_RGB(MASK_R, MASK_G, MASK_B));
@@ -1756,77 +1795,101 @@ void CWKSP_Map::SaveAs_Image_To_KMZ(int nx, int ny)
 	}
 
 	//-----------------------------------------------------
-	CSG_Tool	*pTool	= SG_Get_Tool_Library_Manager().Create_Tool("io_grid_image", 2, true);
+	CSG_Tool *pTool = SG_Get_Tool_Library_Manager().Create_Tool("io_grid_image", 2, true);
 
 	if(	pTool && pTool->Settings_Push()
 	&&  pTool->Set_Parameter("GRID"     , &Map)
 	&&  pTool->Set_Parameter("FILE"     , FileName.GetFullPath().wc_str())
 	&&  pTool->Set_Parameter("COLOURING", 4)	// rgb coded values
 	&&  pTool->Set_Parameter("OUTPUT"   , 2)	// kmz file
-	&&  pTool->Execute() && P("LOAD")->asBool() )
+	&&  pTool->Execute() )
 	{
-		Open_Application(FileName.GetFullPath());
+		if( P["LOAD"].asBool() )
+		{
+			Open_Application(FileName.GetFullPath());
+		}
+
+		SG_Get_Tool_Library_Manager().Delete_Tool(pTool);
+
+		return( true );
 	}
 
-	SG_Get_Tool_Library_Manager().Delete_Tool(pTool);\
+	SG_Get_Tool_Library_Manager().Delete_Tool(pTool);
+
+	return( false );
 }
 
 //---------------------------------------------------------
-void CWKSP_Map::SaveAs_Image_To_Memory(int nx, int ny)
+bool CWKSP_Map::SaveAs_Image_To_Memory(int nx, int ny)
 {
 	if( nx < 1 || ny < 1 )
-		return;
-
-	CSG_Rect		Extent(Get_Extent());
-
-	CSG_Parameters	P(_TL("Save To Memory Grid"));
-
-	P.Add_Double("", "CELLSIZE", _TL("Cellsize"), _TL(""), Extent.Get_XRange() / (double)nx, 0., true);
-
-	if( !DLG_Parameters(&P) || P("CELLSIZE")->asDouble() <= 0. )
-		return;
-
-	nx	= Extent.Get_XRange() / P("CELLSIZE")->asDouble();
-	ny	= Extent.Get_YRange() / P("CELLSIZE")->asDouble();
-
-	wxImage		Image(nx, ny);
-
-	if( Get_Image(Image, Extent) )
 	{
-		CSG_Grid	*pGrid	= SG_Create_Grid(SG_DATATYPE_Int, Image.GetWidth(), Image.GetHeight(), Extent.Get_XRange() / (double)Image.GetWidth(), Extent.Get_XMin(), Extent.Get_YMin());
+		return( false );
+	}
 
-		pGrid->Set_Name(Get_Name().wx_str());
-		pGrid->Set_NoData_Value(16711935);
-		pGrid->Get_Projection().Create(m_Projection);
+	CSG_Grid_System System(Get_Extent().Get_XRange() / nx, Get_Extent());
 
-		for(int y=0, yy=pGrid->Get_NY()-1; y<pGrid->Get_NY(); y++, yy--)
+	CSG_Parameters P(_TL("Save To Memory Grid"), SG_T(""), SG_T("SAVE_IMAGE"));
+	P.Set_Callback_On_Parameter_Changed(_On_Parameter_Changed);
+
+	CSG_Parameters_Grid_Target Target;
+	Target.Create(&P, true, "", "TARGET_");
+	Target.Set_User_Defined(&P, System);
+
+	if( !DLG_Parameters(&P) )
+	{
+		return( false );
+	}
+
+	System = Target.Get_System();
+
+	//-----------------------------------------------------
+	wxImage Image;
+
+	if( !Get_Image(Image, System) )
+	{
+		return( false );
+	}
+
+	//-----------------------------------------------------
+	CSG_Grid *pMap = SG_Create_Grid(System, SG_DATATYPE_Int);
+
+	pMap->Set_Name(Get_Name().wx_str());
+	pMap->Set_NoData_Value(16711935);
+	pMap->Get_Projection().Create(m_Projection);
+
+	for(int y=0, yy=pMap->Get_NY()-1; y<pMap->Get_NY(); y++, yy--)
+	{
+		for(int x=0; x<pMap->Get_NX(); x++)
 		{
-			for(int x=0; x<pGrid->Get_NX(); x++)
-			{
-				pGrid->Set_Value(x, y, SG_GET_RGB(Image.GetRed(x, yy), Image.GetGreen(x, yy), Image.GetBlue(x, yy)));
-			}
-		}
-
-		g_pData->Add(pGrid);
-		g_pData->Get_Parameters(pGrid, &P);
-
-		if( P("COLORS_TYPE") )
-		{
-			P("COLORS_TYPE")->Set_Value(5);	// Color Classification Type: RGB Coded Values
-
-			g_pData->Set_Parameters(pGrid, &P);
+			pMap->Set_Value(x, y, SG_GET_RGB(Image.GetRed(x, yy), Image.GetGreen(x, yy), Image.GetBlue(x, yy)));
 		}
 	}
+
+	//-----------------------------------------------------
+	g_pData->Add(pMap);
+
+	g_pData->Get_Parameters(pMap, &P);
+
+	if( P("COLORS_TYPE") )
+	{
+		P("COLORS_TYPE")->Set_Value(5);	// Color Classification Type: RGB Coded Values
+
+		g_pData->Set_Parameters(pMap, &P);
+	}
+
+	return( true );
 }
 
 //---------------------------------------------------------
-void CWKSP_Map::SaveAs_Image_To_Grid(CSG_Grid &Grid, int Size)
+bool CWKSP_Map::SaveAs_Image_To_Grid(CSG_Grid &Grid, int Size)
 {
 	if( Size < 1 )
-		return;
+	{
+		return( false );
+	}
 
-	CSG_Rect	Extent(Get_Extent());
-	wxImage		Image;
+	CSG_Rect Extent(Get_Extent()); wxImage Image;
 
 	if( Extent.Get_XRange() > Extent.Get_YRange() )
 	{
@@ -1850,10 +1913,12 @@ void CWKSP_Map::SaveAs_Image_To_Grid(CSG_Grid &Grid, int Size)
 			}
 		}
 	}
+
+	return( true );
 }
 
 //---------------------------------------------------------
-void CWKSP_Map::SaveAs_Image_On_Change(void)
+bool CWKSP_Map::SaveAs_Image_On_Change(void)
 {
 	if( m_Img_bSave )
 	{
@@ -1864,6 +1929,8 @@ void CWKSP_Map::SaveAs_Image_On_Change(void)
 		m_Img_bSave	= true;
 		m_Img_Count	= 0;
 	}
+
+	return( true );
 }
 
 //---------------------------------------------------------
