@@ -73,12 +73,12 @@
 //---------------------------------------------------------
 CWKSP_Map_Layer::CWKSP_Map_Layer(CWKSP_Layer *pLayer)
 {
-	m_pLayer		= pLayer;
+	m_pLayer        = pLayer;
 
-	m_bShow			= true;
-	m_bProject		= false;
-	m_bProject_Area	= false;
-	m_bFitColors	= false;
+	m_bShow         = true;
+	m_bProject      = false;
+	m_bProject_Area = false;
+	m_bFitColors    = false;
 }
 
 
@@ -89,7 +89,7 @@ CWKSP_Map_Layer::CWKSP_Map_Layer(CWKSP_Layer *pLayer)
 //---------------------------------------------------------
 wxString CWKSP_Map_Layer::Get_Name(void)
 {
-	wxString	Name(m_pLayer->Get_Name());
+	wxString Name(m_pLayer->Get_Name().Trim(false));
 
 	return( !m_bShow ? "* " + Name : Name );
 }
@@ -350,7 +350,7 @@ bool CWKSP_Map_Layer::Fit_Colors(const CSG_Rect &rWorld)
 
 		if( m_bProject && _Projected_Get_Projections(prj_Layer, prj_Map) )
 		{
-			SG_Get_Projected(prj_Map, prj_Layer, _rWorld.m_rect);
+			SG_Get_Projected(prj_Map, prj_Layer, _rWorld);
 		}
 
 		switch( m_pLayer->Get_Type() )
@@ -535,9 +535,9 @@ bool CWKSP_Map_Layer::_Projected_Shapes_Clipped(const CSG_Rect &rMap, CSG_Shapes
 	{
 		CSG_Shapes	Clip(SHAPE_TYPE_Polygon);	CSG_Shape	*pClip	= Clip.Add_Shape();
 
-		for(int i=0; i<Extent.Get_Count(); i++)
+		for(sLong i=0; i<Extent.Get_Count(); i++)
 		{
-			TSG_Point	p	= Extent.Get_Shape(i)->Get_Point(0);
+			TSG_Point	p	= Extent.Get_Shape(i)->Get_Point();
 
 			if( std::isinf(p.x) ) p.x = std::signbit(p.x) ? Extent.Get_Extent().Get_XMin() : Extent.Get_Extent().Get_XMax();
 			if( std::isinf(p.y) ) p.y = std::signbit(p.y) ? Extent.Get_Extent().Get_YMin() : Extent.Get_Extent().Get_YMax();
@@ -553,7 +553,7 @@ bool CWKSP_Map_Layer::_Projected_Shapes_Clipped(const CSG_Rect &rMap, CSG_Shapes
 		{
 			Shapes.Create(pShapes->Get_Type(), SG_T(""), pShapes);	Shapes.Get_Projection().Create(prj_Layer);
 
-			for(int i=0; i<pShapes->Get_Count(); i++)
+			for(sLong i=0; i<pShapes->Get_Count(); i++)
 			{
 				if( pClip->Intersects(pShapes->Get_Shape(i)) )
 				{
@@ -659,7 +659,7 @@ bool CWKSP_Map_Layer::Draw(CWKSP_Map_DC &dc_Map, int Flags)
 		{
 			CSG_Shapes	Selection(pShapes->Get_Type(), pShapes->Get_Name(), pShapes);
 
-			for(int i=0; i<pShapes->Get_Selection_Count(); i++)
+			for(sLong i=0; i<pShapes->Get_Selection_Count(); i++)
 			{
 				Selection.Add_Shape(pShapes->Get_Selection(i));
 			}
@@ -676,7 +676,7 @@ bool CWKSP_Map_Layer::Draw(CWKSP_Map_DC &dc_Map, int Flags)
 		Points.Create(pPoints);
 		Points.Get_Projection().Create(pPoints->Get_Projection());
 
-		for(int i=0; i<pPoints->Get_Count(); i++)
+		for(sLong i=0; i<pPoints->Get_Count(); i++)
 		{
 			if( rLayer.Contains(pPoints->Get_X(i), pPoints->Get_Y(i)) )
 			{
