@@ -150,7 +150,7 @@ TSG_Rect CWKSP_Shapes::Edit_Get_Extent(void)
 
 	if( Get_Shapes()->Get_Selection_Count() > 0 )
 	{
-		return( Get_Shapes()->Get_Selection_Extent().m_rect );
+		return( Get_Shapes()->Get_Selection_Extent() );
 	}
 
 	return( Get_Shapes()->Get_Extent() );
@@ -509,7 +509,7 @@ bool CWKSP_Shapes::Edit_Set_Attributes(void)
 	{
 		int	Decimals;	GET_TABLE_NUM_PREC(Decimals);
 
-		for(int i=0; i<m_Edit_Attributes.Get_Record_Count(); i++)
+		for(int i=0; i<m_Edit_Attributes.Get_Count(); i++)
 		{
 			if( !pSelection->Set_Value(i, m_Edit_Attributes.Get_Record(i)->asString(1)) )
 			{
@@ -543,7 +543,7 @@ bool CWKSP_Shapes::_Edit_Merge(void)
 	//-----------------------------------------------------
 	CSG_Shape	*pMerged	= Get_Shapes()->Get_Selection(0);
 
-	for(int i=1; i<Get_Shapes()->Get_Selection_Count(); i++)
+	for(sLong i=1; i<Get_Shapes()->Get_Selection_Count(); i++)
 	{
 		CSG_Shape	*pShape	= Get_Shapes()->Get_Selection(i);
 
@@ -649,7 +649,7 @@ bool CWKSP_Shapes::_Edit_Split(void)
 					{
 						m_Edit_pShape->Assign(Split.Get_Shape(0), false);
 
-						for(int iSplit=1; iSplit<Split.Get_Count(); iSplit++)
+						for(sLong iSplit=1; iSplit<Split.Get_Count(); iSplit++)
 						{
 							CSG_Shape	*pSplit	= Split.Get_Shape(iSplit);
 
@@ -668,7 +668,7 @@ bool CWKSP_Shapes::_Edit_Split(void)
 						
 						pSelection->Assign(Split.Get_Shape(0), false);
 
-						for(int iSplit=1; iSplit<Split.Get_Count(); iSplit++)
+						for(sLong iSplit=1; iSplit<Split.Get_Count(); iSplit++)
 						{
 							CSG_Shape	*pSplit	= Get_Shapes()->Add_Shape(Split.Get_Shape(iSplit));
 
@@ -736,11 +736,11 @@ bool CWKSP_Shapes::_Edit_Move(bool bToggle)
 			if( m_Edit_Shapes.Get_Count() > 1 && m_Edit_Shapes.Get_Shape(1)->Get_Point_Count() > 1 )
 			{
 				CSG_Point	Move	= CSG_Point(m_Edit_Shapes.Get_Shape(1)->Get_Point(1))
-									- CSG_Point(m_Edit_Shapes.Get_Shape(1)->Get_Point(0));
+									- CSG_Point(m_Edit_Shapes.Get_Shape(1)->Get_Point());
 
 				m_Edit_Shapes.Get_Shape(1)->Del_Parts();
 
-				if( SG_Get_Length(Move.Get_X(), Move.Get_Y()) > 0.0 )
+				if( SG_Get_Length(Move.x, Move.y) > 0.0 )
 				{
 					for(int iPart=0; iPart<m_Edit_pShape->Get_Part_Count(); iPart++)
 					{
@@ -1150,9 +1150,9 @@ void CWKSP_Shapes::_Edit_Snap_Point(const CSG_Point &Point, CSG_Point &snap_Poin
 {
 	CSG_Shape	*pSelected	= pShapes->Get_Selection(m_Edit_Index);
 
-	if( pShapes->Select(CSG_Rect(Point.Get_X() - snap_Dist, Point.Get_Y() - snap_Dist, Point.Get_X() + snap_Dist, Point.Get_Y() + snap_Dist)) )
+	if( pShapes->Select(CSG_Rect(Point.x - snap_Dist, Point.y - snap_Dist, Point.x + snap_Dist, Point.y + snap_Dist)) )
 	{
-		for(int i=0; i<pShapes->Get_Selection_Count(); i++)
+		for(sLong i=0; i<pShapes->Get_Selection_Count(); i++)
 		{
 			if( pShapes != Get_Shapes() || pSelected != pShapes->Get_Selection(i) )
 			{
@@ -1183,8 +1183,8 @@ void CWKSP_Shapes::_Edit_Snap_Point(const CSG_Point &pos_Point, CSG_Point &snap_
 		for(iPoint=0; iPoint<pShape->Get_Point_Count(iPart); iPoint++)
 		{
 			Point	= pShape->Get_Point(iPoint, iPart);
-			dx		= pos_Point.Get_X() - Point.x;
-			dy		= pos_Point.Get_Y() - Point.y;
+			dx		= pos_Point.x - Point.x;
+			dy		= pos_Point.y - Point.y;
 			d		= sqrt(dx*dx + dy*dy);
 
 			if( d < snap_Dist )
