@@ -861,13 +861,11 @@ bool CGrid_Levels_to_Points::On_Execute(void)
 
 	//-----------------------------------------------------
 //	#pragma omp parallel for
-	for(int iPoint=0; iPoint<pPoints->Get_Count() && Set_Progress(iPoint, pPoints->Get_Count()); iPoint++)
+	for(sLong iPoint=0; iPoint<pPoints->Get_Count() && Set_Progress(iPoint, pPoints->Get_Count()); iPoint++)
 	{
-		CSG_Shape	*pPoint	= pPoints->Get_Shape(iPoint);
+		CSG_Shape *pPoint = pPoints->Get_Shape(iPoint); double Value;
 
-		double	Value;
-
-		if( !pPoint->is_NoData(zField) && Get_Value(pPoint->Get_Point(0), pPoint->asDouble(zField), Value) )
+		if( !pPoint->is_NoData(zField) && Get_Value(pPoint->Get_Point(), pPoint->asDouble(zField), Value) )
 		{
 			pPoint->Set_Value(vField, Value);
 		}
