@@ -248,12 +248,12 @@ TSG_Data_Type CShapes2Grid::Get_Data_Type(int Field)
 //---------------------------------------------------------
 bool CShapes2Grid::On_Execute(void)
 {
-	CSG_Shapes	*pShapes	= Parameters("INPUT")->asShapes();
+	CSG_Shapes *pShapes = Parameters("INPUT")->asShapes();
 
-	m_Multiple	= Parameters("MULTIPLE")->asInt();
+	m_Multiple = Parameters("MULTIPLE")->asInt();
 
 	//-----------------------------------------------------
-	bool	bFat;
+	bool bFat;
 
 	switch( pShapes->Get_Type() )
 	{
@@ -263,17 +263,35 @@ bool CShapes2Grid::On_Execute(void)
 	}
 
 	//-----------------------------------------------------
-	int		Field;
+	int Field; CSG_String Name(pShapes->Get_Name());
 
 	switch( Parameters("OUTPUT")->asInt() )
 	{
-	case  0: Field = OUTPUT_NODATA;         break;	// data / no-data
-	case  1: Field = OUTPUT_INDEX ;         break;	// index number
-	default: Field = Parameters("FIELD")->asInt();	// attribute
-		if( Field < 0 || !SG_Data_Type_is_Numeric(pShapes->Get_Field_Type(Field)) )
+	case  0: // data / no-data
+		Field = OUTPUT_NODATA;
+		break;
+
+	case  1: // index number
+		Field = OUTPUT_INDEX;
+		Name += " [ID]";
+		break;
+
+	default: // attribute
+		Field = Parameters("FIELD")->asInt();
+
+		if( Field < 0 )
+		{
+			Error_Set(_TL("Field needs to be specified!"));
+
+			return( false );
+		}
+
+		if( !SG_Data_Type_is_Numeric(pShapes->Get_Field_Type(Field)) )
 		{
 			Message_Add(_TL("WARNING: selected attribute is not numeric."));
 		}
+
+		Name += CSG_String::Format(" [%s, %s]", pShapes->Get_Field_Name(Field), Parameters("MULTIPLE")->asString());
 		break;
 	}
 
@@ -295,7 +313,7 @@ bool CShapes2Grid::On_Execute(void)
 		m_pGrid->Set_NoData_Value(0.);
 	}
 
-	m_pGrid->Fmt_Name("%s [%s]", pShapes->Get_Name(), Field < 0 ? _TL("ID") : pShapes->Get_Field_Name(Field));
+	m_pGrid->Set_Name(Name);
 	m_pGrid->Assign_NoData();
 
 	//-------------------------------------------------
@@ -315,9 +333,9 @@ bool CShapes2Grid::On_Execute(void)
 	m_pCount->Assign(0.);
 
 	//-----------------------------------------------------
-	for(int i=0; i<pShapes->Get_Count() && Set_Progress(i, pShapes->Get_Count()); i++)
+	for(sLong i=0; i<pShapes->Get_Count() && Set_Progress(i, pShapes->Get_Count()); i++)
 	{
-		CSG_Shape	*pShape	= pShapes->Get_Shape(i);
+		CSG_Shape *pShape = pShapes->Get_Shape(i);
 
 		m_Cells_On_Shape.clear();
 
@@ -667,7 +685,7 @@ void CShapes2Grid::Set_Polygon(CSG_Shape_Polygon *pPolygon, double Value)
 	//-----------------------------------------------------
 	for(int y=0; y<m_pGrid->Get_NY(); y++, A.y+=m_pGrid->Get_Cellsize())
 	{
-		if( A.y >= Extent.m_rect.yMin && A.y <= Extent.m_rect.yMax )
+		if( A.y >= Extent.yMin && A.y <= Extent.yMax )
 		{
 			B.y	= A.y;
 
@@ -939,9 +957,9 @@ bool CPolygons2Grid::On_Execute(void)
 	m_pCoverage->Assign(0.);
 
 	//-----------------------------------------------------
-	for(int i=0; i<pPolygons->Get_Count() && Set_Progress(i, pPolygons->Get_Count()); i++)
+	for(sLong i=0; i<pPolygons->Get_Count() && Set_Progress(i, pPolygons->Get_Count()); i++)
 	{
-		CSG_Shape_Polygon	*pPolygon	= (CSG_Shape_Polygon *)pPolygons->Get_Shape(i);
+		CSG_Shape_Polygon *pPolygon = (CSG_Shape_Polygon *)pPolygons->Get_Shape(i);
 
 		if( pPolygons->Get_Selection_Count() <= 0 || pPolygon->is_Selected() )
 		{
@@ -949,7 +967,7 @@ bool CPolygons2Grid::On_Execute(void)
 			{
 				if( pPolygon->Intersects(m_pGrid->Get_Extent()) )
 				{
-					Set_Polygon(pPolygon, Field < 0 ? i + 1 : pPolygon->asDouble(Field));
+					Set_Polygon(pPolygon, Field < 0 ? i + 1. : pPolygon->asDouble(Field));
 				}
 			}
 		}
