@@ -88,7 +88,7 @@ CPanSharp_IHS::CPanSharp_IHS(void)
 	Add_Reference("Haydn, R., Dalke, G. W., Henkel, J., Bare, J. E.", "1982",
 		"Application of the IHS color transform to the processing of multisensor data and image enhancement",
 		"Proceedings of the International Symposium on Remote Sensing of Arid and Semi-Arid Lands, "
-		"Cairo, Egypt (Environmental Research Institute, Ann Arbor, Mich., 1982), pp. 599616."
+		"Cairo, Egypt (Environmental Research Institute, Ann Arbor, Mich., 1982), pp. 599–616."
 	);
 
 	//-----------------------------------------------------
@@ -506,7 +506,7 @@ bool CPanSharp_CN::On_Execute(void)
 	}
 
 	//-----------------------------------------------------
-	for(int y=0; y<Get_NY() && Set_Progress(y); y++)
+	for(int y=0; y<Get_NY() && Set_Progress_Rows(y); y++)
 	{
 		#pragma omp parallel for private(i)
 		for(int x=0; x<Get_NX(); x++)
@@ -687,7 +687,7 @@ bool CPanSharp_PCA::On_Execute(void)
 
 	PCA[0].Create(Get_System());
 
-	for(int y=0; y<Get_NY() && Set_Progress(y); y++)
+	for(int y=0; y<Get_NY() && Set_Progress_Rows(y); y++)
 	{
 		#pragma omp parallel for
 		for(int x=0; x<Get_NX(); x++)
