@@ -70,7 +70,7 @@ enum EBand_Head
 //---------------------------------------------------------
 CSG_Table CSentinel_2_Scene_Import::Get_Info_Bands(void)
 {
-	CSG_Table	Info_Bands;
+	CSG_Table Info_Bands;
 
 	Info_Bands.Add_Field("ID"        , SG_DATATYPE_Int   );
 	Info_Bands.Add_Field("BAND"      , SG_DATATYPE_String);
@@ -81,32 +81,32 @@ CSG_Table CSentinel_2_Scene_Import::Get_Info_Bands(void)
 	Info_Bands.Add_Field("WAVE_MAX"  , SG_DATATYPE_Double);
 
 	#define ADD_INFO_BAND(band, name, res, wmin, wave, wmax) { CSG_Table_Record &Info = *Info_Bands.Add_Record();\
-		Info.Set_Value(INFO_FIELD_ID  , 1 + Info.Get_Index());\
+		Info.Set_Value(INFO_FIELD_ID  , 1 + (int)Info.Get_Index());\
 		Info.Set_Value(INFO_FIELD_BAND, band);\
 		Info.Set_Value(INFO_FIELD_NAME, CSG_String::Format("[%s] %s", SG_T(band), name));\
 		Info.Set_Value(INFO_FIELD_RES , res);\
-		Info.Set_Value(INFO_FIELD_WAVE_MIN, wave);\
+		Info.Set_Value(INFO_FIELD_WAVE_MIN, wmin);\
 		Info.Set_Value(INFO_FIELD_WAVE    , wave);\
-		Info.Set_Value(INFO_FIELD_WAVE_MAX, wave);\
+		Info.Set_Value(INFO_FIELD_WAVE_MAX, wmax);\
 	}
 
-	ADD_INFO_BAND("B01", _TL("Aerosols"                 ), 60,  412,  442.7,  456);	//  1, 60m
-	ADD_INFO_BAND("B02", _TL("Blue"                     ), 10,  456,  492.7,  533);	//  2, 10m
-	ADD_INFO_BAND("B03", _TL("Green"                    ), 10,  538,  559.8,  583);	//  3, 10m
-	ADD_INFO_BAND("B04", _TL("Red"                      ), 10,  646,  664.6,  684);	//  4, 10m
-	ADD_INFO_BAND("B05", _TL("Red Edge"                 ), 20,  695,  704.1,  714);	//  5, 20m
-	ADD_INFO_BAND("B06", _TL("Red Edge"                 ), 20,  731,  740.5,  749);	//  6, 20m
-	ADD_INFO_BAND("B07", _TL("Red Edge"                 ), 20,  769,  782.8,  797);	//  7, 20m
-	ADD_INFO_BAND("B08", _TL("NIR"                      ), 10,  760,  832.8,  907);	//  8, 10m
-	ADD_INFO_BAND("B8A", _TL("NIR"                      ), 20,  837,  864.7,  881);	//  9, 20m
-	ADD_INFO_BAND("B09", _TL("Water Vapour"             ), 60,  932,  945.1,  958);	// 10, 60m
-	ADD_INFO_BAND("B10", _TL("Cirrus"                   ), 60, 1337, 1373.5, 1412);	// 11, 60m
-	ADD_INFO_BAND("B11", _TL("SWIR"                     ), 20, 1539, 1613.7, 1682);	// 12, 20m
-	ADD_INFO_BAND("B12", _TL("SWIR"                     ), 20, 2078, 2202.4, 2320);	// 13, 20m
-	ADD_INFO_BAND("TCI", _TL("True Color Image"         ), 10,    0,    0.0,    0);	// 14, 10m
-	ADD_INFO_BAND("AOT", _TL("Aerosol Optical Thickness"), 10,    0,    0.0,    0);	// 15, 10m
-	ADD_INFO_BAND("WVP", _TL("Water Vapour"             ), 10,    0,    0.0,    0);	// 16, 10m
-	ADD_INFO_BAND("SCL", _TL("Scene Classification"     ), 20,    0,    0.0,    0);	// 17, 20m
+	ADD_INFO_BAND("B01", _TL("Aerosols"                 ), 60,  412.0,  442.7,  456.0);	//  1, 60m
+	ADD_INFO_BAND("B02", _TL("Blue"                     ), 10,  456.0,  492.7,  533.0);	//  2, 10m
+	ADD_INFO_BAND("B03", _TL("Green"                    ), 10,  538.0,  559.8,  583.0);	//  3, 10m
+	ADD_INFO_BAND("B04", _TL("Red"                      ), 10,  646.0,  664.6,  684.0);	//  4, 10m
+	ADD_INFO_BAND("B05", _TL("Red Edge"                 ), 20,  695.0,  704.1,  714.0);	//  5, 20m
+	ADD_INFO_BAND("B06", _TL("Red Edge"                 ), 20,  731.0,  740.5,  749.0);	//  6, 20m
+	ADD_INFO_BAND("B07", _TL("Red Edge"                 ), 20,  769.0,  782.8,  797.0);	//  7, 20m
+	ADD_INFO_BAND("B08", _TL("NIR"                      ), 10,  760.0,  832.8,  907.0);	//  8, 10m
+	ADD_INFO_BAND("B8A", _TL("NIR"                      ), 20,  837.0,  864.7,  881.0);	//  9, 20m
+	ADD_INFO_BAND("B09", _TL("Water Vapour"             ), 60,  932.0,  945.1,  958.0);	// 10, 60m
+	ADD_INFO_BAND("B10", _TL("Cirrus"                   ), 60, 1337.0, 1373.5, 1412.0);	// 11, 60m
+	ADD_INFO_BAND("B11", _TL("SWIR"                     ), 20, 1539.0, 1613.7, 1682.0);	// 12, 20m
+	ADD_INFO_BAND("B12", _TL("SWIR"                     ), 20, 2078.0, 2202.4, 2320.0);	// 13, 20m
+	ADD_INFO_BAND("TCI", _TL("True Color Image"         ), 10,    0.0,    0.0,    0.0);	// 14, 10m
+	ADD_INFO_BAND("AOT", _TL("Aerosol Optical Thickness"), 10,    0.0,    0.0,    0.0);	// 15, 10m
+	ADD_INFO_BAND("WVP", _TL("Water Vapour"             ), 10,    0.0,    0.0,    0.0);	// 16, 10m
+	ADD_INFO_BAND("SCL", _TL("Scene Classification"     ), 20,    0.0,    0.0,    0.0);	// 17, 20m
 
 	return( Info_Bands );
 }
@@ -171,9 +171,9 @@ CSentinel_2_Scene_Import::CSentinel_2_Scene_Import(void)
 
 	Parameters.Add_Bool("", "LOAD_60M", _TL("Aerosol, Vapour, Cirrus"  ), _TL(""), false);
 //	Parameters.Add_Bool("", "LOAD_TCI", _TL("True Color Image"         ), _TL(""), false);
-//	Parameters.Add_Bool("", "LOAD_AOT", _TL("Aerosol Optical Thickness"), _TL(""), false);
-//	Parameters.Add_Bool("", "LOAD_WVP", _TL("Water Vapour"             ), _TL(""), false);
-	Parameters.Add_Bool("", "LOAD_SCL", _TL("Scene Classification"     ), _TL(""),  true);
+	Parameters.Add_Bool("", "LOAD_AOT", _TL("Aerosol Optical Thickness"), _TL(""), false);
+	Parameters.Add_Bool("", "LOAD_WVP", _TL("Water Vapour"             ), _TL(""), false);
+	Parameters.Add_Bool("", "LOAD_SCL", _TL("Scene Classification"     ), _TL(""), false);
 
 	Parameters.Add_Choice("",
 		"REFLECTANCE"	, _TL("Reflectance Values"),
@@ -184,6 +184,14 @@ CSentinel_2_Scene_Import::CSentinel_2_Scene_Import(void)
 		), 1
 	);
 
+	Parameters.Add_Choice("",
+		"RESOLUTION"	, _TL("Resolution"),
+		_TL("Allows to resample all spectral bands to the same target resolution, either 10 or 20m."),
+		CSG_String::Format("%s|%s|%s",
+			_TL("original"), SG_T("10m"), SG_T("20m")
+		), 0
+	);
+
 	Parameters.Add_Choice("",
 		"PROJECTION"	, _TL("Coordinate System"),
 		_TL(""),
@@ -276,6 +284,7 @@ int CSentinel_2_Scene_Import::On_Parameters_Enable(CSG_Parameters *pParameters,
 		pParameters->Set_Enabled("RESAMPLING", pParameter->asInt() >= 2);
 		pParameters->Set_Enabled("UTM_ZONE"  , pParameter->asInt() == 3);
 		pParameters->Set_Enabled("UTM_SOUTH" , pParameter->asInt() == 3);
+		pParameters->Set_Enabled("RESOLUTION", pParameter->asInt() <= 1);
 	}
 
 	if(	pParameter->Cmp_Identifier("EXTENT") )
@@ -300,7 +309,7 @@ int CSentinel_2_Scene_Import::On_Parameters_Enable(CSG_Parameters *pParameters,
 //---------------------------------------------------------
 bool CSentinel_2_Scene_Import::On_Execute(void)
 {
-	CSG_MetaData	Info_General, Info_Image, Info_Granule;
+	CSG_MetaData Info_General, Info_Image, Info_Granule;
 
 	if( !Load_Metadata(Parameters("METAFILE")->asString(), Info_General, Info_Granule, Info_Image) )
 	{
@@ -309,20 +318,24 @@ bool CSentinel_2_Scene_Import::On_Execute(void)
 		return( false );
 	}
 
-	CSG_String	Date	= Info_General["PRODUCT_START_TIME"].Get_Content().BeforeFirst('T');
+	CSG_String Date = Info_General["PRODUCT_START_TIME"].Get_Content().BeforeFirst('T');
 
 	//-----------------------------------------------------
-	CSG_String	Path	= SG_File_Get_Path(Parameters("METAFILE")->asString());
+	CSG_String Path = SG_File_Get_Path(Parameters("METAFILE")->asString());
 
-	bool	bLoadTCI	= Parameters("LOAD_TCI") && Parameters("LOAD_TCI")->asBool();
-	bool	bLoadAOT	= Parameters("LOAD_AOT") && Parameters("LOAD_AOT")->asBool();
-	bool	bLoadWVP	= Parameters("LOAD_WVP") && Parameters("LOAD_WVP")->asBool();
-	bool	bLoadSCL	= Parameters("LOAD_SCL") && Parameters("LOAD_SCL")->asBool();
-	bool	bLoad60m	= Parameters("LOAD_60M") && Parameters("LOAD_60M")->asBool();
+	bool bLevel2  = SG_File_Get_Name(Path, false).Find("MTD_MSIL2") == 0;
 
-	bool	bMultiGrids	= Parameters("MULTI2GRIDS")->asBool();
+	bool bLoadTCI = Parameters("LOAD_TCI") && Parameters("LOAD_TCI")->asBool();
+	bool bLoadAOT = Parameters("LOAD_AOT") && Parameters("LOAD_AOT")->asBool() && bLevel2;
+	bool bLoadWVP = Parameters("LOAD_WVP") && Parameters("LOAD_WVP")->asBool() && bLevel2;
+	bool bLoadSCL = Parameters("LOAD_SCL") && Parameters("LOAD_SCL")->asBool() && bLevel2;
+	bool bLoad60m = Parameters("LOAD_60M") && Parameters("LOAD_60M")->asBool();
 
-	CSG_Table	Info_Bands(Get_Info_Bands());
+	int Resolution = Parameters("PROJECTION")->asInt() <= 1 ? Parameters("RESOLUTION")->asInt() : 0;
+
+	bool bMultiGrids = Parameters("MULTI2GRIDS")->asBool();
+
+	CSG_Table Info_Bands(Get_Info_Bands());
 
 	//-----------------------------------------------------
 	Parameters("BANDS")->asGridList()->Del_Items();
@@ -340,19 +353,33 @@ bool CSentinel_2_Scene_Import::On_Execute(void)
 			continue;
 		}
 
-		CSG_Grid	*pBand	= Load_Band(Path, Find_Band(Info_Bands[Band], Info_Granule));
+		CSG_Grid *pBand = Load_Band(Path, Find_Band(Info_Bands[Band], Info_Granule));
 
 		if( !pBand )
 		{
 			continue;
 		}
 
+		if( Resolution && (BAND_IS_10m(Band) || BAND_IS_20m(Band)) )
+		{
+			Parameters("BANDS")->asGridList()->Update_Data(); CSG_Grid *pGrid = Parameters("BANDS")->asGridList()->Get_Grid(0);
+
+			CSG_Grid_System System; if( pGrid ) System = pGrid->Get_System(); else System.Create(Resolution == 1 ? 10. : 20., pBand->Get_Extent());
+
+			if( !System.is_Equal(pBand->Get_System()) )
+			{
+				SG_UI_Msg_Lock(true);
+				pGrid = pBand; pBand = SG_Create_Grid(System, pBand->Get_Type()); pBand->Assign(pGrid); delete(pGrid);
+				SG_UI_Msg_Lock(false);
+			}
+		}
+
 		pBand->Get_MetaData().Add_Child(Info_General)->Set_Name("SENTINEL-2");
 		pBand->Set_Description(Info_General.asText());
 
 		if( bMultiGrids && (BAND_IS_10m(Band) || BAND_IS_20m(Band)) )
 		{
-			int	b	= BAND_IS_10m(Band) ? 0 : 1;
+			int b = pBand->Get_Cellsize() == 10. ? 0 : 1;
 
 			if( pBands[b] == NULL )
 			{
@@ -411,7 +438,7 @@ bool CSentinel_2_Scene_Import::On_Execute(void)
 		Load_Classification(pSCL, Parameters("METAFILE")->asString());
 	}
 
-	return( true );
+	return( Parameters("BANDS")->asGridList()->Get_Grid_Count() > 0 );
 }
 
 
@@ -422,7 +449,7 @@ bool CSentinel_2_Scene_Import::On_Execute(void)
 //---------------------------------------------------------
 bool CSentinel_2_Scene_Import::Load_Metadata(const CSG_String &File, CSG_MetaData &General, CSG_MetaData &Granule, CSG_MetaData &Image)
 {
-	CSG_MetaData	Metadata;
+	CSG_MetaData Metadata;
 
 	if( !Metadata.Load(File)
 	||  !Metadata("n1:General_Info")
@@ -434,8 +461,8 @@ bool CSentinel_2_Scene_Import::Load_Metadata(const CSG_String &File, CSG_MetaDat
 		return( false );
 	}
 
-	Granule	= Metadata["n1:General_Info"]["Product_Info"]["Product_Organisation"]["Granule_List"]["Granule"];
-	General	= Metadata["n1:General_Info"]["Product_Info"];
+	Granule = Metadata["n1:General_Info"]["Product_Info"]["Product_Organisation"]["Granule_List"]["Granule"];
+	General = Metadata["n1:General_Info"]["Product_Info"];
 
 	General.Del_Child("Product_Organisation");
 
@@ -444,6 +471,23 @@ bool CSentinel_2_Scene_Import::Load_Metadata(const CSG_String &File, CSG_MetaDat
 		Image = Metadata["n1:General_Info"]["Product_Image_Characteristics"];
 	}
 
+	CSG_String _File(Granule.Get_Content(0)); _File = SG_File_Get_Path(File) + "/GRANULE/" + _File.AfterFirst('/').BeforeFirst('/') + "/MTD_TL.xml";
+
+	if( Metadata.Load(_File)
+	&&  Metadata("n1:Geometric_Info")
+	&&  Metadata["n1:Geometric_Info"]("Tile_Angles")
+	&&  Metadata["n1:Geometric_Info"]["Tile_Angles"]("Mean_Sun_Angle") )
+	{
+		double Azimuth, Zenith;
+
+		if( Metadata["n1:Geometric_Info"]["Tile_Angles"]["Mean_Sun_Angle"].Get_Content("AZIMUTH_ANGLE", Azimuth)
+		&&  Metadata["n1:Geometric_Info"]["Tile_Angles"]["Mean_Sun_Angle"].Get_Content( "ZENITH_ANGLE", Zenith ) )
+		{
+			General.Add_Child("SUN_AZIMUTH",      Azimuth);
+			General.Add_Child("SUN_HEIGHT" , 90. - Zenith);
+		}
+	}
+
 	return( true );
 }
 
@@ -499,22 +543,29 @@ CSG_String CSentinel_2_Scene_Import::Find_Band(const CSG_Table_Record &Band, con
 //---------------------------------------------------------
 CSG_Grid * CSentinel_2_Scene_Import::Load_Band(const CSG_String &Path, const CSG_String &File)
 {
+	if( File.is_Empty() )
+	{
+		return( NULL ); // metadata provides no file name for requested band!
+	}
+
 	Process_Set_Text("%s: %s", _TL("loading"), File.AfterLast('/').c_str());
 
 	//-----------------------------------------------------
-	CSG_String	_File(Path + "/" + File + ".jp2");
+	CSG_String _File(Path + "/" + File + ".jp2");
 
 	#ifdef _SAGA_MSW
 	_File.Replace("/", "\\");
 	#endif
 
 	SG_UI_Msg_Lock(true);
-	CSG_Grid	*pBand	= Load_Grid(_File);
+	CSG_Grid *pBand = Load_Grid(_File);
 	SG_UI_Msg_Lock(false);
 
 	if( !pBand )
 	{
-		Message_Fmt("\n%s [%s]", _TL("loading failed"), File.c_str());
+		CSG_String Message(CSG_String::Format("%s: \"%s\"", _TL("failed to load band"), File.c_str()));
+
+		Message_Add("\n" + Message, false); SG_UI_Msg_Add_Error(Message);
 
 		return( NULL );
 	}
@@ -529,9 +580,7 @@ CSG_Grid * CSentinel_2_Scene_Import::Load_Band(const CSG_String &Path, const CSG
 	//-----------------------------------------------------
 	else if( Parameters("PROJECTION")->asInt() <= 1 ) // UTM North or South
 	{
-		CSG_Grid	*pTmp	= pBand;
-
-		CSG_String	Projection	= pTmp->Get_Projection().Get_Proj4();
+		CSG_Grid *pTmp = pBand; CSG_String Projection = pTmp->Get_Projection().Get_Proj4();
 
 		if( Projection.Find("+proj=utm") >= 0
 		&&  (  (Projection.Find("+south") >= 0 && Parameters("PROJECTION")->asInt() == 0)
@@ -553,12 +602,9 @@ CSG_Grid * CSentinel_2_Scene_Import::Load_Band(const CSG_String &Path, const CSG
 			pBand->Set_Scaling           (pTmp->Get_Scaling(), pTmp->Get_Offset());
 
 			#pragma omp parallel for
-			for(int y=0; y<pBand->Get_NY(); y++)
+			for(int y=0; y<pBand->Get_NY(); y++) for(int x=0; x<pBand->Get_NX(); x++)
 			{
-				for(int x=0; x<pBand->Get_NX(); x++)
-				{
-					pBand->Set_Value(x, y, pTmp->asDouble(x, y));
-				}
+				pBand->Set_Value(x, y, pTmp->asDouble(x, y));
 			}
 
 			delete(pTmp);
@@ -568,7 +614,7 @@ CSG_Grid * CSentinel_2_Scene_Import::Load_Band(const CSG_String &Path, const CSG
 	//-----------------------------------------------------
 	else if( Parameters("PROJECTION")->asInt() == 2 )	// Geographic Coordinates
 	{
-		CSG_Tool	*pTool	= SG_Get_Tool_Library_Manager().Create_Tool("pj_proj4", 4);	// Coordinate Transformation (Grid)
+		CSG_Tool *pTool = SG_Get_Tool_Library_Manager().Create_Tool("pj_proj4", 4);	// Coordinate Transformation (Grid)
 
 		if(	pTool )
 		{
@@ -584,7 +630,7 @@ CSG_Grid * CSentinel_2_Scene_Import::Load_Band(const CSG_String &Path, const CSG
 			{
 				delete(pBand);
 
-				pBand	= pTool->Get_Parameters()->Get_Parameter("GRID")->asGrid();
+				pBand = pTool->Get_Parameters()->Get_Parameter("GRID")->asGrid();
 			}
 
 			SG_Get_Tool_Library_Manager().Delete_Tool(pTool);
@@ -601,7 +647,7 @@ CSG_Grid * CSentinel_2_Scene_Import::Load_Band(const CSG_String &Path, const CSG
 
 		if( !Projection.is_Equal(pBand->Get_Projection()) )
 		{
-			CSG_Tool	*pTool	= SG_Get_Tool_Library_Manager().Create_Tool("pj_proj4", 4);	// Coordinate Transformation (Grid)
+			CSG_Tool *pTool = SG_Get_Tool_Library_Manager().Create_Tool("pj_proj4", 4);	// Coordinate Transformation (Grid)
 
 			if(	pTool )
 			{
@@ -618,7 +664,7 @@ CSG_Grid * CSentinel_2_Scene_Import::Load_Band(const CSG_String &Path, const CSG
 				{
 					delete(pBand);
 
-					pBand	= pTool->Get_Parameters()->Get_Parameter("GRID")->asGrid();
+					pBand = pTool->Get_Parameters()->Get_Parameter("GRID")->asGrid();
 				}
 
 				SG_Get_Tool_Library_Manager().Delete_Tool(pTool);
@@ -661,19 +707,19 @@ CSG_Grid * CSentinel_2_Scene_Import::Load_Grid(const CSG_String &File)
 	}
 
 	//-----------------------------------------------------
-	CSG_Grid	*pGrid	= NULL;
-	CSG_Tool	*pTool	= SG_Get_Tool_Library_Manager().Create_Tool("io_gdal", 0);	// Import Raster
+	CSG_Grid *pGrid = NULL;
+	CSG_Tool *pTool = SG_Get_Tool_Library_Manager().Create_Tool("io_gdal", 0);	// Import Raster
 
 	if( pTool && pTool->Set_Manager(NULL)
-		&&  pTool->Set_Parameter("FILES"      , File)
-		&&	pTool->Set_Parameter("EXTENT"     , 1)
-		&&	pTool->Set_Parameter("EXTENT_XMIN", Extent.Get_XMin())
-		&&	pTool->Set_Parameter("EXTENT_XMAX", Extent.Get_XMax())
-		&&	pTool->Set_Parameter("EXTENT_YMIN", Extent.Get_YMin())
-		&&	pTool->Set_Parameter("EXTENT_YMAX", Extent.Get_YMax())
-		&&  pTool->Execute() )
+	&&  pTool->Set_Parameter("FILES"      , File)
+	&&	pTool->Set_Parameter("EXTENT"     , 1)
+	&&	pTool->Set_Parameter("EXTENT_XMIN", Extent.Get_XMin())
+	&&	pTool->Set_Parameter("EXTENT_XMAX", Extent.Get_XMax())
+	&&	pTool->Set_Parameter("EXTENT_YMIN", Extent.Get_YMin())
+	&&	pTool->Set_Parameter("EXTENT_YMAX", Extent.Get_YMax())
+	&&  pTool->Execute() )
 	{
-		pGrid	= pTool->Get_Parameter("GRIDS")->asGridList()->Get_Grid(0);
+		pGrid = pTool->Get_Parameter("GRIDS")->asGridList()->Get_Grid(0);
 	}
 
 	SG_Get_Tool_Library_Manager().Delete_Tool(pTool);
@@ -689,7 +735,7 @@ CSG_Grid * CSentinel_2_Scene_Import::Load_Grid(const CSG_String &File)
 //---------------------------------------------------------
 bool CSentinel_2_Scene_Import::Load_Classification(CSG_Grid *pGrid, const CSG_String &File)
 {
-	CSG_MetaData	Metadata;
+	CSG_MetaData Metadata;
 
 	if( !Metadata.Load(File)
 	||  !Metadata("n1:General_Info")
