@@ -164,9 +164,7 @@ int CCRS_Transform_Grid::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_P
 	if( pParameter->Cmp_Identifier("CRS_METHOD"     )
 	||  pParameter->Cmp_Identifier("CRS_PROJ4"      )
 	||  pParameter->Cmp_Identifier("CRS_DIALOG"     )
-	||  pParameter->Cmp_Identifier("CRS_GRID"       )
-	||  pParameter->Cmp_Identifier("CRS_SHAPES"     )
-	||  pParameter->Cmp_Identifier("CRS_FILE"       )
+	||  pParameter->Cmp_Identifier("CRS_PICKER"     )
 	||  pParameter->Cmp_Identifier("CRS_EPSG"       )
 	||  pParameter->Cmp_Identifier("CRS_EPSG_AUTH"  )
 	||  pParameter->Cmp_Identifier("CRS_EPSG_GEOGCS")
@@ -336,11 +334,17 @@ bool CCRS_Transform_Grid::Transform(CSG_Grid *pGrid, CSG_Grid *pTarget)
 	bool	bGeogCS_Adjust	= m_Projector.Get_Source().Get_Type() == SG_PROJ_TYPE_CS_Geographic && pGrid->Get_XMax() > 180.;
 
 	//-------------------------------------------------
-	pTarget->Set_NoData_Value_Range (pGrid->Get_NoData_Value(), pGrid->Get_NoData_Value(true));
-	pTarget->Set_Scaling            (pGrid->Get_Scaling(), pGrid->Get_Offset());
-	pTarget->Set_Name               (pGrid->Get_Name());
-	pTarget->Set_Unit               (pGrid->Get_Unit());
 	pTarget->Get_Projection().Create(m_Projector.Get_Target());
+
+	pTarget->Set_Scaling(pGrid->Get_Scaling(), pGrid->Get_Offset());
+	pTarget->Set_Name   (pGrid->Get_Name());
+	pTarget->Set_Unit   (pGrid->Get_Unit());
+
+	if( pTarget->Get_Type() == pGrid->Get_Type() )
+	{
+		pTarget->Set_NoData_Value_Range(pGrid->Get_NoData_Value(), pGrid->Get_NoData_Value(true));
+	}
+
 	pTarget->Assign_NoData();
 
 	//-----------------------------------------------------
@@ -902,8 +906,8 @@ bool CCRS_Transform_Grid::Set_Target_Area(const CSG_Grid_System &Source, const C
 	if( m_Projector.Get_Source().Get_Type() == SG_PROJ_TYPE_CS_Geographic )
 	{
 		if( r.Get_XMax() > 180. )	r.Move(-180., 0.);
-		if( r.Get_YMin() < -90. )	r.m_rect.yMin	= -90.;
-		if( r.Get_YMax() <  90. )	r.m_rect.yMax	=  90.;
+		if( r.Get_YMin() < -90. )	r.yMin	= -90.;
+		if( r.Get_YMax() <  90. )	r.yMax	=  90.;
 	}
 
 	//-----------------------------------------------------
