@@ -159,7 +159,7 @@ bool CGPP_Model::On_Execute(void)
 
 	m_mReleaseAreas.clear();
 
-	for (int y=0; y<Get_NY() && Set_Progress(y); y++)
+	for (int y=0; y<Get_NY() && Set_Progress_Rows(y); y++)
 	{
 		for (int x=0; x<Get_NX(); x++)
 		{
