@@ -128,7 +128,7 @@ bool CVisibility::Finalize(bool bShow)
 {
 	CSG_Parameters Parameters;
 
-	int Update = bShow ? SG_UI_DATAOBJECT_SHOW_LAST_MAP : SG_UI_DATAOBJECT_UPDATE_ONLY;
+	int Update = bShow ? SG_UI_DATAOBJECT_SHOW_MAP_ACTIVE : SG_UI_DATAOBJECT_UPDATE;
 
 	switch( m_Method )
 	{
@@ -430,13 +430,13 @@ bool CVisibility_Points::On_Execute(void)
 	double Height = Parameters("HEIGHT")->asDouble();
 
 	//-----------------------------------------------------
-	for(int iPoint=0; iPoint<pPoints->Get_Count() && Process_Get_Okay(); iPoint++)
+	for(sLong iPoint=0; iPoint<pPoints->Get_Count() && Process_Get_Okay(); iPoint++)
 	{
-		Process_Set_Text("%s %d...", _TL("processing observer"), 1 + iPoint);
+		Process_Set_Text("%s %lld...", _TL("processing observer"), 1 + iPoint);
 
 		CSG_Shape &Point = *pPoints->Get_Shape(iPoint);
 
-		int x, y; Get_System().Get_World_to_Grid(x, y, Point.Get_Point(0));
+		int x, y; Get_System().Get_World_to_Grid(x, y, Point.Get_Point());
 
 		Set_Visibility(x, y, Field < 0 ? Height : Point.asDouble(Field), false);
 	}
